/*    $ DOOROOS: drv_display.c v 0.1 2013/08/011 $    */

/*
 * Copyright (C) 2013    iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: drv_display.c
 *
 * Purpose: display device driver.
 *          
 *
 */

#include <types.h>
#include <Window.h>

#include <dev/gd.h>

#include <lib/string.h>
#include "drv_display.h"

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>

#include <hal_panic.h>

#include <dev/displaydrv.h>

#include <cache.h>

#define DBG_LVL_LCD     1

#include "sil9022a.h"

#ifdef _WITH_HDMI_
        static video_mode_param hdmi_video_mode;
#endif

//#define __USE_PIO_BL__
//#define _JBOOT_DISPLAY_INIT_

/*----------------------------------------------------------------------------
 *        Defines
 *----------------------------------------------------------------------------*/

/* \addtogroup lcdd_disp_id LCD display layers IDs */
/** LCD controller ID, no display, configuration ONLY */
#define LCDD_CONTROLLER     0
/** LCD base layer, display fixed size image */
#define LCDD_BASE           1
/** LCD Overlay 1 */
#define LCDD_OVR1           2
/** LCD Overlay 2 */
#define LCDD_OVR2           4
/** LCD HighEndOverlay, support resize */
#define LCDD_HEO            3
/** LCD Cursor, max size 128x128 */
#define LCDD_CUR            6

/*----------------------------------------------------------------------------
 *        Local types
 *----------------------------------------------------------------------------*/

/** DMA descriptor for LCDC */
typedef struct _LCDCDescriptor
{
    UINT32 addr;
    UINT32 ctrl;
    UINT32 next;
}sLCDCDescriptor;

/** CULT information */
typedef struct _CLUTInfo
{
    UINT8   bpp;
    UINT8   nbColors;
}sCLUTInfo;

/** LCDC General Layer information */
typedef struct _Layer
{
    sLCDCDescriptor dmaD;
    void*           pBuffer;
    sCLUTInfo       clut;
    UINT16          reserved;
} sLayer;

/** LCDC HEO Layer information */
typedef struct _HeoLayer
{
    sLCDCDescriptor dmaD[3];
    void*           pBuffer;
    sCLUTInfo       clut;
    UINT16          reserved;
} sHeoLayer;


#if 1
/** Current selected canvas information */
/** Base Layer */
sLayer *plcddBase;

/** Overlay 1 Layer */
sLayer *plcddOvr1;

sLayer *plcddOvr2;

/** High End Overlay Layer */
sHeoLayer *plcddHeo;

/** Hardware cursor Layer */
sLayer *plcddHcc;
#else
/** Current selected canvas information */
//static sLCDDLayer lcddCanvas;
/** Base Layer */
__attribute__((aligned(16)))
 sLayer lcddBase;

/** Overlay 1 Layer */
__attribute__((aligned(64)))
 sLayer lcddOvr1;

__attribute__((aligned(64)))
 sLayer lcddOvr2;

/** High End Overlay Layer */
__attribute__((aligned(64)))
 sHeoLayer lcddHeo;

/** Hardware cursor Layer */
__attribute__((aligned(64)))
 sLayer lcddHcc;
#endif


EXTERN UINT32 RotateAngle;


static UINT32 s_xcoff = -1;
static UINT32 s_xcoff90 = -1;
static UINT32 s_ycoff90 = -1;
void FillCursorImage( BYTE *pFrame, UINT32 xcoff, UINT32 ycoff )
{
if( RotateAngle == 90 )
{
    if( s_xcoff90 == xcoff && 
        s_ycoff90 == ycoff )
        return;

    s_xcoff90 = xcoff;
    s_ycoff90 = ycoff;

//DBGLINE(1,"xcoff, ycoff = (%d, %d)\n", xcoff, ycoff );
    int i;
    BYTE *p, *c;
    extern BYTE cursor90_Image[];
    //p = (BYTE*)Frame;
    c = cursor90_Image;
//    p += DMA_ADDR_OFFSET;
    UINT32 width  = /*xcoff? CURSOR_WIDTH-1:*/  CURSOR_WIDTH;
    UINT32 height = /*ycoff? CURSOR_HEIGHT-1:*/ CURSOR_HEIGHT;

//DBGLINE(1,"width, height = (%d, %d)\n", width, height );

    p = pFrame;
    c += CURSOR_WIDTH*3*ycoff;
    for( i = 0; i < height - ycoff; i++ )
    {
        int j;
        BYTE    *q;
        q = c;
        j = 0; 
        for( j = 0; j < xcoff; j++ )
        {
            p[0] = 0xFF;
            p[1] = 0;
            p[2] = 0xFF;
            p[3] = 0x00;
            p+=4;
        }
        
        for( ; j < width; j++ )
        {
            p[0] = *q++;
            p[1] = *q++;
            p[2] = *q++;
            p[3] = 0x00;
            p+=4;
        }
        c = c + CURSOR_WIDTH*3;
    }

    //
    for( ;i < height; i++ )
    {
        int j;
        for( j = 0; j < width; j++ )
        {
            p[0] = 0xFF;
            p[1] = 0;
            p[2] = 0xFF;
            p[3] = 0x00;
            p+=4;
        }
    }
    CP15_coherent_dcache_for_dma ((UINT32)pFrame, ((UINT32)pFrame) + 4*CURSOR_WIDTH*CURSOR_HEIGHT);
}
else
{
    if( s_xcoff == xcoff )
        return;

    s_xcoff = xcoff;

    int i;
    BYTE *p, *c;
    extern BYTE cursor_Image[];
    //p = (BYTE*)Frame;
    c = cursor_Image;
//    p += DMA_ADDR_OFFSET;
    UINT32 width = /*xcoff? CURSOR_WIDTH-1:*/ CURSOR_WIDTH;

    p = pFrame;
    for( i = 0; i < CURSOR_HEIGHT; i++ )
    {
        int j;
        BYTE    *q;
        q = c;
        j = 0; 
        for( j = 0; j < xcoff; j++ )
        {
            p[0] = 0xFF;
            p[1] = 0x00;
            p[2] = 0xFF;
            p[3] = 0x00;
            p+=4;
        }

        for( ; j < width; j++ )
        {
            p[0] = *q++;
            p[1] = *q++;
            p[2] = *q++;
            p[3] = 0x00;
            p+=4;
        }
        c = c + CURSOR_WIDTH*3;
    }
    CP15_coherent_dcache_for_dma ((UINT32)pFrame, ((UINT32)pFrame) + 4*CURSOR_WIDTH*CURSOR_HEIGHT);
}

}
/*----------------------------------------------------------------------------
 *        Local functions
 *----------------------------------------------------------------------------*/

/**
 * Return a pointer to layer.
 * \param bLayer Layer ID.
 */
static sLayer *pLayer( UINT8 bLayer )
{
    switch( bLayer )
    {
        case LCDD_BASE:       return plcddBase;
        case LCDD_OVR1:       return plcddOvr1;
        case LCDD_OVR2:       return plcddOvr2;
        case LCDD_HEO:        return (sLayer*)plcddHeo;
        case LCDD_CUR:        return plcddHcc;
    }
    return NULL;
}

/**
 * Return true if Pixel stride supported.
 * \param bLayer Layer ID.
 */
static UINT8 LCDD_IsPStrideSupported( UINT8 bLayer )
{
    switch( bLayer )
    {
        case LCDD_OVR1: case LCDD_OVR2:case LCDD_HEO: return 1;
        default:                       return 0;
    }
}

/**
 * Return a pointer to enable register.
 * (Starts following register list: _ER, _DR, _SR, _IER, _IDR, _IMR, _ISR)
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pEnableReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_CONTROLLER: return (volatile UINT32*)&pLCDC->LCDC_LCDEN;
        case LCDD_BASE:       return (volatile UINT32*)&pLCDC->LCDC_BASECHER;
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1CHER;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2CHER;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOCHER;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRCHER;
    }
    return NULL;
}

/**
 * Return a pointer to blender configuration register.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pBlenderReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32 *)&pLCDC->LCDC_BASECFG4;
        case LCDD_OVR1: return (volatile UINT32 *)&pLCDC->LCDC_OVR1CFG9;
        case LCDD_OVR2: return (volatile UINT32 *)&pLCDC->LCDC_OVR2CFG9;
        case LCDD_HEO:  return (volatile UINT32 *)&pLCDC->LCDC_HEOCFG12;
        case LCDD_CUR:  return (volatile UINT32 *)&pLCDC->LCDC_HCRCFG9;
    }
    return NULL;
}

/**
 * Return a pointer to DMA head register.
 * (Starts following register list: _HEAD, _ADDRESS, _CONTROL, _NEXT)
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pHeadReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE:       return (volatile UINT32*)&pLCDC->LCDC_BASEHEAD;
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1HEAD;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2HEAD;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOHEAD;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRHEAD;
    }
    return NULL;
}

/**
 * Return a pointer to layer configure register.
 * (Including: _CFG0, _CFG1 (RGB mode ...))
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pCfgReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE:       return (volatile UINT32*)&pLCDC->LCDC_BASECFG0;
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG0;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG0;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOCFG0;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRCFG0;
    }
    return NULL;
}

/**
 * Return a pointer to Window configure register.
 * (Including: X Y register, W H register)
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pWinReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG2;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG2;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOCFG2;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRCFG2;
    }
    return NULL;
}

/**
 * Return a pointer to striding regiters.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pStrideReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32*)&pLCDC->LCDC_BASECFG2;
        case LCDD_OVR1: return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG4;
        case LCDD_OVR2: return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG4;
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCFG5;
        case LCDD_CUR:  return (volatile UINT32*)&pLCDC->LCDC_HCRCFG4;
    }
    return NULL;
}

/**
 * Return a pointer to Color configure regiters.
 * (Including: RGB Default, RGB Key, RGB Mask)
 * Note that base layer only has one register (default).
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pColorReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32*)&pLCDC->LCDC_BASECFG3;
        case LCDD_OVR1: return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG6;
        case LCDD_OVR2: return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG6;
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCFG9;
        case LCDD_CUR:  return (volatile UINT32*)&pLCDC->LCDC_HCRCFG6;
    }
    return NULL;
}

/**
 * Return a pointer to scaling register.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pScaleReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCFG13;
    }
    return NULL;
}

#if 0
/**
 * Return bits per pixel from RGB mode settings.
 * (Note the bits is bits occupied in memory, including reserved)
 */
static UINT32 LCDD_GetBitsPerPixel(UINT32 modeReg)
{
    switch (modeReg)
    {
        /* RGB mode */
        case LCDC_HEOCFG1_RGBMODE_12BPP_RGB_444:
        case LCDC_HEOCFG1_RGBMODE_16BPP_ARGB_4444:
        case LCDC_HEOCFG1_RGBMODE_16BPP_RGBA_4444:
        case LCDC_HEOCFG1_RGBMODE_16BPP_RGB_565:
        case LCDC_HEOCFG1_RGBMODE_16BPP_TRGB_1555:
            return 2*8;

        case LCDC_HEOCFG1_RGBMODE_18BPP_RGB_666PACKED:
        case LCDC_HEOCFG1_RGBMODE_19BPP_TRGB_PACKED:
        case LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED:
            return 3*8;

        case LCDC_HEOCFG1_RGBMODE_18BPP_RGB_666:
        case LCDC_HEOCFG1_RGBMODE_19BPP_TRGB_1666:
        case LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888:
        case LCDC_HEOCFG1_RGBMODE_25BPP_TRGB_1888:
        case LCDC_HEOCFG1_RGBMODE_32BPP_ARGB_8888:
        case LCDC_HEOCFG1_RGBMODE_32BPP_RGBA_8888:
            return 3*8;

        /* CLUT mode */
 
        case LCDC_HEOCFG1_CLUTMODE_CLUT_1BPP | LCDC_HEOCFG1_CLUTEN: return 1;
        case LCDC_HEOCFG1_CLUTMODE_CLUT_2BPP | LCDC_HEOCFG1_CLUTEN: return 2;
        case LCDC_HEOCFG1_CLUTMODE_CLUT_4BPP | LCDC_HEOCFG1_CLUTEN: return 4;
        case LCDC_HEOCFG1_CLUTMODE_CLUT_8BPP | LCDC_HEOCFG1_CLUTEN: return 8;

        /* YUV mode */
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_32BPP_AYCBCR:
            return 32;
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE0:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE1:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE2:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE3:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_SEMIPLANAR:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_PLANAR:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_12BPP_YCBCR_SEMIPLANAR:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_12BPP_YCBCR_PLANAR:
            return 16;
    }
    return 0;
}
#endif

/**
 * Enable a LCDC DMA channel
 */
static void LCDD_SetDMA( void* pBuffer,
                         sLCDCDescriptor *pTD,
                         UINT32 regAddr)
{
    volatile UINT32 *pDmaR = (volatile UINT32*)regAddr;

    /* Modify descriptor */
    pTD->addr = (UINT32)pBuffer;
    pTD->ctrl = LCDC_BASECTRL_DFETCH;
    pTD->next = (UINT32)pTD + DMA_ADDR_OFFSET;

    /* Modify registers */
    pDmaR[0] = (UINT32)pTD + DMA_ADDR_OFFSET;
    pDmaR[1] = (UINT32)pBuffer;
    pDmaR[2] = LCDC_BASECTRL_DFETCH;
    pDmaR[3] = (UINT32)pTD + DMA_ADDR_OFFSET;

    //CP15_flush_dcache_for_dma ((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));
    //CacheRangeFlush(pTD, sizeof(sLCDCDescriptor), CACHE_SYNC_WRITEBACK);
    CP15_coherent_dcache_for_dma ((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));

    return;
}

/**
 * Disable a LCDC DMA channel
 */
static void LCDD_ClearDMA( sLCDCDescriptor *pTD, UINT32 regAddr )
{
    UINT32 *pReg = (UINT32 *)regAddr;
    volatile UINT32 *pRegCtrl = (volatile UINT32*)&pReg[1];
    volatile UINT32 *pRegNext = (volatile UINT32*)&pReg[2];

    /* Modify descriptor */
    if (pTD)
    {
        pTD->ctrl &= ~LCDC_BASECTRL_DFETCH;
        pTD->next  =  (UINT32)pTD + DMA_ADDR_OFFSET;

        CP15_coherent_dcache_for_dma ((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));
        //CacheRangeFlush(pTD, sizeof(sLCDCDescriptor), CACHE_SYNC_WRITEBACK);
    }
    /* Modify control registers */
    *pRegCtrl &= ~LCDC_BASECTRL_DFETCH;
    if( pTD == 0 )
        *pRegNext  =  (UINT32)pTD;
    else
        *pRegNext  =  (UINT32)pTD + DMA_ADDR_OFFSET;
}

/**
 * Return scaling factor
 */
static UINT32 LCDD_CalcScaleFactor(UINT32 targetW, UINT32 imgW)
{
    UINT32 factor;

    factor = 2048 * (imgW + 1) / (targetW + 1);
    
    //factor = 1024 * (imgW + 1) / (targetW + 1);
    //if (targetW > imgW * 2)
    //    factor -= 7;
    return factor;
}

/**
 * Return a pointer to Color Palette lookup regiters.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pCLUTReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32*)&pLCDC->LCDC_BASECLUT[0];
        case LCDD_OVR1: return (volatile UINT32*)&pLCDC->LCDC_OVR1CLUT[0];
        case LCDD_OVR2: return (volatile UINT32*)&pLCDC->LCDC_OVR2CLUT[0];
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCLUT[0];
        case LCDD_CUR:  return (volatile UINT32*)&pLCDC->LCDC_HCRCLUT[0];
    }
    return NULL;
}

/**
 * Build 8-bit color palette (actually true color)
 */
static void LCDD_BuildCLUT8(volatile UINT32* pCLUT)
{
    UINT32 r, g, b; /* 3:3:2 */
    for (r = 0; r < 8; r ++)
    {
        for (g = 0; g < 8; g ++)
        {
            for (b = 0; b < 4; b ++)
            {
                *pCLUT ++ =   (r << (16 + 5))
                            + (g << (8  + 5))
                            + (b << (0  + 6));
            }
        }
    }
}

/**
 * Build 4-bit color palette (16 color)
 */
static void LCDD_BuildCLUT4(volatile UINT32* pCLUT)
{
    UINT32 r, g, b;
    for (r = 0; r < 4; r ++)
    {
        for (g = 0; g < 2; g ++)
        {
            for (b = 0; b < 2; b ++)
            {
                *pCLUT ++ =  (r << (16 + 6))
                           + (g << (8  + 7))
                           + (b << (0  + 7));
            }
        }
    }
}

/**
 * Build 2-bit color palette (4 gray)
 */
static void LCDD_BuildCLUT2(volatile UINT32* pCLUT)
{
    pCLUT[0] = 0x000000;
    pCLUT[1] = 0x505050;
    pCLUT[2] = 0xA0A0A0;
    pCLUT[3] = 0xFFFFFF;
}

/**
 * Build 1-bit color palette (black & white)
 */
static void LCDD_BuildCLUT1(volatile UINT32* pCLUT)
{
    pCLUT[0] = 0x000000;
    pCLUT[1] = 0xFFFFFF;
}




LCDC_DESC    LDCD_Desc;

//#define DEFAULT_BRIGHT_LEVEL     0x10
#define DEFAULT_BRIGHT_LEVEL     0xFF


#define LCDC_HCLK_ENABLE    (1 << 3)
#define LCDC_CORECLK_ENABLE (1 << 12)
#define LCDC_PCKR_ID        4

PUBLIC VOID GD_Contrast( UINT32 nContrast );
//PUBLIC VOID GD_BackLight( UINT32 nBaclight );


/**
 * \brief Turn on the LCD.
 */
void LCDD_On(void)
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;

    /* Enable peripheral clock */
    pPMC->PMC_PCER1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    pPMC->PMC_SCER = (0x1u << 3);


    
    /* 1. Configure LCD timing parameters, signal polarity and clock period. */
    pLCDC->LCDC_LCDCFG0 = LCDC_LCDCFG0_CLKDIV((BOARD_MCK*2)/BOARD_LCD_PIXELCLOCK-2)
                       |LCDC_LCDCFG0_CGDISHCR
                       |LCDC_LCDCFG0_CGDISHEO
                       |LCDC_LCDCFG0_CGDISOVR1
                       |LCDC_LCDCFG0_CGDISOVR2
                       |LCDC_LCDCFG0_CGDISBASE
                       //|LCDC_LCDCFG0_CLKPWMSEL // to slow clock selection
                       |LCDC_LCDCFG0_CLKSEL
                       |0;//LCDC_LCDCFG0_CLKPOL;

    pLCDC->LCDC_LCDCFG1 = LCDC_LCDCFG1_VSPW(BOARD_LCD_TIMING_VPW-1)
                       |LCDC_LCDCFG1_HSPW(BOARD_LCD_TIMING_HPW-1);

    pLCDC->LCDC_LCDCFG2 = LCDC_LCDCFG2_VBPW(BOARD_LCD_TIMING_VBP)
                       |LCDC_LCDCFG2_VFPW(BOARD_LCD_TIMING_VFP-1);

    pLCDC->LCDC_LCDCFG3 = LCDC_LCDCFG3_HBPW(BOARD_LCD_TIMING_HBP-1)
                       |LCDC_LCDCFG3_HFPW(BOARD_LCD_TIMING_HFP-1);

    pLCDC->LCDC_LCDCFG4 = LCDC_LCDCFG4_RPF(DISPLAY_HEIGHT-1)
                       |LCDC_LCDCFG4_PPL(DISPLAY_WIDTH-1);

    pLCDC->LCDC_LCDCFG5 = LCDC_LCDCFG5_GUARDTIME(30)
                       |LCDC_LCDCFG5_MODE_OUTPUT_24BPP
                       |LCDC_LCDCFG5_DISPDLY
                       |LCDC_LCDCFG5_VSPDLYS
                       |LCDC_LCDCFG5_VSPOL
                       |LCDC_LCDCFG5_HSPOL;

    pLCDC->LCDC_LCDCFG6 = LCDC_LCDCFG6_PWMCVAL(DEFAULT_BRIGHT_LEVEL)//0)
                       //|LCDC_LCDCFG6_PWMPOL
                       |LCDC_LCDCFG6_PWMPS(0);//(6);

    /* 2. Enable the Pixel Clock by writing one to the CLKEN field of the
          LCDC_LCDEN register. */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_CLKEN;
    /* 3. Poll CLKSTS field of the LCDC_LCDSR register to check that the clock
       is running. */
    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_CLKSTS));

    /* 4. Enable Horizontal and Vertical Synchronization by writing one to the
          SYNCEN field of the LCDC_LCDEN register. */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_SYNCEN;
    /* 5. Poll LCDSTS field of the LCDC_LCDSR register to check that the
          synchronization is up. */
    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_LCDSTS));

    /* 6. Enable the display power signal writing one to the DISPEN field of the
          LCDC_LCDEN register. */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_DISPEN;
    /* 7. Poll DISPSTS field of the LCDC_LCDSR register to check that the power
          signal is activated. */
    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS));
    /* 8. Enable backlight */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_PWMEN;

    pio_set_value(AT91C_PIN_PA(24), 1 ); 
#ifdef __USE_PIO_BL__
    pio_set_value(AT91C_PIN_PA(25), 1 ); 
#endif
    

}

/**
 * \brief Turn off the LCD.
 */
void LCDD_Off(void)
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;


    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */

    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */

    /* Disable all DMA channel descriptors */
    LCDD_ClearDMA(&plcddBase->dmaD, (UINT32)&pLCDC->LCDC_BASEADDR);
    LCDD_ClearDMA(&plcddOvr1->dmaD, (UINT32)&pLCDC->LCDC_OVR1ADDR);
    LCDD_ClearDMA(&plcddOvr2->dmaD, (UINT32)&pLCDC->LCDC_OVR2ADDR);
    LCDD_ClearDMA(&plcddHeo->dmaD[0], (UINT32)&pLCDC->LCDC_HEOADDR);
    LCDD_ClearDMA(&plcddHeo->dmaD[1], (UINT32)&pLCDC->LCDC_HEOUADDR);
    LCDD_ClearDMA(&plcddHeo->dmaD[2], (UINT32)&pLCDC->LCDC_HEOVADDR);
    LCDD_ClearDMA(&plcddHcc->dmaD, (UINT32)&pLCDC->LCDC_HCRADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */

    /* Disable DMA channels */
    pLCDC->LCDC_BASECHDR = LCDC_BASECHDR_CHDIS;
    pLCDC->LCDC_OVR1CHDR = LCDC_OVR1CHDR_CHDIS;
    pLCDC->LCDC_HEOCHDR  = LCDC_HEOCHDR_CHDIS;
    pLCDC->LCDC_HCRCHDR  = LCDC_HCRCHDR_CHDIS;
    pLCDC->LCDC_BASECFG4 = 0;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_BASECHSR & LCDC_BASECHSR_CHSR);
    while (pLCDC->LCDC_OVR1CHSR & LCDC_OVR1CHSR_CHSR);
    while (pLCDC->LCDC_HEOCHSR  & LCDC_HEOCHSR_CHSR);
    while (pLCDC->LCDC_HCRCHDR  & LCDC_HCRCHSR_CHSR);

    
    /* Timing Engine Power Down Software Operation */

    /* Disable backlight */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_PWMDIS;
    while (pLCDC->LCDC_LCDSR & LCDC_LCDSR_PWMSTS);

    /* 1. Disable the DISP signal writing DISPDIS field of the LCDC_LCDDIS
          register. */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_DISPDIS;
    /* 2. Poll DISPSTS field of the LCDC_LCDSR register to verify that the DISP
          is no longer activated. */
    while (pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS);
    
    /* 3. Disable the hsync and vsync signals by writing one to SYNCDIS field of
          the LCDC_LCDDIS register. */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_SYNCDIS;
    /* 4. Poll LCDSTS field of the LCDC_LCDSR register to check that the
          synchronization is off. */
    while (pLCDC->LCDC_LCDSR & LCDC_LCDSR_LCDSTS);
    
    /* 5. Disable the Pixel clock by writing one in the CLKDIS field of the
          LCDC_LCDDIS register. */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_CLKDIS;
    /* 6. Poll CLKSTS field of the LCDC_LCDSR register to check that Pixel Clock
          is disabled. */
    while(pLCDC->LCDC_LCDSR & LCDC_LCDSR_CLKSTS);

    /* Disable peripheral clock */
    pPMC->PMC_PCDR1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    /* LCD Clock Disable */
     pPMC->PMC_SCDR = (0x1u << 3);

    pio_set_value(AT91C_PIN_PA(24), 0); 
#ifdef __USE_PIO_BL__
    pio_set_value(AT91C_PIN_PA(25), 0); 
#endif     

}


#ifndef _JBOOT_DISPLAY_INIT_
// Internal function, get pointers to registers & set PIO/Peripheral configuration
STATIC VOID
IOConfig( VOID )
{
//    #define LCD_PIO_SET     (PIO_OPENDRAIN|PIO_PULLUP|PIO_DEFAULT)
//    #define LCD_PIO_SET     (PIO_OPENDRAIN|PIO_DEFAULT)
//    #define LCD_PIO_SET     (PIO_PULLUP|PIO_DEFAULT)
//    #define LCD_PIO_SET     (PIO_DEFAULT)
//    #define LCD_PIO_UP      (PIO_PULLUP)
//    #define LCD_PIO_UP      (PIO_DEFAULT)

    #define LCD_PIO_SET     (PIO_PULLDN)
    #define LCD_PIO_UP      (PIO_PULLDN)
    const struct pio_desc hw_pio[] = {

        {"B0",        AT91C_PIN_PA(0),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B1",        AT91C_PIN_PA(1),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B2",        AT91C_PIN_PA(2),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B3",        AT91C_PIN_PA(3),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B4",        AT91C_PIN_PA(4),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B5",        AT91C_PIN_PA(5),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B6",        AT91C_PIN_PA(6),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B7",        AT91C_PIN_PA(7),    0, LCD_PIO_SET, PIO_PERIPH_A},

        {"G0",        AT91C_PIN_PA(8),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G1",        AT91C_PIN_PA(9),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G2",        AT91C_PIN_PA(10),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G3",        AT91C_PIN_PA(11),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G4",        AT91C_PIN_PA(12),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G5",        AT91C_PIN_PA(13),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G6",        AT91C_PIN_PA(14),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G7",        AT91C_PIN_PA(15),    0, LCD_PIO_SET, PIO_PERIPH_A},

        {"R0",        AT91C_PIN_PA(16),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R1",        AT91C_PIN_PA(17),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R2",        AT91C_PIN_PA(18),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R3",        AT91C_PIN_PA(19),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R4",        AT91C_PIN_PA(20),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R5",        AT91C_PIN_PA(21),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R6",        AT91C_PIN_PA(22),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R7",        AT91C_PIN_PA(23),    0, LCD_PIO_SET, PIO_PERIPH_A},

    #ifdef __USE_PIO_BL__
        {"LCDPWM",    AT91C_PIN_PA(24),    0, LCD_PIO_UP, PIO_OUTPUT_0},
    #else
        {"LCDPWM",    AT91C_PIN_PA(24),    0, LCD_PIO_SET, PIO_PERIPH_A},
    #endif

#if 0
//        {"VSYNC",    AT91C_PIN_PA(26),    0, LCD_PIO_UP, PIO_INPUT},
//        {"HSYNC",    AT91C_PIN_PA(27),    0, LCD_PIO_UP, PIO_INPUT},
        {"VSYNC",     AT91C_PIN_PA(26),    0, LCD_PIO_UP, PIO_OUTPUT_0},
        {"HSYNC",     AT91C_PIN_PA(27),    0, LCD_PIO_UP, PIO_OUTPUT_0},
        {"DEN",       AT91C_PIN_PA(29),    0, LCD_PIO_SET, PIO_PERIPH_A},
#else
#if 0
        {"VSYNC",     AT91C_PIN_PA(26),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"HSYNC",     AT91C_PIN_PA(27),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"DEN",       AT91C_PIN_PA(29),    0, LCD_PIO_SET, PIO_INPUT},
#else
        {"VSYNC",     AT91C_PIN_PA(26),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"HSYNC",     AT91C_PIN_PA(27),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"DEN",       AT91C_PIN_PA(29),    0, LCD_PIO_SET, PIO_PERIPH_A},
#endif
#endif

    #ifdef __USE_PIO_BL__
        {"LCDDISP",    AT91C_PIN_PA(25),    0, LCD_PIO_UP, PIO_OUTPUT_0},
    #else
        {"LCDDISP",    AT91C_PIN_PA(25),    0, LCD_PIO_SET, PIO_PERIPH_A},
    #endif

        {"LCDPCK",    AT91C_PIN_PA(28),    0, LCD_PIO_SET, PIO_PERIPH_A},

        /* peripheral  */
//        {"SPI1_NPCS",	AT91C_PIN_PC(28),	0, LCD_PIO_UP, PIO_OUTPUT_0},
//		{"SPI1_MISO",	AT91C_PIN_PC(22),	0, LCD_PIO_UP, PIO_OUTPUT_0},
//		{"SPI1_MOSI",	AT91C_PIN_PC(23),	0, LCD_PIO_UP, PIO_OUTPUT_0},
//		{"SPI1_SPCK",	AT91C_PIN_PC(24),	0, LCD_PIO_UP, PIO_OUTPUT_0},

//		{"TOUCH_PD24",	AT91C_PIN_PD(24),	0, LCD_PIO_UP, PIO_OUTPUT_0},

    };

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));
}
#endif

/*----------------------------------------------------------------------------
 *        Exported functions
 *----------------------------------------------------------------------------*/
/**
 * \brief Initializes the LCD controller.
 * Configure SMC to access LCD controller at 64MHz MCK.
 */
#ifndef _JBOOT_DISPLAY_INIT_
static void LCDD_Initialize( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;

    /* Configure PIO */
    IOConfig();

    LCDD_Off();
    
    /* Reset CLUT information */
    plcddBase->clut.bpp = 0;
    plcddOvr1->clut.bpp = 0;
    plcddOvr2->clut.bpp = 0;
    plcddHeo->clut.bpp  = 0;
    plcddHcc->clut.bpp  = 0;

    /* Reset layer information */
    plcddBase->pBuffer = NULL;
    plcddOvr1->pBuffer = NULL;
    plcddOvr2->pBuffer = NULL;
    plcddHeo->pBuffer  = NULL;
    plcddHcc->pBuffer  = NULL;

    ///* No canvas selected */
    //lcddCanvas.pBuffer = NULL;

    /* Enable peripheral clock */
    pPMC->PMC_PCER1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    pPMC->PMC_SCER = (0x1u << 3);

    /* Timing Engine Configuration */
   
    /* Disable interrupt */
    pLCDC->LCDC_LCDIDR = 0xFFFFFFFF;

    /* Configure channels */

    /* Base */
    pLCDC->LCDC_BASECFG0 =  LCDC_BASECFG0_DLBO | LCDC_BASECFG0_BLEN_AHB_INCR16;
    pLCDC->LCDC_BASECFG1 =  //LCDC_BASECFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_OVR1CFG1_RGBMODE_32BPP_ARGB_8888;
    //pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_32BPP_ARGB_8888;

    /* Overlay 1, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_OVR1CFG0 =  LCDC_OVR1CFG0_DLBO | LCDC_OVR1CFG0_BLEN_AHB_BLEN_INCR16
                       |  LCDC_OVR1CFG0_ROTDIS
                       ;
    pLCDC->LCDC_OVR1CFG1 =  //LCDC_OVR1CFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_OVR1CFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_OVR1CFG9 =  LCDC_OVR1CFG9_GA(0x8F) | LCDC_OVR1CFG9_GAEN;
    pLCDC->LCDC_OVR1CFG6 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR1CFG7 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR1CFG8 = 0xFFFFFF;
    pLCDC->LCDC_OVR1CFG9 |= LCDC_OVR1CFG9_CRKEY;

    /* Overlay 2, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_OVR2CFG0 =  LCDC_OVR2CFG0_DLBO | LCDC_OVR2CFG0_BLEN_AHB_INCR16
                       |  LCDC_OVR2CFG0_ROTDIS
                       ;
    pLCDC->LCDC_OVR2CFG1 =  //LCDC_OVR2CFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_OVR2CFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_OVR2CFG9 =  LCDC_OVR2CFG9_GA(0xFF) | LCDC_OVR2CFG9_GAEN;
    pLCDC->LCDC_OVR2CFG6 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR2CFG7 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR2CFG8 = 0xFFFFFF;
    pLCDC->LCDC_OVR2CFG9 |= LCDC_OVR2CFG9_CRKEY;

#if 1
    /* High End Overlay, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_HEOCFG0  =  LCDC_HEOCFG0_DLBO | LCDC_HEOCFG0_BLEN_AHB_BLEN_INCR16
                            |  LCDC_HEOCFG0_ROTDIS
                            ;
    pLCDC->LCDC_HEOCFG1  =  LCDC_HEOCFG1_YUVEN |
                            LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE3 /*|LCDC_HEOCFG1_YUV422SWP*/
                            ;
    pLCDC->LCDC_HEOCFG12 =  LCDC_HEOCFG12_GA(0xFF) | LCDC_HEOCFG12_GAEN;
    pLCDC->LCDC_HEOCFG9  = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG10 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG11 = 0xFFFFFF;
    pLCDC->LCDC_HEOCFG12 |= LCDC_HEOCFG12_CRKEY;

	/* XXX: evaluate pix->colorspace */
    pLCDC->LCDC_HEOCFG14 = 0x4c900091;
    pLCDC->LCDC_HEOCFG15 = 0x7a5f5090;
    pLCDC->LCDC_HEOCFG16 = 0x40040890;

#else    
    /* High End Overlay, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_HEOCFG0  =  LCDC_HEOCFG0_DLBO | LCDC_HEOCFG0_BLEN_AHB_BLEN_INCR16
                            |  LCDC_HEOCFG0_ROTDIS
                            ;
    pLCDC->LCDC_HEOCFG1  =  //LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_HEOCFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_HEOCFG12 =  LCDC_HEOCFG12_GA(0xFF) | LCDC_HEOCFG12_GAEN;
    pLCDC->LCDC_HEOCFG9  = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG10 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG11 = 0xFFFFFF;
    pLCDC->LCDC_HEOCFG12 |= LCDC_HEOCFG12_CRKEY;
#endif
    /* Hardware Cursor, GA 0xFF, , CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_HCRCFG0  =  LCDC_HCRCFG0_DLBO | LCDC_HCRCFG0_BLEN_AHB_BLEN_INCR16;
    pLCDC->LCDC_HCRCFG1  =  //LCDC_HCRCFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_HCRCFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_HCRCFG6  =  DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HCRCFG7  =  DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HCRCFG8  =  0xFFFFFF;
    pLCDC->LCDC_HCRCFG9  =  LCDC_HCRCFG9_GA(0xFF) | LCDC_HCRCFG9_GAEN | LCDC_HCRCFG9_CRKEY;

    //LCDD_On();
}
#endif

/**
 * Check if specified layer is working.
 * \param bLayer Layer ID.
 * \return 1 if layer is on.
 */
UINT8 LCDD_IsLayerOn( UINT8 bLayer )
{
    volatile UINT32 *pReg = pEnableReg(bLayer);
    if (pReg) return ((pReg[2] & LCDC_BASECHSR_CHSR) > 0);
    return 0;
}

/**
 * Enable(turn on)/Disable(hide) specified layer.
 * \param bLayer Layer ID.
 * \param bEnDis Enable/Disable.
 */
void LCDD_EnableLayer( UINT8 bLayer, UINT8 bEnDis )
{
    volatile UINT32 *pReg = pEnableReg(bLayer);
    volatile UINT32 *pBlR = pBlenderReg(bLayer);
    if (pReg && bLayer > LCDD_CONTROLLER)
    {
        if (bEnDis)
        {
            pReg[0]  = LCDC_BASECHER_CHEN | LCDC_BASECHER_UPDATEEN;
            pBlR[0] |= LCDC_HEOCFG12_DMA | LCDC_HEOCFG12_OVR;
        }
        else
        {
            pReg[1]  = LCDC_BASECHDR_CHDIS;
            pBlR[0] &= ~(LCDC_HEOCFG12_DMA | LCDC_HEOCFG12_OVR);
        }
    }
}

/**
 * Refresh layer
 * \param bLayer Layer ID.
 */
void LCDD_Refresh( UINT8 bLayer )
{
    volatile UINT32 *pBlR = pBlenderReg(bLayer);
    volatile UINT32 *pEnR = pEnableReg(bLayer);
    if (pBlR)
    {
        if (pEnR[2] & LCDC_OVR1CHSR_CHSR)
        {
            pBlR[0] |=  LCDC_HEOCFG12_DMA;
            pEnR[0] = LCDC_OVR1CHER_UPDATEEN;
        }
    }
}

/**
 * Set display window position.
 * \param bLayer Layer ID.
 * \param x X position.
 * \param y Y position.
 */
void LCDD_SetPosition( CHANNELINFO *pChannelInfo /*UINT8 bLayer*/, UINT32 x, UINT32 y )
{
    UINT8 bLayer  = LCDD_BASE ;
    switch( pChannelInfo->Channel )
    {
    case CHANNEL_BASE:
        bLayer = LCDD_BASE;
        break;

    case CHANNEL_POPUP:
        bLayer = LCDD_OVR1;

        break;

    case CHANNEL_VIDEO:
        bLayer = LCDD_HEO;
        break;

    case CHANNEL_POPUP2:
        bLayer = LCDD_OVR2;
        break;

    case CHANNEL_CURSOR:
        bLayer = LCDD_CUR;
        break;
    }

    volatile UINT32 *pChReg = pEnableReg(bLayer);
    volatile UINT32 *pXyReg = pWinReg(bLayer);
    UINT32 w, h;
    
    w = (pXyReg[1] & LCDC_OVR1CFG3_XSIZE_Msk) >> LCDC_OVR1CFG3_XSIZE_Pos;
    h = (pXyReg[1] & LCDC_OVR1CFG3_YSIZE_Msk) >> LCDC_OVR1CFG3_YSIZE_Pos;

    if( bLayer != LCDD_CUR )
    {
        if (x+w >= DISPLAY_WIDTH)  x = DISPLAY_WIDTH - w;
        if (y+h >= DISPLAY_HEIGHT) y = DISPLAY_HEIGHT - h;
    }
    else
    {
#if 1
        if( RotateAngle == 90 )
        {
            INT32 xt;
            INT32 yt;

            xt = y;
            yt = DISPLAY_HEIGHT - CURSOR_WIDTH - x;

            UINT32 xcoff = xt;
            if (xt+CURSOR_HEIGHT >= DISPLAY_WIDTH)  xt = DISPLAY_WIDTH - (CURSOR_HEIGHT);
            //if (y+h >= DISPLAY_HEIGHT) y = DISPLAY_HEIGHT - h;
            xcoff = xcoff - xt;

            UINT32 ycoff;
            if (yt < 0) {ycoff = -yt;  yt = 0;}
            else        {ycoff = 0;}

//DBGLINE(1,"coff = %d\n", coff );
            FillCursorImage( (BYTE *)pChannelInfo->Frame.Addr, xcoff, ycoff );

            x = xt;
            y = yt;
//DBGLINE(1,"x,y = (%d,%d)\n", x, y );
        }
        else
        {
            UINT32 xcoff = x;
            //if (x+w >= DISPLAY_WIDTH)  x = DISPLAY_WIDTH - w; 
            if (x+CURSOR_WIDTH >= DISPLAY_WIDTH)  x = DISPLAY_WIDTH - CURSOR_WIDTH; 
            //if (y+h >= DISPLAY_HEIGHT) y = DISPLAY_HEIGHT - h;

            xcoff = xcoff - x;
//DBGLINE(1,"coff = %d\n", coff );
            FillCursorImage( (BYTE *)pChannelInfo->Frame.Addr, xcoff, 0);
//DBGLINE(1,"x,y = (%d,%d)\n", x, y );
        }
#else
        if (x+w >= DISPLAY_WIDTH)  w = DISPLAY_WIDTH - x;
        else                       w = CURSOR_WIDTH;

        h = CURSOR_HEIGHT;
        if (y+h >= DISPLAY_HEIGHT)  h = DISPLAY_HEIGHT - y;
        else                       h = CURSOR_HEIGHT;
        FillCursorImage( (BYTE *)pChannelInfo->Frame.Addr, w, h);
#endif
    }

    if (pXyReg)
    {
        pXyReg[0] = LCDC_OVR1CFG2_XPOS(x) | LCDC_OVR1CFG2_YPOS(y);
        if (pChReg[2] & LCDC_OVR1CHSR_CHSR)
            pChReg[0] = LCDC_OVR1CHER_UPDATEEN;
    }
}

/**
 * Set Prioty of layer (only for HEO now).
 * \param bLayer Layer ID (HEO).
 * \param bPri   Prority value.
 */
void LCDD_SetPrioty( UINT8 bLayer, UINT8 bPri )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    if ( bLayer != LCDD_HEO ) return;
    if (bPri)
        pLCDC->LCDC_HEOCFG12 |=  LCDC_HEOCFG12_VIDPRI;
    else
        pLCDC->LCDC_HEOCFG12 &= ~LCDC_HEOCFG12_VIDPRI;
    pLCDC->LCDC_HEOCHER = LCDC_HEOCHER_UPDATEEN;
}

/**
 * Return Prioty of layer (only for HEO now).
 * \param bLayer Layer ID (HEO).
 */
UINT8 LCDD_GetPrioty( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    if ( bLayer != LCDD_HEO ) return 0;
    return (pLCDC->LCDC_HEOCFG12 & LCDC_HEOCFG12_VIDPRI) > 0;
}

/**
 * Global & Local Alpha Enable/Disable
 * \param bLayer   Layer ID.
 * \param bEnDisLA Enable/Disable local  alpha.
 * \param bEnDisGA Enable/Disable global alpha.
 */
void LCDD_EnableAlpha( UINT8 bLayer,
                       UINT8 bEnDisLA,
                       UINT8 bEnDisGA )
{
    volatile UINT32 *pEnR  = pEnableReg(bLayer);
    volatile UINT32 *pCfgR = pBlenderReg(bLayer);
    UINT32 cfg;
    if (pCfgR)
    {
        cfg = (*pCfgR) & ~(LCDC_OVR1CFG9_LAEN | LCDC_OVR1CFG9_GAEN);
        if (bEnDisGA) cfg |= LCDC_OVR1CFG9_GAEN;
        if (bEnDisLA) cfg |= LCDC_OVR1CFG9_LAEN;
        (*pCfgR) = cfg;

        pEnR[0] = LCDC_OVR1CHER_UPDATEEN;
    }
}

/**
 * Set alpha value
 * \param bLayer Layer ID (OVR1, HEO or CUR).
 * \param bReverse Reverse alpha (alpha -> 1 - alpha).
 * \param bAlpha   Global alpha value.
 */
void LCDD_SetAlpha( UINT8 bLayer,
                    UINT8 bReverse,
                    UINT8 bAlpha )
{
    volatile UINT32 *pEnR  = pEnableReg(bLayer);
    volatile UINT32 *pCfgR = pBlenderReg(bLayer);
    UINT32 cfg;

    if (pCfgR)
    {
        cfg = (*pCfgR) & ~(LCDC_OVR1CFG9_REVALPHA | LCDC_OVR1CFG9_GA_Msk);
        if (bReverse) cfg |= LCDC_OVR1CFG9_REVALPHA;
        (*pCfgR) = cfg | LCDC_OVR1CFG9_GA(bAlpha);

        pEnR[0] = LCDC_OVR1CHER_UPDATEEN;
    }
}

/**
 * Get alpha value
 * \param bLayer Layer ID (OVR1, HEO or CUR).
 */
UINT8 LCDD_GetAlpha( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    volatile UINT32 *pCfg;
    UINT32 bmMask = LCDC_OVR1CFG9_GA_Msk;
    UINT32 bShift = LCDC_OVR1CFG9_GA_Pos;

    switch( bLayer )
    {
        case LCDD_OVR1: pCfg = (volatile UINT32 *)&pLCDC->LCDC_OVR1CFG9; break;
        case LCDD_OVR2: pCfg = (volatile UINT32 *)&pLCDC->LCDC_OVR2CFG9; break;
        case LCDD_HEO:  pCfg = (volatile UINT32 *)&pLCDC->LCDC_HEOCFG9;  break;
        case LCDD_CUR:  pCfg = (volatile UINT32 *)&pLCDC->LCDC_HCRCFG9;  break;
        default: return 0;
    }

    return (((*pCfg) >> bShift) & bmMask);
}

/**
 * Enable and Set Color Keying
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 * \param bDstSrc Destination/Source keying.
 * \param dwColor Color to matching.
 * \param dwMask  Color bit mask.
 */
void LCDD_SetColorKeying( UINT8 bLayer, UINT8 bDstSrc,
                          UINT32 dwColor, UINT32 dwMask )
{
    volatile UINT32 *pEnR    = pEnableReg(bLayer);
    volatile UINT32 *pBCfgR  = pBlenderReg(bLayer);
    volatile UINT32 *pColorR = pColorReg(bLayer);
    if (pBCfgR == NULL) return;
    /* Select the Overlay to Blit */
    /* Dest/Source Keying */
    if (bDstSrc)    *pBCfgR |=  LCDC_HEOCFG12_DSTKEY;
    else            *pBCfgR &= ~LCDC_HEOCFG12_DSTKEY;
    /* Activate Color Keying */
    *pBCfgR |= LCDC_HEOCFG12_CRKEY;
    /* Program Color Keying */
    pColorR[0] = dwColor; // added by jbo
    pColorR[1] = dwColor;
    pColorR[2] = dwMask;
    /* Update */
    pEnR[0] = LCDC_HEOCHER_UPDATEEN;
}

/**
 * Disable Color Keying
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 */
void LCDD_DisableColorKeying( UINT8 bLayer )
{
    volatile UINT32 *pEnR    = pEnableReg(bLayer);
    volatile UINT32 *pBCfgR  = pBlenderReg(bLayer);
    volatile UINT32 *pColorR = pColorReg(bLayer);
    if (pBCfgR == NULL) return;
    *pBCfgR &= ~LCDC_HEOCFG12_CRKEY;
    pColorR[2] = 0;
    /* Update */
    pEnR[0] = LCDC_HEOCHER_UPDATEEN;
}

/**
 * Set Color Lookup Table
 * \param bLayer   Layer ID (OVR1, HEO or CUR).
 * \param pCLUT    Pointer to color lookup table.
 * \param bpp      Bits Per Pixel (1, 2, 4, 8).
 * \param nbColors Number of colors indexed in table.
 */
void LCDD_SetCLUT( UINT8 bLayer,
                   UINT32 *pCLUT,
                   UINT8 bpp, UINT8 nbColors )
{
    //AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    volatile UINT32* pCLUTR = pCLUTReg(bLayer);
    sCLUTInfo *        pInfo  = &pLayer(bLayer)->clut;

    if (pInfo == NULL) return;

    pInfo->bpp = bpp;
    /* Customize CLUT */
    if (pCLUT)
    {
        UINT32 i;
        if (nbColors == 0) nbColors = 1 << bpp;
        pInfo->nbColors = nbColors;
        for (i = 0; i < nbColors; i ++) pCLUTR[i] = pCLUT[i];
    }
    /* Build CLUT */
    else
    {
        pInfo->nbColors = 1 << bpp;
        switch (bpp)
        {
            case 1: LCDD_BuildCLUT1(pCLUTR); break;
            case 2: LCDD_BuildCLUT2(pCLUTR); break;
            case 4: LCDD_BuildCLUT4(pCLUTR); break;
            case 8: LCDD_BuildCLUT8(pCLUTR); break;
        }
    }
}

/**
 * Display an image on specified layer.
 * (Image scan origion: Left -> Right, Top -> Bottom.)
 * \note w & h should be the rotated result.
 * \note for LCDD_BASE: x, y don't care. w always > 0.
 * \note for LCDD_HEO:imgW & imgH is used.
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 * \param pBuffer Pointer to image data.
 * \param bPP     Bits Per Pixel.
 *                - 16: TRGB 1555
 *                - 24:  RGB  888  packed
 *                - 32: ARGB 8888
 * \param x       X position.
 * \param y       Y position.
 * \param w       Width  (<0 means Right  -> Left data).
 * \param h       Height (<0 means Bottom -> Top data).
 * \param imgW    Source image width.
 * \param imgH    Source image height.
 * \param wRotate Rotation (clockwise, 0, 90, 180, 270 accepted).
 */
void *LCDD_ShowBMPRotated( UINT8 bLayer,
                           void* pBuffer, UINT8 bpp,
                           UINT32 x, UINT32 y,
                           INT32  w, INT32  h,
                           UINT32 imgW, UINT32 imgH,
                           INT16 wRotate)
{
    //AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    sLayer            *pLD   = pLayer(bLayer);
    //sCLUTInfo         *pClut = &pLD->clut;
    sLCDCDescriptor   *pTD   = &pLD->dmaD;
    volatile UINT32 *pEnR  = pEnableReg(bLayer);
    volatile UINT32 *pDmaR = pHeadReg(bLayer);
    volatile UINT32 *pWinR = pWinReg(bLayer);
    volatile UINT32 *pStrR = pStrideReg(bLayer);
    volatile UINT32 *pSclR = pScaleReg(bLayer);
    volatile UINT32 *pBlR  = pBlenderReg(bLayer);
    volatile UINT32 *pCfgR = pCfgReg(bLayer);
    UINT8 bPStride = LCDD_IsPStrideSupported(bLayer);

    UINT8 bBottomUp  = (h < 0);
    UINT8 bRightLeft = (w < 0);
    UINT32 padding = 0;
    INT32 srcW, srcH;
    UINT32 bitsPRow, bytesPRow;
    UINT32 bytesPPix = bpp >> 3;

    void* pOldBuffer = pLD->pBuffer;

    if (pCfgR == NULL)  return pOldBuffer;

kprintf("Show %x @ %d: (%d,%d)+(%d,%d) img %d x %d * %d\n\r", pBuffer, bLayer, x, y, w, h, imgW, imgH, bpp);

    if( bLayer == LCDD_HEO)
    {
        pCfgR[1] = LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE2 /*| LCDC_HEOCFG1_YUV422SWP*/;
    }
    else
    {
        switch (bpp)
        {
            case 16: /*  TRGB 1555 */
                pCfgR[1] = LCDC_HEOCFG1_RGBMODE_16BPP_TRGB_1555;
                break;
            case 24: /*  RGB  888 packed */
                pCfgR[1] = LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED;
                break;
            case 32: /* ARGB 8888 */
                pCfgR[1] = LCDC_HEOCFG1_RGBMODE_32BPP_ARGB_8888;
                break;
            default: return pOldBuffer;
        }
    }

    /* Windows position & size check */
    if (h < 0)          h =- h;
    if (w < 0)          w =- w;
    if (x + w > DISPLAY_WIDTH)
    {
        //printf("! w %d -> %d\n\r", w, DISPLAY_WIDTH-x);
        w = DISPLAY_WIDTH - x;
    }
    if (y + h > DISPLAY_HEIGHT)
    {
        //printf("! h %d -> %d\n\r", h, DISPLAY_HEIGHT-y);
        h = DISPLAY_HEIGHT - y;
    }
    if (w == 0) w ++;
    if (h == 0) h ++;
    if (imgW == 0) imgW ++;
    if (imgH == 0) imgH ++;

    /* Only 0,(-)90,(-)180,(-)270 accepted */
    switch(wRotate)
    {
        case 0: case 90: case 180: case 270:
            break;
        case -90: case -180: case -270:
            wRotate += 360;
            break;
        default: return NULL;
    }

    /* Setup display buffer & window */
    if (pBuffer) pLD->pBuffer = pBuffer;
    else         pBuffer = pLD->pBuffer;

    /* Set display buffer & mode */
    bitsPRow  = imgW * bpp;
    bytesPRow = bitsPRow >> 3;
    if (bitsPRow & 0x7) bytesPRow ++;
    if (bytesPRow & 0x3) padding = 4 - (bytesPRow & 0x3);

    /* No X mirror supported layer, no Right->Left scan */
    if (!bPStride)  bRightLeft = 0;

    /* --------- Mirror & then rotate --------- */
    /* Normal direction: Left,Top -> Right,Down */
    if (   (!bRightLeft && !bBottomUp && wRotate ==   0)
        || ( bRightLeft &&  bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X0 ++ */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0);
        /* Y0 ++ */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(padding);
        /* Pointer to Left,Top (x0,y0) */
    }
    /* X mirror: Right,Top -> Left,Down */
    else if ( ( bRightLeft && !bBottomUp && wRotate ==   0)
            ||(!bRightLeft &&  bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X1 -- */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-2*bytesPPix);
        /* Y0 ++ */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(bytesPRow*2+padding-2*bytesPPix);
        /* Pointer to Right,Top (x1,y0) */
        pBuffer = (void*)((UINT32)pBuffer
                + bytesPPix*(imgW-1));
    }
    /* Y mirror: Left,Down -> Right,Top */
    else if ( (!bRightLeft &&  bBottomUp && wRotate ==   0)
            ||( bRightLeft && !bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X0 ++ */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0);
        /* Y1 -- */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-(bytesPRow*2+padding));
        /* Pointer to Left,Down (x0,y1) */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1));
    }
    /* X,Y mirror: Right,Top -> Left,Down */
    else if ( ( bRightLeft &&  bBottomUp && wRotate ==   0)
            ||(!bRightLeft && !bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X1 -- */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-2*bytesPPix);
        /* Y1 -- */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-(bytesPPix*2+padding));
        /* Pointer to Left,Down (x1,y1) */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1)
                + (bytesPPix)*(imgW-1));
    }
    /* Rotate  90: Down,Left -> Top,Right (with w,h swap) */
    else if ( (!bRightLeft && !bBottomUp && wRotate ==  90)
            ||( bRightLeft &&  bBottomUp && wRotate == 270) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y -- as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-(bytesPPix+bytesPRow+padding));
        /* X ++ as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE((bytesPRow+padding)*(imgH-1));
        /* Pointer to Bottom,Left */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1));
    }
    /* Rotate 270: Top,Right -> Down,Left (with w,h swap) */
    else if ( (!bRightLeft && !bBottomUp && wRotate == 270)
            ||( bRightLeft &&  bBottomUp && wRotate ==  90) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y ++ as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(bytesPRow+padding-bytesPPix);
        /* X -- as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-2*bytesPPix-(bytesPRow+padding)*(imgH-1));
        /* Pointer to top right */
        pBuffer = (void*)((UINT32)pBuffer
                + bytesPPix*(imgW-1));
    }
    /* Mirror X then Rotate 90: Down,Right -> Top,Left */
    else if ( ( bRightLeft && !bBottomUp && wRotate ==  90)
            ||(!bRightLeft &&  bBottomUp && wRotate == 270) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y -- as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-(bytesPPix+bytesPRow+padding));
        /* X -- as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-2*bytesPPix+(bytesPRow+padding)*(imgH-1));
        /* Pointer to down right (x1,y1) */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1)
                + (bytesPPix)*(imgW-1));
    }
    /* Mirror Y then Rotate 90: Top,Left -> Down,Right */
    else if ( (!bRightLeft &&  bBottomUp && wRotate ==  90)
            ||( bRightLeft && !bBottomUp && wRotate == 270) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y ++ as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(bytesPRow+padding-bytesPPix);
        /* X ++ as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-(bytesPRow+padding)*(imgH-1));
        /* Pointer to top left (x0,y0) */
    }
    /** DMA is running, just add new descriptor to queue */
    if (pBlR[0] & LCDC_HEOCFG12_DMA)
    {
        pTD->addr = (UINT32)pBuffer;
        pTD->ctrl = LCDC_HEOCTRL_DFETCH;
        pTD->next = (UINT32)pTD + DMA_ADDR_OFFSET;
        pDmaR[0] = (UINT32)pTD + DMA_ADDR_OFFSET;
        pEnR[0] = LCDC_HEOCHER_A2QEN;

        //CacheRangeFlush(pTD, sizeof(sLCDCDescriptor), CACHE_SYNC_WRITEBACK);
        CP15_coherent_dcache_for_dma ((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));
    }
    else
    {
        /* 2. Write the channel descriptor (DSCR) structure in the system memory by
              writing DSCR.CHXADDR Frame base address, DSCR.CHXCTRL channel control
              and DSCR.CHXNEXT next descriptor location.        
           3. If more than one descriptor is expected, the DFETCH field of
              DSCR.CHXCTRL is set to one to enable the descriptor fetch operation.      
           4. Write the DSCR.CHXNEXT register with the address location of the
              descriptor structure and set DFETCH field of the DSCR.CHXCTRL register
              to one. */
        LCDD_SetDMA(pBuffer, pTD, (UINT32)pDmaR);
    }
    //CP15_flush_dcache_for_dma ((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));

    /* Set window & position */
    if (pWinR)
    {
        pWinR[0] = LCDC_HEOCFG2_XPOS(x) | LCDC_HEOCFG2_YPOS(y);
        pWinR[1] = LCDC_HEOCFG3_XSIZE(w-1) | LCDC_HEOCFG3_YSIZE(h-1);
    }
    /* Scaling setup */
    if (pSclR)
    {
        /* Image size only used in scaling */
        /* Scaling target */
        if (wRotate == 90 || wRotate == 270)
        {
            srcW = imgH; srcH = imgW;
        }
        else
        {
            srcW = imgW; srcH = imgH;
        }
        pWinR[2] = LCDC_HEOCFG4_XMEM_SIZE(srcW-1) | LCDC_HEOCFG4_YMEM_SIZE(srcH-1);
        /* Scaled */
        if (w != srcW || h != srcH)
        {
            UINT16 wYf, wXf;
            wXf = LCDD_CalcScaleFactor(w, srcW);
            wYf = LCDD_CalcScaleFactor(h, srcH);
            //printf("- Scale(%d,%d)\n\r", wXf, wYf);
            pSclR[0] = LCDC_HEOCFG13_YFACTOR(wYf)
                     | LCDC_HEOCFG13_XFACTOR(wXf)
                     | LCDC_HEOCFG13_SCALEN
                     ;
        }
        /* Disable scaling */
        else
        {
            pSclR[0] = 0;
        }
    }
    /* Enable DMA */
    if (pBuffer)
    {
        pBlR[0] |= LCDC_HEOCFG12_DMA
                 | LCDC_HEOCFG12_OVR
                 ;
    }
    /* Enable & Update */
    /* 5. Enable the relevant channel by writing one to the CHEN field of the
          CHXCHER register. */
    pEnR[0] = LCDC_HEOCHER_UPDATEEN | LCDC_HEOCHER_CHEN;

    /* 6. An interrupt may be raised if unmasked when the descriptor has been
          loaded.  */

    return pOldBuffer;
}

/**
 * Display an image on specified layer.
 * (Image scan: Left -> Right, Top -> Bottom.)
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 * \param pBuffer Pointer to image data.
 * \param bPP     Bits Per Pixel.
 *                - 16: TRGB 1555
 *                - 24:  RGB  888  packed
 *                - 32: ARGB 8888
 * \param x       X position.
 * \param y       Y position.
 * \param w       Width  (<0 means Right  -> Left data).
 * \param h       Height (<0 means Bottom -> Top data).
 * \param imgW    Source image width.
 * \param imgH    Source image height.
 * \return Pointer to old display image data.
 */
void *LCDD_ShowBMPScaled( UINT8 bLayer,
                          void* pBuffer, UINT8 bpp,
                          UINT32 x, UINT32 y,
                          INT32  w, INT32  h,
                          UINT32 imgW, UINT32 imgH )
{
    return LCDD_ShowBMPRotated(bLayer, pBuffer, bpp,
                               x, y, w, h, imgW, imgH, 0);
}

/**
 * Display an image on specified layer.
 * (Image scan: Left -> Right, Top -> Bottom.)
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 * \param pBuffer Pointer to image data.
 * \param bPP     Bits Per Pixel.
 *                - 16: TRGB 1555
 *                - 24:  RGB  888  packed
 *                - 32: ARGB 8888
 * \param x       X position.
 * \param y       Y position.
 * \param w       Width
 * \param h       Height (<0 means Bottom -> Top data).
 * \return Pointer to old display image data.
 */
void *LCDD_ShowBMP( UINT8 bLayer,
                    void* pBuffer, UINT8 bpp,
                    UINT32 x, UINT32 y,
                    INT32  w, INT32  h )
{
    return LCDD_ShowBMPRotated(bLayer, pBuffer, bpp,
                               x, y, w, h,
                               w, (h < 0) ? (-h) : h,
                               0);
}

/**
 * Start display on base layer
 * \param pBuffer   Pointer to image data.
 * \param bpp       Bits Per Pixel.
 * \param bBottomUp Scan from bottom to top.
 * \return Pointer to old display image data.
 */
void *LCDD_ShowBase( void* pBuffer, UINT8 bpp, UINT8 bBottomUp )
{
    return LCDD_ShowBMP(LCDD_BASE, pBuffer, bpp,
                        0, 0,
                        DISPLAY_WIDTH,
                        bBottomUp ? -DISPLAY_HEIGHT : DISPLAY_HEIGHT);
}

/**
 * Stop display on base layer
 */
void LCDD_StopBase( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    if (!(pLCDC->LCDC_BASECHSR & LCDC_BASECHSR_CHSR))
        return;

    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */
    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */
    LCDD_ClearDMA(&plcddBase->dmaD, (UINT32)&pLCDC->LCDC_BASEADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */
    pLCDC->LCDC_BASECHDR = LCDC_BASECHDR_CHDIS;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_BASECHSR & LCDC_BASECHSR_CHSR);
}

/**
 * Start display on overlay 1 layer
 */
void *LCDD_ShowOvr1( void* pBuffer, UINT8 bpp,
                     UINT32 x, UINT32 y, INT32 w, INT32 h )
{
#if 0
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    /* Enable default transparent keying */
    if (!(pLCDC->LCDC_OVR1CFG9 & LCDC_OVR1CFG9_CRKEY))
    {
        pLCDC->LCDC_OVR1CFG7 = DEFAULT_TRANSPARENT_COLOR;
        pLCDC->LCDC_OVR1CFG8 = 0xFFFFFF;
        pLCDC->LCDC_OVR1CFG9 |= LCDC_OVR1CFG9_CRKEY;
    }
#endif

    return LCDD_ShowBMP(LCDD_OVR1,
                        pBuffer, bpp, x, y, w, h);
}

/**
 * Stop display on overlay 1 layer
 */
void LCDD_StopOvr1( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    if (!(pLCDC->LCDC_OVR1CHSR & LCDC_OVR1CHSR_CHSR))
        return;

    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */
    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */
    LCDD_ClearDMA(&plcddOvr1->dmaD, (UINT32)&pLCDC->LCDC_OVR1ADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */
    pLCDC->LCDC_OVR1CHDR = LCDC_OVR1CHDR_CHDIS;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_OVR1CHSR & LCDC_OVR1CHSR_CHSR);
}
/**
 * Start display on overlay 1 layer
 */
void *LCDD_ShowOvr2( void* pBuffer, UINT8 bpp,
                     UINT32 x, UINT32 y, INT32 w, INT32 h )
{
#if 0
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    /* Enable default transparent keying */
    if (!(pLCDC->LCDC_OVR2CFG9 & LCDC_OVR2CFG9_CRKEY))
    {
        pLCDC->LCDC_OVR2CFG7 = DEFAULT_TRANSPARENT_COLOR;
        pLCDC->LCDC_OVR2CFG8 = 0xFFFFFF;
        pLCDC->LCDC_OVR2CFG9 |= LCDC_OVR2CFG9_CRKEY;
    }
#endif

    return LCDD_ShowBMP(LCDD_OVR2,
                        pBuffer, bpp, x, y, w, h);
}

/**
 * Stop display on overlay 1 layer
 */
void LCDD_StopOvr2( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    if (!(pLCDC->LCDC_OVR1CHSR & LCDC_OVR2CHSR_CHSR))
        return;

    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */
    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */
    LCDD_ClearDMA(&plcddOvr2->dmaD, (UINT32)&pLCDC->LCDC_OVR2ADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */
    pLCDC->LCDC_OVR2CHDR = LCDC_OVR2CHDR_CHDIS;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_OVR2CHSR & LCDC_OVR2CHSR_CHSR);
}

/**
 * Start display on High End Overlay layer
 */
void * LCDD_ShowHeo( void *pBuffer, UINT8 bpp,
                     UINT32 x, UINT32 y, INT32 w, INT32 h,
                     UINT32 imgW, UINT32 imgH )
{
    return LCDD_ShowBMPRotated(LCDD_HEO,
                               pBuffer, bpp,
                               x, y, w, h,
                               imgW,imgH,
                               0);
}

/**
 * Stop display on High End Overlay layer
 */
void LCDD_StopHeo( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    if (!(pLCDC->LCDC_HEOCHSR  & LCDC_HEOCHSR_CHSR))
        return;

    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */
    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */
    LCDD_ClearDMA(&plcddHeo->dmaD[0], (UINT32)&pLCDC->LCDC_HEOADDR);
    LCDD_ClearDMA(&plcddHeo->dmaD[1], (UINT32)&pLCDC->LCDC_HEOUADDR);
    LCDD_ClearDMA(&plcddHeo->dmaD[2], (UINT32)&pLCDC->LCDC_HEOVADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */
    pLCDC->LCDC_HEOCHDR  = LCDC_HEOCHDR_CHDIS;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_HEOCHSR  & LCDC_HEOCHSR_CHSR);
}

/**
 * Start display on Hardware Cursor layer
 * (Default transparent color is set to #000000, black)
 */
void *LCDD_ShowHcr( void* pBuffer, UINT8 bpp,
                    UINT32 x, UINT32 y, INT32 w, INT32 h )
{
#if 1
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    /* Enable default transparent keying */
    if (!(pLCDC->LCDC_HCRCFG9 & LCDC_HCRCFG9_CRKEY))
    {
        pLCDC->LCDC_HCRCFG7 = DEFAULT_TRANSPARENT_COLOR;
        pLCDC->LCDC_HCRCFG8 = 0xFFFFFF;
        pLCDC->LCDC_HCRCFG9 |= LCDC_HCRCFG9_CRKEY;
    }
#endif
    return LCDD_ShowBMP(LCDD_CUR,
                        pBuffer, bpp,
                        x, y, w, h);
}

/**
 * Stop display on Hardware Cursor layer
 */
void LCDD_StopHcr( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    if (!(pLCDC->LCDC_HCRCHDR  & LCDC_HCRCHSR_CHSR))
        return;

    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */
    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */
    LCDD_ClearDMA(&plcddHcc->dmaD, (UINT32)&pLCDC->LCDC_HCRADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */
    pLCDC->LCDC_HCRCHDR  = LCDC_HCRCHDR_CHDIS;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_HCRCHSR  & LCDC_HCRCHSR_CHSR);
}




//// Internal function, setup all register in the LCDC
//STATIC VOID
//ChipConfig( UINT32 FrameBuffer )
//{
//
//    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
//    //AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_VIRT_LCDC;
//
//
//    /* Disable interrupt */
//    pLCDC->LCDC_LCDIDR = 0xFFFFFFFF;
//
//    /* Configure channels */
//
//    /* Base */
//    pLCDC->LCDC_BASECFG0 =  LCDC_BASECFG0_DLBO | LCDC_BASECFG0_BLEN_AHB_INCR16;
//    //pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_32BPP_ARGB_8888;
//
//    /* Overlay 1, GA 0xFF */
//    pLCDC->LCDC_OVR1CFG0 =  LCDC_OVR1CFG0_DLBO | LCDC_OVR1CFG0_BLEN_AHB_BLEN_INCR16
//                       |  LCDC_OVR1CFG0_ROTDIS
//                       ;
//    pLCDC->LCDC_OVR1CFG1 =  LCDC_OVR1CFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_OVR1CFG9 =  LCDC_OVR1CFG9_GA(0xFF) | LCDC_OVR1CFG9_GAEN;
//
//    /* Overlay 2, GA 0xFF */
//    pLCDC->LCDC_OVR2CFG0 =  LCDC_OVR2CFG0_DLBO | LCDC_OVR2CFG0_BLEN_AHB_INCR16
//                       |  LCDC_OVR2CFG0_ROTDIS
//                       ;
//    pLCDC->LCDC_OVR2CFG1 =  LCDC_OVR2CFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_OVR2CFG9 =  LCDC_OVR2CFG9_GA(0xFF) | LCDC_OVR2CFG9_GAEN;
//
//    /* High End Overlay, GA 0xFF */
//    pLCDC->LCDC_HEOCFG0  =  LCDC_HEOCFG0_DLBO | LCDC_HEOCFG0_BLEN_AHB_BLEN_INCR16
//                       |  LCDC_HEOCFG0_ROTDIS
//                       ;
//    pLCDC->LCDC_HEOCFG1  =  LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_HEOCFG12 =  LCDC_HEOCFG12_GA(0xFF) | LCDC_HEOCFG12_GAEN;
//
//    /* Hardware Cursor, GA 0xFF, Key #000000 */
//    pLCDC->LCDC_HCRCFG0  =  LCDC_HCRCFG0_DLBO | LCDC_HCRCFG0_BLEN_AHB_BLEN_INCR16;
//    pLCDC->LCDC_HCRCFG1  =  LCDC_HCRCFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_HCRCFG7  =  0x000000;
//    pLCDC->LCDC_HCRCFG8  =  0xFFFFFF;
//    pLCDC->LCDC_HCRCFG9  =  LCDC_HCRCFG9_GA(0xFF) | LCDC_HCRCFG9_GAEN;
//
//    /* 1. Configure LCD timing parameters, signal polarity and clock period. */
//    pLCDC->LCDC_LCDCFG0 = LCDC_LCDCFG0_CLKDIV((BOARD_MCK*2)/BOARD_LCD_PIXELCLOCK-2)
//                       |LCDC_LCDCFG0_CGDISHCR
//                       |LCDC_LCDCFG0_CGDISHEO
//                       |LCDC_LCDCFG0_CGDISOVR1
//                       |LCDC_LCDCFG0_CGDISOVR2
//                       |LCDC_LCDCFG0_CGDISBASE
//                       |LCDC_LCDCFG0_CLKPWMSEL  
//                       |LCDC_LCDCFG0_CLKSEL
//                       |LCDC_LCDCFG0_CLKPOL;
//
//    pLCDC->LCDC_LCDCFG1 = LCDC_LCDCFG1_VSPW(BOARD_LCD_TIMING_VPW-1)
//                       |LCDC_LCDCFG1_HSPW(BOARD_LCD_TIMING_HPW-1);
//
//    pLCDC->LCDC_LCDCFG2 = LCDC_LCDCFG2_VBPW(BOARD_LCD_TIMING_VBP)
//                       |LCDC_LCDCFG2_VFPW(BOARD_LCD_TIMING_VFP-1);
//
//    pLCDC->LCDC_LCDCFG3 = LCDC_LCDCFG3_HBPW(BOARD_LCD_TIMING_HBP-1)
//                       |LCDC_LCDCFG3_HFPW(BOARD_LCD_TIMING_HFP-1);
//
//    pLCDC->LCDC_LCDCFG4 = LCDC_LCDCFG4_RPF(DISPLAY_HEIGHT-1)
//                       |LCDC_LCDCFG4_PPL(DISPLAY_WIDTH-1);
//
//    pLCDC->LCDC_LCDCFG5 = LCDC_LCDCFG5_GUARDTIME(30)
//                       |LCDC_LCDCFG5_MODE_OUTPUT_24BPP
//                       |LCDC_LCDCFG5_DISPDLY
//                       |LCDC_LCDCFG5_VSPDLYS
//                       |LCDC_LCDCFG5_VSPOL
//                       |LCDC_LCDCFG5_HSPOL;
//
//    pLCDC->LCDC_LCDCFG6 = LCDC_LCDCFG6_PWMCVAL(0xF0)
//                       |LCDC_LCDCFG6_PWMPOL
//                       |LCDC_LCDCFG6_PWMPS(6);
//
//    /* 2. Enable the Pixel Clock by writing one to the CLKEN field of the
//          LCDC_LCDEN register. */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_CLKEN;
//    /* 3. Poll CLKSTS field of the LCDC_LCDSR register to check that the clock
//       is running. */
//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_CLKSTS));
//
//    /* 4. Enable Horizontal and Vertical Synchronization by writing one to the
//          SYNCEN field of the LCDC_LCDEN register. */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_SYNCEN;
//    /* 5. Poll LCDSTS field of the LCDC_LCDSR register to check that the
//          synchronization is up. */
//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_LCDSTS));
//
//    /* 6. Enable the display power signal writing one to the DISPEN field of the
//          LCDC_LCDEN register. */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_DISPEN;
//    /* 7. Poll DISPSTS field of the LCDC_LCDSR register to check that the power
//          signal is activated. */
//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS));
//    /* 8. Enable backlight */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_PWMEN;
//
//    pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_32BPP_ARGB_8888;
//
//    LDCD_Desc.addr = (UINT32)FrameBuffer;
//    LDCD_Desc.ctrl = LCDC_BASECTRL_DFETCH;
//    LDCD_Desc.next = (UINT32)&LDCD_Desc + DMA_ADDR_OFFSET;
//
//    DEBUGMSG(DBG_LVL_LCD,"%s(%d) :  LDCD_Desc %X LDCD_Desc->next =%X\n", __FUNCTION__ ,__LINE__,&LDCD_Desc,LDCD_Desc.next);
//    pLCDC->LCDC_BASEHEAD = (UINT32)&LDCD_Desc + DMA_ADDR_OFFSET;;
//    pLCDC->LCDC_BASEADDR = (UINT32)FrameBuffer;
//    pLCDC->LCDC_BASECTRL |= LCDC_BASECTRL_DFETCH;
//    pLCDC->LCDC_BASENEXT = (UINT32)&LDCD_Desc + DMA_ADDR_OFFSET;
//
//    DEBUGMSG(DBG_LVL_LCD,"%s(%d) :  &pLCDC->LCDC_BASEHEAD =%X\n", __FUNCTION__ ,__LINE__,&pLCDC->LCDC_BASEHEAD);
//    DEBUGMSG(DBG_LVL_LCD,"%s(%d) :  pLCDC->LCDC_HCRCLUT =%X\n", __FUNCTION__ ,__LINE__,pLCDC->LCDC_HCRCLUT);
//
//    pLCDC->LCDC_BASECFG4 |= LCDC_BASECFG4_DMA;
//    pLCDC->LCDC_BASECHER = (LCDC_BASECHER_CHEN | LCDC_BASECHER_UPDATEEN);
////    =========================================================
//#if 1//AC fill yellow to LCD
//{
//    int i;
//    PUINT32 p = (PUINT32)FrameBuffer;
//
//    for(i = 0;i < DISPLAY_STRIDE*DISPLAY_HEIGHT/4;i++)
//        *p++ = 0xFFFF00;
//}
//#endif
//
//    return;
//}

#ifndef _JBOOT_DISPLAY_INIT_
STATIC VOID
BackLightInit( VOID )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    UINT32 cfg = pLCDC->LCDC_LCDCFG6 & ~LCDC_LCDCFG6_PWMCVAL_Msk;
/*
    const struct pio_desc hw_pio[] = {
        {"EN",    AT91C_PIN_PC(26),    1, PIO_DEFAULT, PIO_PERIPH_A}
    };

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));
*/
    pLCDC->LCDC_LCDCFG6 = cfg | LCDC_LCDCFG6_PWMCVAL(DEFAULT_BRIGHT_LEVEL);
    DEBUGMSG(DBG_LVL_LCD,"LCDC_LCDCFG6 = %X\n",pLCDC->LCDC_LCDCFG6 );
    return;
}
#endif

//STATIC VOID
VOID
DisplayEnableLayer( CHANNELINFO *pChannelInfo ) 
{
    switch( pChannelInfo->Channel )
    {
    case CHANNEL_BASE:
        if( pChannelInfo->bOn == FALSE )
        {
            LCDD_StopBase();
            break;
        }
        LCDD_ShowBase((void *)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET)/*FrameBuffer*/, 
                               pChannelInfo->Frame.nBpp, FALSE);
        LCDD_EnableLayer(LCDD_BASE, TRUE);
        break;

    case CHANNEL_POPUP:
        if( pChannelInfo->bOn == FALSE )
        {
            LCDD_StopOvr1();
            LCDD_EnableLayer(LCDD_OVR1, FALSE);
            break;
        }
        LCDD_ShowOvr1((void *)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET)/*FrameBuffer*/, 
                              pChannelInfo->Frame.nBpp,
                              pChannelInfo->xOff, pChannelInfo->yOff, pChannelInfo->Width, pChannelInfo->Height);
        LCDD_EnableLayer(LCDD_OVR1, TRUE);

        
        //LCDD_SetAlpha( LCDD_OVR1, FALSE, 0x80);
        //LCDD_EnableAlpha( LCDD_OVR1, TRUE, TRUE );

        break;

    case CHANNEL_VIDEO:
        if( pChannelInfo->bOn == FALSE )
        {
            LCDD_StopHeo();
            LCDD_EnableLayer(LCDD_HEO, FALSE);
            break;
        }
        LCDD_ShowHeo((void *)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET)/*FrameBuffer*/, 
                              pChannelInfo->Frame.nBpp,
                              pChannelInfo->xOff, pChannelInfo->yOff, pChannelInfo->Width, pChannelInfo->Height,
                              pChannelInfo->Width, pChannelInfo->Height);
        LCDD_EnableLayer(LCDD_HEO, TRUE);
        break;

    case CHANNEL_POPUP2:
        if( pChannelInfo->bOn == FALSE )
        {
            LCDD_StopOvr2();
            LCDD_EnableLayer(LCDD_OVR2, FALSE);
            break;
        }
        LCDD_ShowOvr2((void *)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET)/*FrameBuffer*/, 
                               pChannelInfo->Frame.nBpp,
                               pChannelInfo->xOff, pChannelInfo->yOff, pChannelInfo->Width, pChannelInfo->Height);
        LCDD_EnableLayer(LCDD_OVR2, TRUE);

        //LCDD_SetAlpha( LCDD_OVR2, FALSE, 0x80);
        //LCDD_EnableAlpha( LCDD_OVR2, TRUE, TRUE );
        break;

    case CHANNEL_CURSOR:
        if( pChannelInfo->bOn == FALSE )
        {
            LCDD_StopHcr();
            LCDD_EnableLayer(LCDD_CUR, FALSE);
            break;
        }
        LCDD_ShowHcr((void *)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET)/*FrameBuffer*/, 
                              pChannelInfo->Frame.nBpp,
                              pChannelInfo->xOff, pChannelInfo->yOff, pChannelInfo->Width, pChannelInfo->Height);
        LCDD_EnableLayer(LCDD_CUR, TRUE);

        LCDD_SetAlpha( LCDD_CUR, FALSE, 0x80);
        LCDD_EnableAlpha( LCDD_CUR, TRUE, TRUE );
        break;
    }

    return;
}

PUBLIC BOOL
DisplayInit( CHANNELINFO *pChannelInfo ) 
{
    PSHARED_MEMORY  pSharedMemory = (PSHARED_MEMORY)SHARED_MEMORY_BASE;

    plcddBase = (sLayer *)&pSharedMemory->VideoLayer[0];
    plcddOvr1 = (sLayer *)&pSharedMemory->VideoLayer[1];
    plcddOvr2 = (sLayer *)&pSharedMemory->VideoLayer[2];
    plcddHeo = (sHeoLayer *)&pSharedMemory->VideoLayer[3];
    plcddHcc = (sLayer *)&pSharedMemory->VideoLayer[4];

#ifdef _WITH_HDMI_

DBGLINE(1,"\n");
    STATIC BOOL bHMI_Inited = FALSE;
    if( !bHMI_Inited )
    {
        bHMI_Inited = TRUE;
DBGLINE(1,"\n");
        /* Config SIL9022 parameter */
        Sil9022a_InitializeStateVariables();
        //Sil9022a_VideoSel( 0xFF, &hdmi_video_mode );
        Sil9022a_VideoSel( HDMI_480P60_4X3, &hdmi_video_mode );
        //Sil9022a_VideoSel( HDMI_480I60_4X3, &hdmi_video_mode );
        //Sil9022a_VideoSel( HDMI_800_600_60, &hdmi_video_mode );
        //Sil9022a_VideoSel( HDMI_1024_768_60, &hdmi_video_mode );
        
DBGLINE(1,"\n");
        Sil9022a_AudioSel( AFS_44K1 );

DBGLINE(1,"\n");
        /* reset the SIL9022 */
        const struct pio_desc hw_pio_1[] = 
        {
            {"HDMI_RESET",	AT91C_PIN_PC(30),	1, PIO_DEFAULT, PIO_OUTPUT_1},
            {"HDMI_INT",	AT91C_PIN_PC(31),	0, PIO_DEFAULT, PIO_INPUT},

            //{"HDMI_RESET",    AT91C_PIN_PC(28),    0, PIO_DEFAULT, PIO_OUTPUT_1},
            //{"HDMI_SDA",      AT91C_PIN_PC(23),    0, PIO_DEFAULT, PIO_OUTPUT_1},
            //{"HDMI_SCL",      AT91C_PIN_PC(24),    0, PIO_DEFAULT, PIO_OUTPUT_1},
        };
        pio_setup(hw_pio_1, sizeof(hw_pio_1)/sizeof(struct pio_desc));

        AT91F_PIO_CfgOutput( AT91C_BASE_PIOC, 1<<30 );
        AT91F_PIO_SetOutput( AT91C_BASE_PIOC, 1<<30 );
        Sleep(500);

        AT91F_PIO_ClearOutput( AT91C_BASE_PIOC, 1<<30 );
        Sleep(500);

        AT91F_PIO_SetOutput( AT91C_BASE_PIOC, 1<<30 );
        Sleep(500);

DBGLINE(1,"\n");
        AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;
        //#ifdef __EK_BD__
        //pPMC->PMC_PCER0 = 1<<SAMA5D3_ID_PIOA ;
        //#else
        pPMC->PMC_PCER0 = 1<<SAMA5D3_ID_PIOC ;
        //#endif

DBGLINE(1,"\n");
        /* inter the TPI mode and scan the 9022 */
        BYTE data = 0;
        while(1)
        {

DBGLINE(1,"\n");
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, SIL9022A_TPI_MODE_REG, 0); 
            data = Sil9022a_Read( SIL9022A_SLAVE_ADDRESS, SIL9022A_DEVICE_ID_REG);
            if (data!= 0) break;
            //Sleep(100);
        }
        if(SIL9022A_DEVICE_ID_VAL != data)
        {
            kprintf("-E- Sil9022a not found!\n\r");
            //while(1);
        }    
DBGLINE(1,"\n");

        /* enable interrupt */
        Sil9022a_EnableInterrupt( SIL9022A_SLAVE_ADDRESS, HOT_PLUG_EVENT);
DBGLINE(1,"\n");

        #if 0
        /* read the TV status */
        if (Sil9022a_DoEdidRead() != EDID_OK)
        {
            kprintf("-E- EDID info parse error.\n\r");
            while(1);
        }
        #endif
                    extern GLOBAL_EDID            g_edid;
                    g_edid.HDMI_Sink = FALSE;
                    g_edid.YCbCr_4_4_4 = FALSE;
                    g_edid.YCbCr_4_2_2 = FALSE;
                    g_edid.CEC_A_B = 0x00;
                    g_edid.CEC_C_D = 0x00;
    
        kprintf ("-I- Detecting HDMI Sink/DVI... \n\r");
        /* select output mode (HDMI/DVI) according to sink capabilty */
        if (Sil9022a_HDMI_isSignatured())
        {
            kprintf (("-I- HDMI Sink Detected\n\r"));
            Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, OUTPUT_MODE_MASK, OUTPUT_MODE_HDMI);
        }
        else
        {
            kprintf (("DVI Sink Detected\n\r"));
            Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, OUTPUT_MODE_MASK, OUTPUT_MODE_DVI);
        }

        Sil9022a_ReadModifyWriteIndexedRegister(SIL9022A_SLAVE_ADDRESS,INDEXED_PAGE_0, 0x0A, 0x08, 0x08);

        /* Rx power up and Config chip  */
        Sil9022a_OnDownstreamRxPoweredUp();
    }

#endif
    
#ifdef _JBOOT_DISPLAY_INIT_

DBGLINE(DBG_LVL_LCD+1,"pSharedMemory->BootMode = %x\n", pSharedMemory->BootMode);
if( pSharedMemory->BootMode ==  WARM_BOOTMODE )
{
    #if 0
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;
    pPMC->PMC_PCER1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    pPMC->PMC_SCER = (0x1u << 3);

    DBGLINE(DBG_LVL_LCD+1,"pPMC->PMC_PCSR1 = %x\n", pPMC->PMC_PCSR1);
    DBGLINE(DBG_LVL_LCD+1,"pPMC->PMC_SCSR = %x\n", pPMC->PMC_SCSR);

    GD_Contrast( DEFAULT_BRIGHT_LEVEL );
    #endif

}
else
{
    PVOID pBuffer;
    pBuffer = (PVOID)(pChannelInfo->Intf.Addr + DMA_ADDR_OFFSET);
    {
    int i;
    PUINT32 q = (PUINT32)pBuffer;
    PUINT32 p = (PUINT32)0x7000000; // ref the jboot....

    for(i = 0;i < DISPLAY_WIDTH*DISPLAY_HEIGHT;i++)
        *q++ = *p++;
    }

    pBuffer = (PVOID)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET);
    {
    int i;
    PUINT32 q = (PUINT32)pBuffer;
    PUINT32 p = (PUINT32)0x7000000; // ref the jboot....

    for(i = 0;i < DISPLAY_WIDTH*DISPLAY_HEIGHT;i++)
        *q++ = *p++;
    }

    sLayer            *pLD   = pLayer(LCDD_BASE);
    sLCDCDescriptor   *pTD   = &pLD->dmaD;
    volatile UINT32 *pDmaR   = pHeadReg(LCDD_BASE);

    // wait ...
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    //while( !(pLCDC->LCDC_BASEISR & LCDC_BASEISR_DMA) );
    volatile UINT32 Temp = pLCDC->LCDC_BASEISR & LCDC_BASEISR_DMA;
    while( !(pLCDC->LCDC_BASEISR & LCDC_BASEISR_DMA) );

    LCDD_SetDMA(pBuffer, pTD, (UINT32)pDmaR);
}

    
#else
    STATIC BOOL bDisplayInited = FALSE;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;



    if( !bDisplayInited )
    {

//        DEBUGMSG(DBG_LVL_LCD,"%s(%d): START\n",__FUNCTION__,__LINE__ );

        LCDD_Initialize();
//        DEBUGMSG(DBG_LVL_LCD,"%s(%d): START\n",__FUNCTION__,__LINE__ );

        LCDD_On();
//        DEBUGMSG(DBG_LVL_LCD,"%s(%d): START\n",__FUNCTION__,__LINE__ );

        // enable layer
        DisplayEnableLayer( pChannelInfo );

        BackLightInit();
        //GD_BackLight( TRUE );
        DEBUGMSG(DBG_LVL_LCD,"%s(%d): START\n",__FUNCTION__,__LINE__ );

        // Initialize the contrast
        //GD_Contrast( DEFAULT_BRIGHT_LEVEL );
        bDisplayInited = TRUE;
    }


    DEBUGMSG(DBG_LVL_LCD,"%s(%d): START\n",__FUNCTION__,__LINE__ );

#if 0
//=========================================================
if( pChannelInfo->Channel == CHANNEL_BASE)
{
    #include "dooroos_logo.c"
    int i,j,k;
    PUINT32 q = (PUINT32)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET);

    for(i = 0;i < DISPLAY_WIDTH*DISPLAY_HEIGHT;i++)
        *q++ = 0x00FFFFFF;

    PBYTE   p;
    
    p = (PBYTE)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET);
    k = ((DISPLAY_STRIDE  - Image_dooroos_logo.Width*4) / 2)&~3;
    k += ((DISPLAY_HEIGHT - Image_dooroos_logo.Height)/2) * DISPLAY_STRIDE;
    p += k;
    k = 0;
    for(i = 0;i < Image_dooroos_logo.Height;i++)
    {
        for(j = 0;j < Image_dooroos_logo.Width;j++)
        {
            *p++ = Image[k++];
            *p++ = Image[k++];
            *p++ = Image[k++];
            *p++ = 0;
        }

        p += (DISPLAY_STRIDE - Image_dooroos_logo.Width*4);
        k += (Image_dooroos_logo.nStride - Image_dooroos_logo.Width*Image_dooroos_logo.nBytePerPixel);
    }
}
else
{
#if 1//AC fill yellow to LCD
    int i;
    PUINT32 p;

    p = (PUINT32)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET);
    for(i = 0;i < DISPLAY_STRIDE*DISPLAY_HEIGHT/4;i++)
        *p++ = 0x00FF00FF;  // transparent color
//        *p++ = 0x80FFFF00;

//    p = (PUINT32)(pChannelInfo->Frame.Addr + DMA_ADDR_OFFSET);
//    for(i = 0;i < DISPLAY_STRIDE*DISPLAY_HEIGHT/8;i++)
//        *p++ = 0xFF00FF;
#endif
}
#endif

#endif
    return TRUE;
}

PUBLIC VOID
DisplayEnableTransparent( CHANNELINFO *pChannelInfo )
{
    BYTE    Layer;
    switch( pChannelInfo->Channel )
    {
    case CHANNEL_BASE:
        Layer = LCDD_BASE;
        break;
    case CHANNEL_POPUP:
        Layer = LCDD_OVR1;
        break;
    case CHANNEL_VIDEO:
        Layer = LCDD_HEO;
        break;
    case CHANNEL_POPUP2:
        Layer = LCDD_OVR2;
        break;
    case CHANNEL_CURSOR:
        Layer = LCDD_CUR;
        break;
    default:
        return;
    }

    if( pChannelInfo->bTransparent )
        LCDD_SetColorKeying( Layer, FALSE, (BYTE)pChannelInfo->cTransparent, 0xFFFFFF );
    else
        LCDD_DisableColorKeying( Layer );

    return;
}

PUBLIC VOID
DisplayEnableAlpha( CHANNELINFO *pChannelInfo )
{
    BYTE    Layer;
    switch( pChannelInfo->Channel )
    {
    case CHANNEL_BASE:
        Layer = LCDD_BASE;
        break;
    case CHANNEL_POPUP:
        Layer = LCDD_OVR1;
        break;
    case CHANNEL_VIDEO:
        Layer = LCDD_HEO;
        break;
    case CHANNEL_POPUP2:
        Layer = LCDD_OVR2;
        break;
    case CHANNEL_CURSOR:
        Layer = LCDD_CUR;
        break;
    default:
        return;
    }

    if( pChannelInfo->bAlpha )
    {
        LCDD_SetAlpha( Layer, FALSE, (BYTE)pChannelInfo->Alpha );
        LCDD_EnableAlpha( Layer, TRUE, TRUE );
    }
    else
    {
        LCDD_EnableAlpha( Layer, FALSE, FALSE );
    }
        

    return;
}


PUBLIC VOID
GD_Contrast( UINT32 nContrast )
{
#ifdef __USE_PIO_BL__
	AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
#define IOCTRL_HAL_BL                    ( 254|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))

    nContrast = 100;
    if( nContrast < 0x10 )
    {
        pLCDC->LCDC_LCDDIS = (3<<2);
        pio_set_value(AT91C_PIN_PA(24), 0 );
    }
    else
    {
        pLCDC->LCDC_LCDEN = (3<<2);
        pio_set_value(AT91C_PIN_PA(24), 1 );
    }

    nContrast = nContrast/26;
    KernelIoControl( IOCTRL_HAL_BL, &nContrast, sizeof(nContrast), NULL, NULL );

#else
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    UINT32 cfg = pLCDC->LCDC_LCDCFG6 & ~LCDC_LCDCFG6_PWMCVAL_Msk;

    if( nContrast < 0x10 )
        pLCDC->LCDC_LCDDIS = (3<<2);
    else
        pLCDC->LCDC_LCDEN = (3<<2);

        
/*
    const struct pio_desc hw_pio[] = {
        {"EN",    AT91C_PIN_PA(24),    1, PIO_DEFAULT, PIO_PERIPH_A}
    };

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));
*/
    pLCDC->LCDC_LCDCFG6 = cfg | LCDC_LCDCFG6_PWMCVAL(nContrast);
#endif
    return;
}

//
//PUBLIC VOID
//GD_BackLight( UINT32 nBaclight )
//{
//    pio_set_value(AT91C_PIN_PC(26), nBaclight );
//
//    return;
//}


PUBLIC VOID
GD_PowerOff( VOID )
{
    return;
}


PUBLIC VOID
GD_PowerOn( VOID )
{
    return;
}


PUBLIC VOID SetVideoBuffer( PVOID pBuffer )
{
    sLayer            *pLD   = pLayer(LCDD_HEO);
    sLCDCDescriptor   *pTD   = &pLD->dmaD;
    volatile UINT32 *pDmaR = pHeadReg(LCDD_HEO);

    LCDD_SetDMA(pBuffer, pTD, (UINT32)pDmaR);
}


