
#ifndef _DRV_DISPLAY_H_
#define _DRV_DISPLAY_H_

#include <mach/sama5d3Xbd.h>
#include <mach/hal_interrupt.h>

    #define DISPLAY_WIDTH               800
    #define DISPLAY_HEIGHT              600
    /** Frame rate in Hz. */
    #define BOARD_LCD_FRAMERATE         40

    /** Vertical front porch in number of lines. */
    #define BOARD_LCD_TIMING_VFP        12 //22
    /** Vertical back porch in number of lines. */
    #define BOARD_LCD_TIMING_VBP        23 //21
    /** Vertical pulse width in number of lines. */
    #define BOARD_LCD_TIMING_VPW        10
    /** Horizontal front porch in LCDDOTCLK cycles. */
    #define BOARD_LCD_TIMING_HFP        64
    /** Horizontal back porch in LCDDOTCLK cycles. */
    #define BOARD_LCD_TIMING_HBP        46
    /** Horizontal pulse width in LCDDOTCLK cycles. */
    #define BOARD_LCD_TIMING_HPW        20

#define DISPLAY_BPP                 (32)
#define DISPLAY_STRIDE              (DISPLAY_WIDTH<<2)

//====================================================================

#define DMA_ADDR_OFFSET         0x20000000

/** Frequency of the board main oscillator */
#define BOARD_MAINOSC           12000000

/** Master clock frequency (when using board_lowlevel.c) */
//#define BOARD_MCK                ((unsigned long)((BOARD_MAINOSC / 3 / 2 / 3) * 200 ))


#if 0

/** Display interface width in bits. */
#define BOARD_LCD_IFWIDTH           24
/** Frame size in words (height * width * bpp / 32) */
#define BOARD_LCD_FRAMESIZE         (BOARD_LCD_WIDTH * BOARD_LCD_HEIGHT * BOARD_LCD_IFWIDTH / 32)

/** Vertical front porch in number of lines. */
#define BOARD_LCD_TIMING_VFP        22
/** Vertical back porch in number of lines. */
#define BOARD_LCD_TIMING_VBP        21
/** Vertical pulse width in number of lines. */
#define BOARD_LCD_TIMING_VPW        2
/** Horizontal front porch in LCDDOTCLK cycles. */
#define BOARD_LCD_TIMING_HFP        64
/** Horizontal back porch in LCDDOTCLK cycles. */
#define BOARD_LCD_TIMING_HBP        64
/** Horizontal pulse width in LCDDOTCLK cycles. */
#define BOARD_LCD_TIMING_HPW        128

/** Frame rate in Hz. */
#define BOARD_LCD_FRAMERATE         40
#endif

/** Pixel clock rate in Hz (HS period * VS period * BOARD_LCD_FRAMERATE). */
#define BOARD_LCD_PIXELCLOCK        ((BOARD_LCD_TIMING_HPW+BOARD_LCD_TIMING_HBP+DISPLAY_WIDTH+BOARD_LCD_TIMING_HFP)*(BOARD_LCD_TIMING_VPW+BOARD_LCD_TIMING_VBP+DISPLAY_HEIGHT+BOARD_LCD_TIMING_VFP)*BOARD_LCD_FRAMERATE)


// LCD Controller structure
typedef struct 
{
	UINT16	vl_col;		/* Number of columns (i.e. 640) */
	UINT16	vl_row;		/* Number of rows (i.e. 480) */
	UINT16	vl_width;	/* Width of display area in millimeters */
	UINT16	vl_height;	/* Height of display area in millimeters */
    UINT32	vl_pixclock;	/* pixel clock in ps	*/

	/* LCD configuration register */
	BYTE	vl_clkp;	/* Clock polarity */
	BYTE	vl_oep;		/* Output Enable polarity */
	BYTE	vl_hsp;		/* Horizontal Sync polarity */
	BYTE	vl_vsp;		/* Vertical Sync polarity */
	BYTE	vl_dp;		/* Data polarity */
	BYTE	vl_bpix;	/* Bits per pixel, 0 = 1, 1 = 2, 2 = 4, 3 = 8, 4 = 16 */
	BYTE	vl_lbw;		/* LCD Bus width, 0 = 4, 1 = 8 */
	BYTE	vl_splt;	/* Split display, 0 = single-scan, 1 = dual-scan */
	BYTE	vl_clor;	/* Color, 0 = mono, 1 = color */
	BYTE	vl_tft;		/* 0 = passive, 1 = TFT */

	/* Horizontal control register. Timing from data sheet */
	UINT16	vl_hpw;		/* Horz sync pulse width */
	BYTE	vl_blw;		/* Wait before of line */
	BYTE	vl_elw;		/* Wait end of line */

	/* Vertical control register. */
	BYTE	vl_vpw;		/* Vertical sync pulse width */
	BYTE	vl_bfw;		/* Wait before of frame */
	BYTE	vl_efw;		/* Wait end of frame */
} PANNEL_INFO, *PANNEL_PINFO;


typedef struct {
    UINT32 addr;
    UINT32 ctrl;
    UINT32 next;
}LCDC_DESC, *LCDC_PDESC;



typedef struct _CHANNELINFO_
{
    UINT32  Channel;
    BOOL    bOn;
    BOOL    bTransparent;
    BOOL    bAlpha;
    UINT32  cTransparent;   // transparent color
    UINT32  cBase;          // base color
    UINT32  Alpha;
    INT16   xOff;
    INT16   yOff;
    INT16   Width;
    INT16   Height;
    UINT32  RotAngle;
    SDI     Intf;
    SDI     Frame;
    RECT    Rect;
    UINT32  Rsved[4];       // reserved for next use
}CHANNELINFO;

#define YUV_BPP (33)

#endif // _DRV_DISPLAY_H_

































