/* ----------------------------------------------------------------------------
 *         SAM Software Package License
 * ----------------------------------------------------------------------------
 * Copyright (c) 2012, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */

/** \file */

#ifdef _WITH_HDMI_

/*----------------------------------------------------------------------------
 *        Headers
 *----------------------------------------------------------------------------*/
//#include <board.h>
#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>
#include <dev/i2cdrv.h> 
#include <message.h> 

#include "sil9022a.h"

SIL9022A_CONFIG        siHdmiTx;
GLOBAL_SYSTEM          g_sys;
GLOBAL_EDID            g_edid;
UINT8 g_CommData [EDID_BLOCK_SIZE];
UINT8                 tpivmode[3];  // saved TPI Reg0x08/Reg0x09/Reg0x0A values.

struct VModeInfoType VModesTable[] =
{
    //===================================================================================================================================================================================================================================
    //         VIC                  Refresh type Refresh-Rate Pixel-Totals  Position     Active     Aspect   Int  Length          Hbit  Vbit  Field  Pixel          Vact Space/Blank
    //        1   2  SubM   PixClk  V/H Position       VFreq   H      V      H    V       H    V    Ratio    Adj  H   V  Top  Dly HSync VSync Offset Repl  Space1 Space2 Blank1 Blank2 Blank3  3D
    //===================================================================================================================================================================================================================================
    {{        1,  0, NSM},  2517,  {ProgrVNegHNeg,     6000, { 800,  525}}, {144, 35}, { 640, 480}, R_4,     {0,  96, 2, 33,  48,  16,  10,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 0 - 1.       640  x 480p @ 60 VGA
    {{        2,  3, NSM},  2700,  {ProgrVNegHNeg,     6000, { 858,  525}}, {122, 36}, { 720, 480}, R_4or16, {0,  62, 6, 30,  60,  19,   9,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 1 - 2,3      720  x 480p
    {{        4,  0, NSM},  7425,  {ProgrVPosHPos,     6000, {1650,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 110,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 2 - 4        1280 x 720p@60Hz
    {{        5,  0, NSM},  7425,  {InterlaceVPosHPos, 6000, {2200,  562}}, {192, 20}, {1920,1080}, R_16,    {0,  44, 5, 15, 148,  88,   2, 1100},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 3 - 5        1920 x 1080i
    {{        6,  7, NSM},  2700,  {InterlaceVNegHNeg, 6000, {1716,  264}}, {119, 18}, { 720, 480}, R_4or16, {3,  62, 3, 15, 114,  17,   5,  429},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 4 - 6,7      1440 x 480i,pix repl
    {{        8,  9,   1},  2700,  {ProgrVNegHNeg,     6000, {1716,  262}}, {119, 18}, {1440, 240}, R_4or16, {0, 124, 3, 15, 114,  38,   4,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 5 - 8,9(1)   1440 x 240p
    {{        8,  9,   2},  2700,  {ProgrVNegHNeg,     6000, {1716,  263}}, {119, 18}, {1440, 240}, R_4or16, {0, 124, 3, 15, 114,  38,   4,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 6 - 8,9(2)   1440 x 240p
    {{       10, 11, NSM},  5400,  {InterlaceVNegHNeg, 6000, {3432,  525}}, {238, 18}, {2880, 480}, R_4or16, {0, 248, 3, 15, 228,  76,   4, 1716},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 7 - 10,11    2880 x 480i
    {{       12, 13,   1},  5400,  {ProgrVNegHNeg,     6000, {3432,  262}}, {238, 18}, {2880, 240}, R_4or16, {0, 248, 3, 15, 228,  76,   4,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 8 - 12,13(1) 2880 x 240p
    {{       12, 13,   2},  5400,  {ProgrVNegHNeg,     6000, {3432,  263}}, {238, 18}, {2880, 240}, R_4or16, {0, 248, 3, 15, 228,  76,   4,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 9 - 12,13(2) 2880 x 240p
    {{       14, 15, NSM},  5400,  {ProgrVNegHNeg,     6000, {1716,  525}}, {244, 36}, {1440, 480}, R_4or16, {0, 124, 6, 30, 120,  32,   9,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 10 - 14,15    1440 x 480p
    {{       16,  0, NSM}, 14835,  {ProgrVPosHPos,     6000, {2200, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148,  88,   4,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 11 - 16       1920 x 1080p
    {{       17, 18, NSM},  2700,  {ProgrVNegHNeg,     5000, { 864,  625}}, {132, 44}, { 720, 576}, R_4or16, {0,  64, 5, 39,  68,  12,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 12 - 17,18    720  x 576p
    {{       19,  0, NSM},  7425,  {ProgrVPosHPos,     5000, {1980,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 440,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 13 - 19       1280 x 720p@50Hz
    {{       20,  0, NSM},  7425,  {InterlaceVPosHPos, 5000, {2640, 1125}}, {192, 20}, {1920,1080}, R_16,    {0,  44, 5, 15, 148, 528,   2, 1320},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 14 - 20       1920 x 1080i
    {{       21, 22, NSM},  2700,  {InterlaceVNegHNeg, 5000, {1728,  625}}, {132, 22}, { 720, 576}, R_4,     {3,  63, 3, 19, 138,  24,   2,  432},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 15 - 21,22    1440 x 576i
    {{       23, 24,   1},  2700,  {ProgrVNegHNeg,     5000, {1728,  312}}, {132, 22}, {1440, 288}, R_4or16, {0, 126, 3, 19, 138,  24,   2,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 16 - 23,24(1) 1440 x 288p
    {{       23, 24,   2},  2700,  {ProgrVNegHNeg,     5000, {1728,  313}}, {132, 22}, {1440, 288}, R_4or16, {0, 126, 3, 19, 138,  24,   2,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 17 - 23,24(2) 1440 x 288p
    {{       23, 24,   3},  2700,  {ProgrVNegHNeg,     5000, {1728,  314}}, {132, 22}, {1440, 288}, R_4or16, {0, 126, 3, 19, 138,  24,   2,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 18 - 23,24(3) 1440 x 288p
    {{       25, 26, NSM},  5400,  {InterlaceVNegHNeg, 5000, {3456,  625}}, {264, 22}, {2880, 576}, R_4or16, {0, 252, 3, 19, 276,  48,   2, 1728},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 19 - 25,26    2880 x 576i
    {{       27, 28,   1},  5400,  {ProgrVNegHNeg,     5000, {3456,  312}}, {264, 22}, {2880, 288}, R_4or16, {0, 252, 3, 19, 276,  48,   2,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 20 - 27,28(1) 2880 x 288p
    {{       27, 28,   2},  5400,  {ProgrVNegHNeg,     5000, {3456,  313}}, {264, 22}, {2880, 288}, R_4or16, {0, 252, 3, 19, 276,  48,   3,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 21 - 27,28(2) 2880 x 288p
    {{       27, 28,   3},  5400,  {ProgrVNegHNeg,     5000, {3456,  314}}, {264, 22}, {2880, 288}, R_4or16, {0, 252, 3, 19, 276,  48,   4,    0},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 22 - 27,28(3) 2880 x 288p
    {{       29, 30, NSM},  5400,  {ProgrVPosHNeg,     5000, {1728,  625}}, {264, 44}, {1440, 576}, R_4or16, {0, 128, 5, 39, 136,  24,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 23 - 29,30    1440 x 576p
    {{       31,  0, NSM}, 14850,  {ProgrVPosHPos,     5000, {2640, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 528,   4,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 24 - 31(1)    1920 x 1080p
    {{       32,  0, NSM},  7417,  {ProgrVPosHPos,     2400, {2750, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 638,   4,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 25 - 32(2)    1920 x 1080p@24Hz
    {{       33,  0, NSM},  7425,  {ProgrVPosHPos,     2500, {2640, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 528,   4,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 26 - 33(3)    1920 x 1080p
    {{       34,  0, NSM},  7417,  {ProgrVPosHPos,     3000, {2200, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 528,   4,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 27 - 34(4)    1920 x 1080p
    {{       35, 36, NSM}, 10800,  {ProgrVNegHNeg,     5994, {3432,  525}}, {488, 36}, {2880, 480}, R_4or16, {0, 248, 6, 30, 240,  64,  10,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 28 - 35, 36   2880 x 480p@59.94/60Hz
    {{       37, 38, NSM}, 10800,  {ProgrVNegHNeg,     5000, {3456,  625}}, {272, 39}, {2880, 576}, R_4or16, {0, 256, 5, 40, 272,  48,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 29 - 37, 38   2880 x 576p@50Hz
    {{       39,  0, NSM},  7200,  {InterlaceVNegHNeg, 5000, {2304, 1250}}, {352, 62}, {1920,1080}, R_16,    {0, 168, 5, 87, 184,  32,  24,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 30 - 39       1920 x 1080i@50Hz
    {{       40,  0, NSM}, 14850,  {InterlaceVPosHPos, 10000,{2640, 1125}}, {192, 20}, {1920,1080}, R_16,    {0,  44, 5, 15, 148, 528,   2, 1320},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 31 - 40       1920 x 1080i@100Hz
    {{       41,  0, NSM}, 14850,  {InterlaceVPosHPos, 10000,{1980,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 400,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 32 - 41       1280 x 720p@100Hz
    {{       42, 43, NSM},  5400,  {ProgrVNegHNeg,     10000,{ 864,  144}}, {132, 44}, { 720, 576}, R_4or16, {0,  64, 5, 39,  68,  12,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 33 - 42, 43,  720p x 576p@100Hz
    {{       44, 45, NSM},  5400,  {InterlaceVNegHNeg, 10000,{ 864,  625}}, {132, 22}, { 720, 576}, R_4or16, {0,  63, 3, 19,  69,  12,   2,  432},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 34 - 44, 45,  720p x 576i@100Hz, pix repl
    {{       46,  0, NSM}, 14835,  {InterlaceVPosHPos, 11988,{2200, 1125}}, {192, 20}, {1920,1080}, R_16,    {0,  44, 5, 15, 149,  88,   2, 1100},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 35 - 46,      1920 x 1080i@119.88/120Hz
    {{       47,  0, NSM}, 14835,  {ProgrVPosHPos,     11988,{1650,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 110,   5, 1100},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 36 - 47,      1280 x 720p@119.88/120Hz
    {{       48, 49, NSM},  5400,  {ProgrVNegHNeg,     11988,{ 858,  525}}, {122, 36}, { 720, 480}, R_4or16, {0,  62, 6, 30,  60,  16,  10,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 37 - 48, 49   720  x 480p@119.88/120Hz
    {{       50, 51, NSM},  5400,  {InterlaceVNegHNeg, 11988,{ 858,  525}}, {119, 18}, { 720, 480}, R_4or16, {0,  62, 3, 15,  57,  19,   4,  429},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 38 - 50, 51   720  x 480i@119.88/120Hz
    {{       52, 53, NSM}, 10800,  {ProgrVNegHNeg,     20000,{ 864,  625}}, {132, 44}, { 720, 576}, R_4or16, {0,  64, 5, 39,  68,  12,   5,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 39 - 52, 53,  720  x 576p@200Hz
    {{       54, 55, NSM}, 10800,  {InterlaceVNegHNeg, 20000,{ 864,  625}}, {132, 22}, { 720, 576}, R_4or16, {0,  63, 3, 19,  69,  12,   2,  432},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 40 - 54, 55,  1440 x 576i @200Hz, pix repl
    {{       56, 57, NSM}, 10800,  {ProgrVNegHNeg,     24000,{ 858,  525}}, {122, 42}, { 720, 480}, R_4or16, {0,  62, 6, 30,  60,  16,   9,    0},    0,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 41 - 56, 57,  720  x 480p @239.76/240Hz
    {{       58, 59, NSM}, 10800,  {InterlaceVNegHNeg, 24000,{ 858,  525}}, {119, 18}, { 720, 480}, R_4or16, {0,  62, 3, 15,  57,  19,   4,  429},    1,    {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 42 - 58, 59,  1440 x 480i @239.76/240Hz, pix repl

/** 4K x 2K Modes: */
    //===================================================================================================================================================================================================================================
    //                                                                                                            Pulse
    //         VIC                  Refresh type Refresh-Rate Pixel-Totals   Position     Active    Aspect   Int  Width           Hbit  Vbit  Field  Pixel          Vact Space/Blank
    //        1   2  SubM   PixClk  V/H Position       VFreq   H      V      H    V       H    V    Ratio    Adj  H   V  Top  Dly HSync VSync Offset Repl  Space1 Space2 Blank1 Blank2 Blank3  3D
    //===================================================================================================================================================================================================================================
    {{        1,  0, NSM}, 297000, {ProgrVNegHNeg,     30000,{4400, 2250}}, {384, 82}, {3840,2160}, R_16,    {0,  88, 10, 72, 296, 176,  8,    0},   0,     {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 43 - 4k x 2k 29.97/30Hz (297.000 MHz)
    {{        2,  0, NSM}, 297000, {ProgrVNegHNeg,     29700,{5280, 2250}}, {384, 82}, {3840,2160}, R_16,    {0,  88, 10, 72, 296,1056,  8,    0},   0,     {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 44 - 4k x 2k 25Hz
    {{        3,  0, NSM}, 297000, {ProgrVNegHNeg,     24000,{5500, 2250}}, {384, 82}, {3840,2160}, R_16,    {0,  88, 10, 72, 296,1276,  8,    0},   0,     {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 45 - 4k x 2k 24Hz (297.000 MHz)
    {{        4  ,0, NSM}, 297000, {ProgrVNegHNeg,     24000,{6500, 2250}}, {384, 82}, {4096,2160}, R_16,    {0,  88, 10, 72, 296,1020,  8,    0},   0,     {0,     0,     0,     0,    0},    NO_3D_SUPPORT}, // 46 - 4k x 2k 24Hz (SMPTE)

// 3D Modes:
    //===================================================================================================================================================================================================================================
    //                                                                                                            Pulse
    //         VIC                  Refresh type Refresh-Rate Pixel-Totals  Position      Active    Aspect   Int  Width           Hbit  Vbit  Field  Pixel          Vact Space/Blank
    //        1   2  SubM   PixClk  V/H Position       VFreq   H      V      H    V       H    V    Ratio    Adj  H   V  Top  Dly HSync VSync Offset Repl  Space1 Space2 Blank1 Blank2 Blank3  3D
    //===================================================================================================================================================================================================================================
    {{        2,  3, NSM},  2700,  {ProgrVPosHPos,     6000, {858,   525}}, {122, 36}, { 720, 480}, R_4or16, {0,  62, 6, 30,  60,  16,   9,    0},    0,    {0,     0,     0,     0,    0},     8}, // 47 - 3D, 2,3 720p x 480p /60Hz, Side-by-Side (Half)
    {{        4,  0, NSM}, 14850,  {ProgrVPosHPos,     6000, {1650,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 110,   5,    0},    0,    {0,     0,     0,     0,    0},     0}, // 48 - 3D  4   1280 x 720p@60Hz,  Frame Packing
    {{        5,  0, NSM}, 14850,  {InterlaceVPosHPos, 6000, {2200,  562}}, {192, 20}, {1920, 540}, R_16,    {0,  44, 5, 15, 148,  88,   2, 1100},    0,    {23,   22,     0,     0,    0},     0}, // 49 - 3D, 5,  1920 x 1080i/60Hz, Frame Packing
    {{        5,  0, NSM}, 14850,  {InterlaceVPosHPos, 6000, {2200,  562}}, {192, 20}, {1920, 540}, R_16,    {0,  44, 5, 15, 148,  88,   2, 1100},    0,    {0,     0,    22,    22,   23},     1}, // 50 - 3D, 5,  1920 x 1080i/60Hz, Field Alternative
    {{       16,  0, NSM}, 29700,  {ProgrVPosHPos,     6000, {2200, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148,  88,   4,    0},    0,    {0,     0,     0,     0,    0},     0}, // 51 - 3D, 16, 1920 x 1080p/60Hz, Frame Packing
    {{       16,  0, NSM}, 29700,  {ProgrVPosHPos,     6000, {2200, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148,  88,   4,    0},    0,    {0,     0,     0,     0,    0},     2}, // 52 - 3D, 16, 1920 x 1080p/60Hz, Line Alternative
    {{       16,  0, NSM}, 29700,  {ProgrVPosHPos,     6000, {2200, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148,  88,   4,    0},    0,    {0,     0,     0,     0,    0},     3}, // 53 - 3D, 16, 1920 x 1080p/60Hz, Side-by-Side (Full)
    {{       16,  0, NSM}, 14850,  {ProgrVPosHPos,     6000, {2200, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148,  88,   4,    0},    0,    {0,     0,     0,     0,    0},     8}, // 54 - 3D, 16, 1920 x 1080p/60Hz, Side-by-Side (Half)
    {{       19,  0, NSM}, 14850,  {ProgrVPosHPos,     5000, {1980,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 440,   5,    0},    0,    {0,     0,     0,     0,    0},     0}, // 55 - 3D, 19, 1280 x 720p@50Hz,  Frame Packing
    {{       19,  0, NSM}, 14850,  {ProgrVPosHPos,     5000, {1980,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 440,   5,    0},    0,    {0,     0,     0,     0,    0},     4}, // 56 - 3D, 19, 1280 x 720p/50Hz,  (L + depth)
    {{       19,  0, NSM}, 29700,  {ProgrVPosHPos,     5000, {1980,  750}}, {260, 25}, {1280, 720}, R_16,    {0,  40, 5, 20, 220, 440,   5,    0},    0,    {0,     0,     0,     0,    0},     5}, // 57 - 3D, 19, 1280 x 720p/50Hz,  (L + depth + Gfx + G-depth)
    {{       20,  0, NSM}, 14850,  {InterlaceVPosHPos, 5000, {2640,  562}}, {192, 20}, {1920, 540}, R_16,    {0,  44, 5, 15, 148, 528,   2, 1220},    0,    {23,   22,     0,     0,    0},     0}, // 58 - 3D, 20, 1920 x 1080i/50Hz, Frame Packing
    {{       20,  0, NSM}, 14850,  {InterlaceVPosHPos, 5000, {2640,  562}}, {192, 20}, {1920, 540}, R_16,    {0,  44, 5, 15, 148, 528,   2, 1220},    0,    {0,     0,    22,    22,   23},     1}, // 59 - 3D, 20, 1920 x 1080i/50Hz, Field Alternative
    {{       31,  0, NSM}, 29700,  {ProgrVPosHPos,     5000, {2640, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 528,   4,    0},    0,    {0,     0,     0,     0,    0},     0}, // 60 - 3D, 31, 1920 x 1080p/50Hz, Frame Packing
    {{       31,  0, NSM}, 29700,  {ProgrVPosHPos,     5000, {2640, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 528,   4,    0},    0,    {0,     0,     0,     0,    0},     2}, // 61 - 3D, 31, 1920 x 1080p/50Hz, Line Alternative
    {{       31,  0, NSM}, 29700,  {ProgrVPosHPos,     5000, {2650, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 528,   4,    0},    0,    {0,     0,     0,     0,    0},     3}, // 62 - 3D, 31, 1920 x 1080p/50Hz, Side-by-Side (Full)
    {{       32,  0, NSM}, 14850,  {ProgrVPosHPos,     2400, {2750, 1125}}, {192, 41}, {1920,1080}, R_16,    {0,  44, 5, 36, 148, 638,   4,    0},    0,    {0,     0,     0,     0,    0},     0}, // 63 - 3D, 32, 1920 x 1080p@24Hz, Frame Packing

// NOTE: DO NOT ATTEMPT INPUT RESOLUTIONS THAT REQUIRE PIXEL CLOCK FREQUENCIES HIGHER THAN THOSE SUPPORTED BY THE TRANSMITTER CHIP
    //===================================================================================================================================================================================================================================
    //                                                                                                              Sync Pulse
    //  VIC                          Refresh type   fresh-Rate  Pixel-Totals    Position    Active     Aspect   Int  Width            Hbit  Vbit  Field  Pixel          Vact Space/Blank
    // 1   2  SubM         PixClk    V/H Position       VFreq   H      V        H    V       H    V     Ratio   {Adj  H   V  Top  Dly HSync VSync Offset} Repl  Space1 Space2 Blank1 Blank2 Blank3  3D
    //===================================================================================================================================================================================================================================
    {{PC_BASE  , 0,NSM},    3150,   {ProgrVNegHPos,     8508,   {832, 445}},    {160,63},   {640,350},   R_16,  {0,  64,  3,  60,  96,  32,  32,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 64 - 640x350@85.08
    {{PC_BASE+1, 0,NSM},    3150,   {ProgrVPosHNeg,     8508,   {832, 445}},    {160,44},   {640,400},   R_16,  {0,  64,  3,  41,  96,  32,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 65 - 640x400@85.08
    {{PC_BASE+2, 0,NSM},    2700,   {ProgrVPosHNeg,     7008,   {900, 449}},    {0,0},      {720,400},   R_16,  {0,   0,  0,   0,   0,   0,   0,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 66 - 720x400@70.08
    {{PC_BASE+3, 0,NSM},    3500,   {ProgrVPosHNeg,     8504,   {936, 446}},    {20,45},    {720,400},   R_16,  {0,  72,  3,  42, 108,  36,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 67 - 720x400@85.04
    {{PC_BASE+4, 0,NSM},    2517,   {ProgrVNegHNeg,     5994,   {800, 525}},    {144,35},   {640,480},   R_4,   {0,  96,  2,  33,  48,  16,  10,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 68 - 640x480@59.94
    {{PC_BASE+5, 0,NSM},    3150,   {ProgrVNegHNeg,     7281,   {832, 520}},    {144,31},   {640,480},   R_4,   {0,  40,  3,  28, 128, 128,   9,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 69 - 640x480@72.80
    {{PC_BASE+6, 0,NSM},    3150,   {ProgrVNegHNeg,     7500,   {840, 500}},    {21,19},    {640,480},   R_4,   {0,  64,  3,  28, 128,  24,   9,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 70 - 640x480@75.00
    {{PC_BASE+7,0,NSM},     3600,   {ProgrVNegHNeg,     8500,   {832, 509}},    {168,28},   {640,480},   R_4,   {0,  56,  3,  25, 128,  24,   9,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 71 - 640x480@85.00
    {{PC_BASE+8,0,NSM},     3600,   {ProgrVPosHPos,     5625,   {1024, 625}},   {200,24},   {800,600},   R_4,   {0,  72,  2,  22, 128,  24,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 72 - 800x600@56.25
    {{PC_BASE+9,0,NSM},     4000,   {ProgrVPosHPos,     6032,   {1056, 628}},   {216,27},   {800,600},   R_4,   {0, 128,  4,  23,  88,  40,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 73 - 800x600@60.317
    {{PC_BASE+10,0,NSM},    5000,   {ProgrVPosHPos,     7219,   {1040, 666}},   {184,29},   {800,600},   R_4,   {0, 120,  6,  23,  64,  56,  37,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 74 - 800x600@72.19
    {{PC_BASE+11,0,NSM},    4950,   {ProgrVPosHPos,     7500,   {1056, 625}},   {240,24},   {800,600},   R_4,   {0,  80,  3,  21, 160,  16,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 75 - 800x600@75
    {{PC_BASE+12,0,NSM},    5625,   {ProgrVPosHPos,     8506,   {1048, 631}},   {216,30},   {800,600},   R_4,   {0,  64,  3,  27, 152,  32,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 76 - 800x600@85.06
    {{PC_BASE+13,0,NSM},    6500,   {ProgrVNegHNeg,     6000,   {1344, 806}},   {296,35},   {1024,768},  R_4,   {0, 136,  6,  29, 160,  24,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 77 - 1024x768@60
    {{PC_BASE+14,0,NSM},    7500,   {ProgrVNegHNeg,     7007,   {1328, 806}},   {280,35},   {1024,768},  R_4,   {0, 136,  6,  19, 144,  24,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 78 - 1024x768@70.07
    {{PC_BASE+15,0,NSM},    7875,   {ProgrVPosHPos,     7503,   {1312, 800}},   {272,31},   {1024,768},  R_4,   {0,  96,  3,  28, 176,  16,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 79 - 1024x768@75.03
    {{PC_BASE+16,0,NSM},    9450,   {ProgrVPosHPos,     8500,   {1376, 808}},   {304,39},   {1024,768},  R_4,   {0,  96,  3,  36, 208,  48,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 80 - 1024x768@85
    {{PC_BASE+17,0,NSM},   10800,   {ProgrVPosHPos,     7500,   {1600, 900}},   {384,35},   {1152,864},  R_4,   {0, 128,  3,  32, 256,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 81 - 1152x864@75
    {{PC_BASE+18,0,NSM},   16200,   {ProgrVPosHPos,     6000,   {2160, 1250}},  {496,49},   {1600,1200}, R_4,   {0, 304,  3,  46, 304,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 82 - 1600x1200@60
    {{PC_BASE+19,0,NSM},    6825,   {ProgrVNegHPos,     6000,   {1440, 790}},   {112,19},   {1280,768},  R_16,  {0,  32,  7,  12,  80,  48,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 83 - 1280x768@59.95
    {{PC_BASE+20,0,NSM},    7950,   {ProgrVPosHNeg,     5987,   {1664, 798}},   {320,27},   {1280,768},  R_16,  {0, 128,  7,  20, 192,  64,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 84 - 1280x768@59.87
    {{PC_BASE+21,0,NSM},   10220,   {ProgrVPosHNeg,     6029,   {1696, 805}},   {320,27},   {1280,768},  R_16,  {0, 128,  7,  27, 208,  80,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 85 - 1280x768@74.89
    {{PC_BASE+22,0,NSM},   11750,   {ProgrVPosHNeg,     8484,   {1712, 809}},   {352,38},   {1280,768},  R_16,  {0, 136,  7,  31, 216,  80,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 86 - 1280x768@85
    {{PC_BASE+23,0,NSM},   10800,   {ProgrVPosHPos,     6000,   {1800, 1000}},  {424,39},   {1280,960},  R_4,   {0, 112,  3,  36, 312,  96,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 87 - 1280x960@60
    {{PC_BASE+24,0,NSM},   14850,   {ProgrVPosHPos,     8500,   {1728, 1011}},  {384,50},   {1280,960},  R_4,   {0, 160,  3,  47, 224,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 88 - 1280x960@85
    {{PC_BASE+25,0,NSM},   10800,   {ProgrVPosHPos,     6002,   {1688, 1066}},  {360,41},   {1280,1024}, R_4,   {0, 112,  3,  38, 248,  48,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 89 - 1280x1024@60
    {{PC_BASE+26,0,NSM},   13500,   {ProgrVPosHPos,     7502,   {1688, 1066}},  {392,41},   {1280,1024}, R_4,   {0, 144,  3,  38, 248,  16,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 90 - 1280x1024@75
    {{PC_BASE+27,0,NSM},   15750,   {ProgrVPosHPos,     8502,   {1728, 1072}},  {384,47},   {1280,1024}, R_4,   {0, 160,  3,   4, 224,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 91 - 1280x1024@85
    {{PC_BASE+28,0,NSM},    8550,   {ProgrVPosHPos,     6002,   {1792, 795}},   {368,24},   {1360,768},  R_16,  {0, 112,  6,  18, 256,  64,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 92 - 1360x768@60
    {{PC_BASE+29,0,NSM},   10100,   {ProgrVNegHPos,     5995,   {1560, 1080}},  {112,27},   {1400,1050}, R_4,   {0,  32,  4,  23,  80,  48,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 93 - 1400x105@59.95
    {{PC_BASE+30,0,NSM},   12175,   {ProgrVPosHNeg,     5998,   {1864, 1089}},  {376,36},   {1400,1050}, R_4,   {0, 144,  4,  32, 232,  88,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 94 - 1400x105@59.98
    {{PC_BASE+31,0,NSM},   15600,   {ProgrVPosHNeg,     7487,   {1896, 1099}},  {392,46},   {1400,1050}, R_4,   {0, 144,  4,  22, 248, 104,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 95 - 1400x105@74.87
    {{PC_BASE+32,0,NSM},   17950,   {ProgrVPosHNeg,     8496,   {1912, 1105}},  {408,52},   {1400,1050}, R_4,   {0, 152,  4,  48, 256, 104,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 96 - 1400x105@84.96
    {{PC_BASE+33,0,NSM},   17550,   {ProgrVPosHPos,     6500,   {2160, 1250}},  {496,49},   {1600,1200}, R_4,   {0, 192,  3,  46, 304,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 97 - 1600x1200@65
    {{PC_BASE+34,0,NSM},   18900,   {ProgrVPosHPos,     7000,   {2160, 1250}},  {496,49},   {1600,1200}, R_4,   {0, 192,  3,  46, 304,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 98 - 1600x1200@70
    {{PC_BASE+35,0,NSM},   20250,   {ProgrVPosHPos,     7500,   {2160, 1250}},  {496,49},   {1600,1200}, R_4,   {0, 192,  3,  46, 304,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 99 - 1600x1200@75
    {{PC_BASE+36,0,NSM},   22950,   {ProgrVPosHPos,     8500,   {2160, 1250}},  {496,49},   {1600,1200}, R_4,   {0, 192,  3,  46, 304,  64,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 100 - 1600x1200@85
    {{PC_BASE+37,0,NSM},   20475,   {ProgrVPosHNeg,     6000,   {2448, 1394}},  {528,49},   {1792,1344}, R_4,   {0, 200,  3,  46, 328, 128,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 101 - 1792x1344@60
    {{PC_BASE+38,0,NSM},   26100,   {ProgrVPosHNeg,     7500,   {2456, 1417}},  {568,72},   {1792,1344}, R_4,   {0, 216,  3,  69, 352,  96,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 102 - 1792x1344@74.997
    {{PC_BASE+39,0,NSM},   21825,   {ProgrVPosHNeg,     6000,   {2528, 1439}},  {576,46},   {1856,1392}, R_4,   {0, 224,  3,  43, 352,  96,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 103 - 1856x1392@60
    {{PC_BASE+40,0,NSM},   28800,   {ProgrVPosHNeg,     7500,   {2560, 1500}},  {576,107},  {1856,1392}, R_4,   {0, 224,  3, 104, 352, 128,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 104 - 1856x1392@75
    {{PC_BASE+41,0,NSM},   15400,   {ProgrVNegHPos,     5995,   {2080, 1235}},  {112,32},   {1920,1200}, R_16,  {0,  32,  6,  26,  80,  48,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 106 - 1920x1200@59.95
    {{PC_BASE+42,0,NSM},   19325,   {ProgrVPosHNeg,     5988,   {2592, 1245}},  {536,42},   {1920,1200}, R_16,  {0, 200,  6,  36, 336, 136,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 107 - 1920x1200@59.88
    {{PC_BASE+43,0,NSM},   24525,   {ProgrVPosHNeg,     7493,   {2608, 1255}},  {552,52},   {1920,1200}, R_16,  {0, 208,  6,  46, 344, 136,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 108 - 1920x1200@74.93
    {{PC_BASE+44,0,NSM},   28125,   {ProgrVPosHNeg,     8493,   {2624, 1262}},  {560,59},   {1920,1200}, R_16,  {0, 208,  6,  53, 352, 144,   3,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 109 - 1920x1200@84.93
    {{PC_BASE+45,0,NSM},   23400,   {ProgrVPosHNeg,     6000,   {2600, 1500}},  {552,59},   {1920,1440}, R_4,   {0, 208,  3,  56, 344, 128,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 110 - 1920x1440@60
    {{PC_BASE+46,0,NSM},   29700,   {ProgrVPosHNeg,     7500,   {2640, 1500}},  {576,59},   {1920,1440}, R_4,   {0, 224,  3,  56, 352, 144,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 111 - 1920x1440@75
    {{PC_BASE+47,0,NSM},   24150,   {ProgrVPosHNeg,     6000,   {2720, 1481}},  {48,  3},   {2560,1440}, R_16,  {0,  32,  5,  56, 352, 144,   1,       0},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 112 - 2560x1440@60 // %%% need work
    {{PC_BASE+48,0,NSM},    2700,   {InterlaceVNegHNeg, 6000,   {1716,  264}},  {244,18},   {1440, 480},R_4or16,{3, 124,  3,  15, 114,  17,   5,     429},  0,  {0,     0,     0,     0,    0},   NO_3D_SUPPORT}, // 113 - 1440 x 480i 
};


/** Aspect Ratio table defines the aspect ratio as function of VIC. This table
    should be used in conjunction with the 861-D part of VModeInfoType VModesTable[]
   (formats 0 - 59) because some formats that differ only in their AR are grouped
   together (e.g., formats 2 and 3). */
STATIC UINT8 AspectRatioTable[] =
{
    R_4,   R_4, R_16, R_16, R_16,  R_4, R_16,  R_4, R_16,  R_4,
    R_16,  R_4, R_16,  R_4, R_16, R_16,  R_4, R_16, R_16, R_16,
    R_4,   R_16, R_4, R_16,  R_4, R_16,  R_4, R_16,  R_4, R_16,
    R_16,  R_16, R_16, R_16,  R_4, R_16,  R_4, R_16, R_16, R_16,
    R_16,  R_4, R_16,  R_4, R_16, R_16, R_16,  R_4, R_16,  R_4,
    R_16,  R_4, R_16,  R_4, R_16,  R_4, R_16,  R_4, R_16
};

/** VIC to Indexc table defines which VideoModeTable entry is appropreate for this VIC code. 
   Note: This table is valid ONLY for VIC codes in 861-D formats, NOT for HDMI_VIC codes
   or 3D codes! */
STATIC UINT8 VIC2Index[] =
{
    0,  0,  1,  1,  2,  3,  4,  4,  5,  5,
    7,  7,  8,  8, 10, 10, 11, 12, 12, 13,
    14, 15, 15, 16, 16, 19, 19, 20, 20, 23,
    23, 24, 25, 26, 27, 28, 28, 29, 29, 30,
    31, 32, 33, 33, 34, 34, 35, 36, 37, 37,
    38, 38, 39, 39, 40, 40, 41, 41, 42, 42
};


#if 0

//#define __EK_BD__
    #ifdef __EK_BD__
#define I2C_PIO_BASE                (AT91C_BASE_PIOA)
#define BIT_I2CSCL                    (AT91C_PIO_PA31)
#define BIT_I2CSDA                    (AT91C_PIO_PA30)
    #else
#define I2C_PIO_BASE                (AT91C_BASE_PIOC)
#define BIT_I2CSCL                    (AT91C_PIO_PC24)
#define BIT_I2CSDA                    (AT91C_PIO_PC23)
    #endif


#define PIN_HOLD_0      0
#define PIN_OFF         1
    

static void DELAY( INT32 Cnt )
{
    volatile INT32   i;
    for( i = 0 ; i < Cnt*100;  )
    {
        i++;
    }
}

static void
SetSCL( BOOL bHigh )
{
#if 1
    I2C_PIO_BASE->PIO_OER = BIT_I2CSCL;
    I2C_PIO_BASE->PIO_PER = BIT_I2CSCL;

    /* Enable pull-up(s) if necessary */
    I2C_PIO_BASE->PIO_PPDDR = BIT_I2CSCL;
    I2C_PIO_BASE->PIO_PPUER = BIT_I2CSCL;

    I2C_PIO_BASE->PIO_MDER = BIT_I2CSCL; // open drain
    I2C_PIO_BASE->PIO_IFDR = BIT_I2CSCL;  // glitch

    if( bHigh == PIN_OFF )
        I2C_PIO_BASE->PIO_SODR = BIT_I2CSCL;
    else
        I2C_PIO_BASE->PIO_CODR = BIT_I2CSCL;
#else
    I2C_PIO_BASE->PIO_MDER = BIT_I2CSCL;
    I2C_PIO_BASE->PIO_IFDR = BIT_I2CSCL;
    if( bHigh == PIN_OFF )
    {
        I2C_PIO_BASE->PIO_SODR = BIT_I2CSCL;   // set to 1

    }
    else
    {
        I2C_PIO_BASE->PIO_CODR = BIT_I2CSCL;   // set to 0
        
    }
        I2C_PIO_BASE->PIO_OER = BIT_I2CSCL;    //  output
        I2C_PIO_BASE->PIO_PER = BIT_I2CSCL;
#endif
    return;
}

static void
SetSDA( BOOL bHigh )
{
#if 1
    I2C_PIO_BASE->PIO_OER = BIT_I2CSDA;
    I2C_PIO_BASE->PIO_PER = BIT_I2CSDA;

    /* Enable pull-up(s) if necessary */
    I2C_PIO_BASE->PIO_PPDDR = BIT_I2CSDA;
    I2C_PIO_BASE->PIO_PPUER = BIT_I2CSDA;

    I2C_PIO_BASE->PIO_MDER = BIT_I2CSDA; // open drain
    I2C_PIO_BASE->PIO_IFDR = BIT_I2CSDA;  // glitch

    if( bHigh == PIN_OFF )
        I2C_PIO_BASE->PIO_SODR = BIT_I2CSDA;
    else
        I2C_PIO_BASE->PIO_CODR = BIT_I2CSDA;
#else
    if( bHigh == PIN_OFF )
    {
        /* Enable pull-up(s) i*/
        I2C_PIO_BASE->PIO_PPDDR = BIT_I2CSDA;
        I2C_PIO_BASE->PIO_PUER = BIT_I2CSDA;
        I2C_PIO_BASE->PIO_MDER = BIT_I2CSDA;

        // glitch
        I2C_PIO_BASE->PIO_IFDR = BIT_I2CSDA;

        /* Configure pin as input */
        I2C_PIO_BASE->PIO_ODR = BIT_I2CSDA;
        I2C_PIO_BASE->PIO_PER = BIT_I2CSDA;
    }
    else
    {
        I2C_PIO_BASE->PIO_CODR = BIT_I2CSDA;   // set to 0
        
        I2C_PIO_BASE->PIO_OER = BIT_I2CSDA;    //  output
        I2C_PIO_BASE->PIO_PER = BIT_I2CSDA;
    }
#endif
    return;
}


static void
Start_I2C( void )
{
//test();
        SetSCL(PIN_OFF);
        SetSDA(PIN_OFF);

    DELAY(10);
        SetSDA(PIN_HOLD_0);
    DELAY(30);
        SetSCL(PIN_HOLD_0);
    DELAY(20);
}

static void
Stop_I2C( void )
{
//        SetSCL(PIN_HOLD_0);
//        SetSDA(PIN_HOLD_0);
//    DELAY( 20 );
        SetSCL(PIN_OFF);
    DELAY( 20 );
        SetSDA(PIN_OFF);
    DELAY(20);
    
}


static BYTE
Read_I2C_Data( void )
{
    BYTE Data = 0;
    UINT16 wClockCnt=0, wBitPointer=0;
    INT32 Count=0;

        //SetSDA(PIN_HOLD_0);  // 
        //SetSCL(PIN_HOLD_0);
        //DELAY(20);

        SetSDA(PIN_OFF);  // input
    for(wClockCnt = 0; wClockCnt < 8; wClockCnt++)
    {
        DELAY(50);
            SetSCL(PIN_OFF);

        DELAY(30);

        Data <<= 1;
//printf("PIO_PDSR= 0x%0.8x\r\n",I2C_PIO_BASE->PIO_PDSR );
        if( (I2C_PIO_BASE->PIO_PDSR & BIT_I2CSDA) == BIT_I2CSDA )
            Data |= 1;

        DELAY(20);
        
            SetSCL(PIN_HOLD_0);
        DELAY(20);
    }
    
    // SEND ACK
        SetSDA(PIN_HOLD_0);
    DELAY(50);
        SetSCL(PIN_OFF);
    DELAY(30);
        SetSCL(PIN_HOLD_0);
    DELAY(20);


//        SetSDA(PIN_OFF);
    
    return Data;
}

static BOOL
Write_I2C_Data( BYTE Data )
{
    UINT16 wClockCnt=0, wBitPointer=0;
    INT32 Count=0;
    wBitPointer=0x80;

    
        SetSDA(PIN_HOLD_0);
        SetSCL(PIN_HOLD_0);
        DELAY(30);

    for(wClockCnt = 0; wClockCnt < 8; wClockCnt++)
    {
        if(Data & (BYTE)wBitPointer)
            SetSDA(PIN_OFF);
        else
            SetSDA(PIN_HOLD_0);

        DELAY(10);
            SetSCL(PIN_OFF);
        DELAY(30);
            SetSCL(PIN_HOLD_0);
        DELAY(20);
        wBitPointer >>= 1;
    }

        SetSDA(PIN_OFF); // data in
    DELAY( 20 );
        SetSCL(PIN_OFF);
    DELAY( 20 );
    //while( AT91F_PIO_IsInputSet(I2C_PIO_BASE, BIT_I2CSDA) )
    while( (I2C_PIO_BASE->PIO_PDSR & BIT_I2CSDA) )
    {
        Count++;
        if(Count >= 1000)
        {
            // stop
            Stop_I2C();
            return FALSE;
        }
        DELAY( 1 );
    }

        SetSCL(PIN_HOLD_0);
    //DELAY( 20 );
    //AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSCL );
    DELAY(50 );
    return TRUE;
}





STATIC VOID
TWID_Write( BYTE Addr, UINT32 Reg, BYTE Size, PBYTE pData, UINT32 Num )
//(BYTE Val, BYTE Reg, BYTE nReg )
{
//    WriteL3Addr(Reg);
//    WriteBIT_AUDIO_L3DAT( (BYTE)Val, 0);
    BOOL ret1,ret2,ret3;
    
    BYTE    Val = *pData;
//kprintf("%s(%d):Reg = 0x%2.2x, Val = 0x%2.2x\n",__FUNCTION__,__LINE__, Reg, Val );
RETRY:
    Start_I2C();
    ret1 = Write_I2C_Data( (Addr<<1)|0 );
    ret2 = Write_I2C_Data( Reg );
    ret3 = Write_I2C_Data( Val );
    Stop_I2C();
//kprintf( "%s ret (%x , %x , %x)\n",__FUNCTION__,ret1,ret2,ret3 );

    if( !ret1 || !ret2 || !ret3 )
        goto RETRY;
}

STATIC VOID
TWID_Read( BYTE Addr, UINT32 Reg, BYTE Size, PBYTE pData, UINT32 Num )
//BYTE Val, BYTE Reg, BYTE nReg )
{
//    WriteL3Addr(Reg);
//    WriteBIT_AUDIO_L3DAT( (BYTE)Val, 0);
    BOOL ret1,ret2,ret3;
RETRY:
    Start_I2C();
    ret1 = Write_I2C_Data( (Addr<<1) | 0);
    ret2 = Write_I2C_Data( Reg );
    Stop_I2C();
Sleep(10);
    Start_I2C();
    ret3 = Write_I2C_Data( (Addr<<1) | 1);
//    ret2 = Write_I2C_Data( Reg );
DELAY(200);
    *pData = Read_I2C_Data(  );
    Stop_I2C();
//kprintf("%s(%d):Reg = 0x%2.2x, Val = 0x%2.2x\n",__FUNCTION__,__LINE__, Reg, *pData );
//kprintf("%s ret (%x , %x, %x)\n",__FUNCTION__,ret1,ret2, ret3 );
    if( !ret1 || !ret2 || !ret3 )
    {
        Sleep(20);
        goto RETRY;
    }
}

#else
STATIC HANDLE   hI2C = NULL;

STATIC VOID
TWID_Write( BYTE Addr, UINT32 Reg, BYTE Size, PBYTE pData, UINT32 Num )
{
    BOOL    bRet;

    TWI_IO_DESC TwiIoDesc;

    TwiIoDesc.SlaveAddress = Addr;
    TwiIoDesc.PageSize = 0;
    TwiIoDesc.Delay = 1;
    TwiIoDesc.InterSize = Size/8;
    TwiIoDesc.InterAddress = Reg;
    TwiIoDesc.Count = Num;
    TwiIoDesc.pData = pData;
    
    bRet = IoControl( hI2C, IOCTRL_IIC_MASTER_WRITE, &TwiIoDesc, sizeof(TWI_IO_DESC), NULL, NULL);

}

STATIC VOID
TWID_Read( BYTE Addr, UINT32 Reg, BYTE Size, PBYTE pData, UINT32 Num )
{
    BOOL    bRet;
    TWI_IO_DESC TwiIoDesc;

//kprintf("%s(%d): Reg = %2.2x \n",__FUNCTION__,__LINE__, Reg);

    TwiIoDesc.SlaveAddress = Addr;
    TwiIoDesc.PageSize = 0;
    TwiIoDesc.Delay = 1;
    TwiIoDesc.InterSize = 1;
    TwiIoDesc.InterAddress = Reg;
    TwiIoDesc.Count = Num;
    TwiIoDesc.pData = pData;
    
    bRet = IoControl( hI2C, IOCTRL_IIC_MASTER_READ, &TwiIoDesc, sizeof(TWI_IO_DESC), NULL, NULL);

    return;
}
#endif


//STATIC INT32
//HDMI_Enable( VOID )
//{
//    int         err;
//    BYTE        val, vals[14];
//    int         i;
//    UINT16      out_len = 0;
//    BYTE        edid[HDMI_EDID_MAX_LENGTH];
//    UINT16      horizontal_res;
//    UINT16      vertical_res;
//    UINT16      pixel_clk;
//
//
//    memset(edid, 0, HDMI_EDID_MAX_LENGTH);
//    memset(vals, 0, 14);
//
//#if 0
//    err = hdmi_read_edid(sil9022_client, HDMI_EDID_MAX_LENGTH,
//                 edid, &out_len);
//    if (err < 0 || out_len == 0)
//    {
//        dev_err(&sil9022_client->dev,
//            "<%s> Unable to read EDID for monitor\n", __func__);
//        return err;
//    }
//
//    get_edid_timing_data(edid,
//                 &pixel_clk,
//                 &horizontal_res,
//                 &vertical_res
//                 );
//
//    /*  Fill the TPI Video Mode Data structure */
//    vals[0] = (pixel_clk & 0xFF);                  /* Pixel clock */
//    vals[1] = ((pixel_clk & 0xFF00) >> 8);
//    vals[2] = VERTICAL_FREQ;                    /* Vertical freq */
//    vals[3] = 0x00;
//    vals[4] = (horizontal_res & 0xFF);         /* Horizontal pixels*/
//    vals[5] = ((horizontal_res & 0xFF00) >> 8);
//    vals[6] = (vertical_res & 0xFF);           /* Vertical pixels */
//    vals[7] = ((vertical_res & 0xFF00) >> 8);
//
//    kprintf( "<%s>\nHDMI Monitor E-EDID Timing Data\n"
//                    "horizontal_res    = %d\n"
//                    "vertical_res    = %d\n"
//                    "pixel_clk    = %d\n"
//                    "hfp        = %d\n"
//                    "hsw        = %d\n"
//                    "hbp        = %d\n"
//                    "vfp        = %d\n"
//                    "vsw        = %d\n"
//                    "vbp        = %d\n",
//         __func__,
//         omap_dss_hdmi_timings.x_res,
//         omap_dss_hdmi_timings.y_res,
//         omap_dss_hdmi_timings.pixel_clock,
//         omap_dss_hdmi_timings.hfp,
//         omap_dss_hdmi_timings.hsw,
//         omap_dss_hdmi_timings.hbp,
//         omap_dss_hdmi_timings.vfp,
//         omap_dss_hdmi_timings.vsw,
//         omap_dss_hdmi_timings.vbp
//         );
//
//    dssdev->panel.timings = omap_dss_hdmi_timings;
//#else
//
//    /*  Fill the TPI Video Mode Data structure */
//    vals[0] = (pixel_clk & 0xFF);                  /* Pixel clock */
//    vals[1] = ((pixel_clk & 0xFF00) >> 8);
//    vals[2] = VERTICAL_FREQ;                    /* Vertical freq */
//    vals[3] = 0x00;
//    vals[4] = (horizontal_res & 0xFF);         /* Horizontal pixels*/
//    vals[5] = ((horizontal_res & 0xFF00) >> 8);
//    vals[6] = (vertical_res & 0xFF);           /* Vertical pixels */
//    vals[7] = ((vertical_res & 0xFF00) >> 8);
//
//#endif
//
//    /*  Write out the TPI Video Mode Data */
//    out_len = 0;
//    err = sil9022_blockwrite_reg(sil9022_client,
//                     HDMI_TPI_VIDEO_DATA_BASE_REG,
//                     8, vals, &out_len);
//    if (err < 0)
//    {
//        dev_err(&sil9022_client->dev,
//            "<%s> ERROR: writing TPI video mode data\n", __func__);
//        return err;
//    }
//
//    /* Write out the TPI Pixel Repetition Data (24 bit wide bus,
//    falling edge, no pixel replication) */
//    val = TPI_AVI_PIXEL_REP_BUS_24BIT |
//          TPI_AVI_PIXEL_REP_FALLING_EDGE |
//          TPI_AVI_PIXEL_REP_NONE;
//    err = sil9022_write_reg(sil9022_client,
//                HDMI_TPI_PIXEL_REPETITION_REG,
//                val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: writing TPI pixel repetition data\n",
//            __func__);
//        return err;
//    }
//
//     /*  Write out the TPI AVI Input Format */
//    val =   TPI_AVI_INPUT_BITMODE_8BIT |
//            TPI_AVI_INPUT_RANGE_AUTO |
//            TPI_AVI_INPUT_COLORSPACE_RGB;
//    err = sil9022_write_reg(sil9022_client,
//                HDMI_TPI_AVI_IN_FORMAT_REG,
//                val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: writing TPI AVI Input format\n", __func__);
//        return err;
//    }
//
//    /*  Write out the TPI AVI Output Format */
//    val =   TPI_AVI_OUTPUT_CONV_BT709 |
//            TPI_AVI_OUTPUT_RANGE_AUTO |
//            TPI_AVI_OUTPUT_COLORSPACE_RGBHDMI;
//    err = sil9022_write_reg(sil9022_client,
//                HDMI_TPI_AVI_OUT_FORMAT_REG, val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: writing TPI AVI output format\n",
//            __func__);
//        return err;
//    }
//
//    /* Write out the TPI System Control Data to power down */
//    val = TPI_SYS_CTRL_POWER_DOWN;
//    err = sil9022_write_reg(sil9022_client, HDMI_SYS_CTRL_DATA_REG, val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: writing TPI power down control data\n",
//            __func__);
//        return err;
//    }
//
//    /* Write out the TPI AVI InfoFrame Data (all defaults) */
//    /* Compute CRC*/
//    val = 0x82 + 0x02 + 13;
//
//    for (i = 0; i < sizeof(avi_info_frame_data); i++)
//        val += avi_info_frame_data[i];
//
//    avi_info_frame_data[0] = 0x100 - val;
//
//    out_len = 0;
//    err = sil9022_blockwrite_reg(sil9022_client,
//                     HDMI_TPI_AVI_DBYTE_BASE_REG,
//                     sizeof(avi_info_frame_data),
//                     avi_info_frame_data, &out_len);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: writing TPI AVI infoframe data\n",
//            __func__);
//        return err;
//    }
//
//    /*  Audio Configuration  */
//    err = hdmi_enable_audio(sil9022_client);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: Unable set audio configuration\n",
//            __func__);
//        return err;
//    }
//
//    /*  Write out the TPI Device Power State (D0) */
//    val = TPI_AVI_POWER_STATE_D0;
//    err = sil9022_write_reg(sil9022_client,
//                HDMI_TPI_POWER_STATE_CTRL_REG, val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: Setting device power state to D0\n",
//            __func__);
//        return err;
//    }
//
//    /* Write out the TPI System Control Data to power up and
//     * select output mode
//     */
//    val = TPI_SYS_CTRL_POWER_ACTIVE | TPI_SYS_CTRL_OUTPUT_MODE_HDMI;
//    err = sil9022_write_reg(sil9022_client, HDMI_SYS_CTRL_DATA_REG, val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: Writing system control data\n", __func__);
//        return err;
//    }
//
//    /*  Read back TPI System Control Data to latch settings */
//    msleep(10);
//    err = sil9022_read_reg(sil9022_client, 1, HDMI_SYS_CTRL_DATA_REG, &val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: Writing system control data\n",
//            __func__);
//        return err;
//    }
//
//    /* HDCP Enable - Disable */
//    val = 0;
//    err = sil9022_write_reg(sil9022_client,
//                HDMI_TPI_HDCP_CONTROLDATA_REG, val);
//    if (err < 0)
//    {
//        kprintf(
//            "<%s> ERROR: Enable (1) / Disable (0) => HDCP: %d\n",
//            __func__, val);
//        return err;
//    }
//
//    kprintf("<%s> hdmi enabled\n", __func__);
//
//    return 0;
//}
//
//
//STATIC INT32
//HDMI_Disable( VOID )
//{
//    u8 val = 0;
//    int err = 0;
//
//    err = hdmi_disable_audio(sil9022_client);
//    if (err < 0) {
//        dev_err(&sil9022_client->dev,
//            "<%s> ERROR: failed to disable audio\n", __func__);
//        return err;
//    }
//
//    /*  Write out the TPI System Control Data to power down  */
//    val = TPI_SYS_CTRL_POWER_DOWN;
//    err = sil9022_write_reg(sil9022_client, HDMI_SYS_CTRL_DATA_REG, val);
//    if (err < 0) {
//        dev_err(&sil9022_client->dev,
//            "<%s> ERROR: writing control data - power down\n",
//            __func__);
//        return err;
//    }
//
//    /*  Write out the TPI Device Power State (D2) */
//    val = TPI_AVI_POWER_STATE_D2;
//    err = sil9022_write_reg(sil9022_client,
//              HDMI_TPI_DEVICE_POWER_STATE_DATA, val);
//    if (err < 0) {
//        dev_err(&sil9022_client->dev,
//            "<%s> ERROR: Setting device power state to D2\n",
//            __func__);
//        return err;
//    }
//
//    /*  Read back TPI System Control Data to latch settings */
//    mdelay(10);
//    err = sil9022_read_reg(sil9022_client, 1, HDMI_SYS_CTRL_DATA_REG, &val);
//    if (err < 0) {
//        dev_err(&sil9022_client->dev,
//            "<%s> ERROR:  Reading System control data "
//            "- latch settings\n", __func__);
//        return err;
//    }
//
//    dev_dbg(&sil9022_client->dev, "<%s> hdmi disabled\n", __func__);
//
//    return 0;
//
//}
//



/**
 * \brief Read value in given register 
 * \param device  Twi slave address.
 * \param regAddr Register address to read.
 * \return value in the given register.
 */
UINT8 Sil9022a_Read(
                      UINT32 device,
                      UINT32 regAddr
                     )
{
    UINT8 Tdata = 0;
    TWID_Read( device, regAddr, 1, &Tdata, 1 );
    Sleep(10);
    return Tdata;
}

/**
 * \brief Write value to given register 
 * \param device  Twi slave address.
 * \param regAddr Register address to read.
 * \param data    Data to write
 */
VOID Sil9022a_Write(
                    UINT32 device,
                    UINT32 regAddr,
                    UINT8  data
                   )
{
    TWID_Write(device, regAddr, 1, &data, 1);
    Sleep(10);
}

/**
 * \brief Modify value to given register 
 * \param device  Twi slave address.
 * \param regAddr Register address to read.
 * \param bitMask Bit need to modify
 */
VOID Sil9022a_ModifySet( UINT32 device, UINT32 regAddr, UINT8 bitMask)
{
    UINT8 dataTmp = 0;
    dataTmp = Sil9022a_Read(device, regAddr);
    //kprintf("Read   data[%x]\n\r", dataTmp);    
    dataTmp |= bitMask;
    TWID_Write( device, regAddr, 1, &dataTmp, 1);
}

VOID Sil9022a_ModifyClear(
                  UINT32 device,
                  UINT32 regAddr,
                  UINT8 bitMask)
{
    UINT8 dataTmp = 0;
    dataTmp = Sil9022a_Read( device, regAddr);
    //kprintf("Read   data[%x]\n\r", dataTmp);
    dataTmp &= ~bitMask;
    TWID_Write(device, regAddr, 1, &dataTmp, 1);
}

VOID Sil9022a_Modify(
                     UINT32 device,
                     UINT32 regAddr,
                     UINT8 bitMask,
                     UINT8 value)
{
    UINT8 dataTmp = 0;
    dataTmp = Sil9022a_Read( device, regAddr);
    dataTmp &= ~bitMask;
    dataTmp |= (bitMask & value);
    TWID_Write( device, regAddr, 1, &dataTmp, 1);
}

STATIC VOID Sil9022a_WriteIndexedRegister( 
                                    UINT32 device,
                                    UINT8 PageNum,
                                    UINT8 RegOffset,
                                    UINT8 Value)
{
    Sil9022a_Write( device, TPI_INTERNAL_PAGE_REG, PageNum);
    Sil9022a_Write( device, TPI_INDEXED_OFFSET_REG, RegOffset);
    Sil9022a_Write( device, TPI_INDEXED_VALUE_REG, Value);
}

VOID Sil9022a_ReadModifyWriteIndexedRegister(
                                              UINT32 device,
                                              UINT8 PageNum,
                                              UINT8 RegOffset,
                                              UINT8 Mask,
                                              UINT8 Value)
{
    UINT8 Tmp;
    Sil9022a_Write( device, TPI_INTERNAL_PAGE_REG, PageNum);
    Sil9022a_Write( device, TPI_INDEXED_OFFSET_REG, RegOffset);
    Tmp = Sil9022a_Read( device, TPI_INDEXED_VALUE_REG);
    Tmp &= ~Mask;
    Tmp |= (Value & Mask);
    Sil9022a_Write( device, TPI_INDEXED_VALUE_REG, Tmp);
}


VOID Sil9022a_EnableInterrupt(
                              UINT32 device, 
                              UINT8 InterruptMask)
{
    Sil9022a_ModifySet( device, SIL9022A_INTERRUPT_ENABLE_REG, InterruptMask);
}

VOID Sil9022a_DisableInterrupt( 
                               UINT32 device, 
                               UINT8 InterruptMask)
{
    Sil9022a_ModifyClear( device, SIL9022A_INTERRUPT_ENABLE_REG, InterruptMask);
}

/**
 * \brief Select output video mode
 */
VOID Sil9022a_VideoSel (UINT8 vmode, video_mode_param *pVideo)
{
    siHdmiTx.HDMIVideoFormat = VMD_HDMIFORMAT_CEA_VIC;
    siHdmiTx.ColorSpace  = RGB;
    siHdmiTx.ColorDepth  = VMD_COLOR_DEPTH_8BIT;
    siHdmiTx.SyncMode = EXTERNAL_HSVSDE;
        
    switch (vmode)
    {
        case HDMI_480I60_4X3:
            siHdmiTx.VIC         = 6;
            siHdmiTx.AspectRatio     = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry    = COLORIMETRY_601;
            siHdmiTx.TclkSel        = XXX2;
            pVideo->g_h_max = 640;
            pVideo->g_v_max = 480;
            pVideo->g_pixelFreq = 27000000;
            pVideo->g_v_pulseW = 3;
            pVideo->g_h_pulseW = 62;
            pVideo->g_v_frontB = 15;
            pVideo->g_v_backB = 15;
            pVideo->g_h_topB = 114;
            pVideo->g_h_bottomB = 114;
            break;

        case HDMI_576I50_4X3:
            siHdmiTx.VIC         = 21;
            siHdmiTx.AspectRatio     = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry    = COLORIMETRY_601;
            siHdmiTx.TclkSel        = XXX2;
            pVideo->g_h_max = 720;
            pVideo->g_v_max = 576;     
            pVideo->g_pixelFreq = 27000000; 
            pVideo->g_v_pulseW = 3;
            pVideo->g_h_pulseW = 63;
            pVideo->g_v_frontB = 19;
            pVideo->g_v_backB = 19;
            pVideo->g_h_topB = 138;
            pVideo->g_h_bottomB = 138;
            break;

        case HDMI_480P60_4X3:
            siHdmiTx.VIC         = 2;
            siHdmiTx.AspectRatio     = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry    = COLORIMETRY_601;
            siHdmiTx.TclkSel        = XXX1;
            pVideo->g_h_max = 640;
            pVideo->g_v_max = 480;  
            pVideo->g_pixelFreq = 27000000;
            pVideo->g_v_pulseW = 6;
            pVideo->g_h_pulseW = 62;
            pVideo->g_v_frontB = 30;
            pVideo->g_v_backB = 30;
            pVideo->g_h_topB = 60;
            pVideo->g_h_bottomB = 60;
            break;

        case HDMI_576P50_4X3:
            siHdmiTx.VIC         = 17;
            siHdmiTx.AspectRatio     = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry    = COLORIMETRY_601;
            siHdmiTx.TclkSel        = XXX1;
            pVideo->g_h_max = 720;
            pVideo->g_v_max = 576;
            pVideo->g_pixelFreq = 27000000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 64;
            pVideo->g_v_frontB = 39;
            pVideo->g_v_backB = 39;
            pVideo->g_h_topB = 68;
            pVideo->g_h_bottomB = 68;
            break;
            
        case HDMI_720P60:
            siHdmiTx.VIC = 4;
            siHdmiTx.AspectRatio  = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry  = COLORIMETRY_709;
            siHdmiTx.TclkSel      = XXX1;
            pVideo->g_h_max = 1280;
            pVideo->g_v_max = 720;
            pVideo->g_pixelFreq = 74250000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 40;
            pVideo->g_v_frontB = 20;
            pVideo->g_v_backB = 20;
            pVideo->g_h_topB = 220;
            pVideo->g_h_bottomB = 220;
            break;

        case HDMI_720P50:
            siHdmiTx.VIC = 19;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1280;
            pVideo->g_v_max = 720;
            pVideo->g_pixelFreq = 74250000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 40;
            pVideo->g_v_frontB = 20;
            pVideo->g_v_backB = 20;
            pVideo->g_h_topB = 220;
            pVideo->g_h_bottomB = 220;
            break;

#ifdef HDMI_1080IP
        case HDMI_1080I60:
            siHdmiTx.VIC = 5;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1920;
            pVideo->g_v_max = 1080;
            pVideo->g_pixelFreq = 74250000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 44;
            pVideo->g_v_frontB = 15;
            pVideo->g_v_backB = 15;
            pVideo->g_h_topB = 148;
            pVideo->g_h_bottomB = 148;
            break;

        case HDMI_1080I50:
            siHdmiTx.VIC = 20;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1920;
            pVideo->g_v_max = 1080; 
            pVideo->g_pixelFreq = 74250000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 44;
            pVideo->g_v_frontB = 15;
            pVideo->g_v_backB = 15;
            pVideo->g_h_topB = 148;
            pVideo->g_h_bottomB = 148;
            break;

        case HDMI_1080P60:
            siHdmiTx.VIC  = 16;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1920;
            pVideo->g_v_max = 1080; 
            pVideo->g_pixelFreq = 148350000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 44;
            pVideo->g_v_frontB = 36;
            pVideo->g_v_backB = 36;
            pVideo->g_h_topB = 148;
            pVideo->g_h_bottomB = 148;
            break;

        case HDMI_1080P50:
            siHdmiTx.VIC = 31;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1920;
            pVideo->g_v_max = 1080;  
            pVideo->g_pixelFreq = 148500000; 
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 44;
            pVideo->g_v_frontB = 36;
            pVideo->g_v_backB = 36;
            pVideo->g_h_topB = 148;
            pVideo->g_h_bottomB = 148;
            break;
#endif
        case HDMI_1024_768_60:
            siHdmiTx.VIC = PC_BASE+13;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1024;
            pVideo->g_v_max = 768;
            pVideo->g_pixelFreq = 65000000; 
            pVideo->g_v_pulseW = 6;
            pVideo->g_h_pulseW = 136;
            pVideo->g_v_frontB = 29;
            pVideo->g_v_backB = 29;
            pVideo->g_h_topB = 160;
            pVideo->g_h_bottomB = 160; 
            break;
        case HDMI_800_600_60:
            siHdmiTx.VIC = PC_BASE+9;
            siHdmiTx.AspectRatio  = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel  = XXX1;
            pVideo->g_h_max = 800;
            pVideo->g_v_max = 600;  
            pVideo->g_pixelFreq = 40000000;
            pVideo->g_v_pulseW = 4;
            pVideo->g_h_pulseW = 128;
            pVideo->g_v_frontB = 23;
            pVideo->g_v_backB = 23;
            pVideo->g_h_topB = 88;
            pVideo->g_h_bottomB = 88;
            break;

        case HDMI_800_480_60:
            siHdmiTx.VIC = PC_BASE+9;
            siHdmiTx.AspectRatio  = VMD_ASPECT_RATIO_4x3;
            siHdmiTx.Colorimetry    = COLORIMETRY_709;
            siHdmiTx.TclkSel  = XXX1;
            pVideo->g_h_max = 800;
            pVideo->g_v_max = 480;
            pVideo->g_pixelFreq = 28230000;
            pVideo->g_v_pulseW = 4;
            pVideo->g_h_pulseW = 128;
            pVideo->g_v_frontB = 23;
            pVideo->g_v_backB = 23;
            pVideo->g_h_topB = 88;
            pVideo->g_h_bottomB = 88;
            break;

        default:
            siHdmiTx.VIC = 4;
            siHdmiTx.AspectRatio = VMD_ASPECT_RATIO_16x9;
            siHdmiTx.Colorimetry = COLORIMETRY_709;
            siHdmiTx.TclkSel = XXX1;
            pVideo->g_h_max = 1280;
            pVideo->g_v_max = 720;
            pVideo->g_pixelFreq = 74250000;
            pVideo->g_v_pulseW = 5;
            pVideo->g_h_pulseW = 40;
            pVideo->g_v_frontB = 20;
            pVideo->g_v_backB = 20;
            pVideo->g_h_topB = 220;
            pVideo->g_h_bottomB = 220;
            break;
    }
}

/**
 * \brief Select output audio mode
 */
VOID Sil9022a_AudioSel (UINT8 Afs)
{
    siHdmiTx.AudioMode            = AMODE_I2S;
    siHdmiTx.AudioChannels        = ACHANNEL_2CH;
    siHdmiTx.AudioFs                = Afs;
    siHdmiTx.AudioWordLength        = ALENGTH_16BITS;//ALENGTH_16BITS
    siHdmiTx.AudioI2SFormat        = (MCLK256FS << 4) |SCK_SAMPLE_RISING_EDGE |0x00;
}

VOID Sil9022a_InitializeStateVariables (VOID)
{

    hI2C = IoOpen( (PBYTE)"I2C2" ); 

    g_sys.tmdsPoweredUp = FALSE;
    g_sys.hdmiCableConnected = FALSE;
    g_sys.dsRxPoweredUp = FALSE;

#ifdef DEV_SUPPORT_EDID
    g_edid.edidDataValid = FALSE;
    g_edid.HDMI_Sink = TRUE;
#endif
}

//------------------------------------------------------------------------------
//        EDID functions
//------------------------------------------------------------------------------

#define T_DDC_ACCESS    50

STATIC UINT8 GetDDC_Access ( UINT8* SysCtrlRegVal)
{
    UINT8 sysCtrl;
    UINT8 DDCReqTimeout = T_DDC_ACCESS;
    UINT8 TPI_ControlImage;

    // Read and store original value. Will be passed into ReleaseDDC()
    sysCtrl = Sil9022a_Read( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG);
    *SysCtrlRegVal = sysCtrl;
    sysCtrl |= DDC_BUS_REQUEST_REQUESTED;
    Sil9022a_Write(SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, sysCtrl); 

    // Loop till 0x1A[1] reads "1"
    while (DDCReqTimeout--) 
    {
        TPI_ControlImage = Sil9022a_Read( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG);
        // When 0x1A[1] reads "1"
        if (TPI_ControlImage & DDC_BUS_GRANT_MASK)
        {
            sysCtrl |= DDC_BUS_GRANT_GRANTED;
            // lock host DDC bus access (0x1A[2:1] = 11)
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, sysCtrl);
            return TRUE;
        }
        // 0x1A[2] = "1" - Requst the DDC bus
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, sysCtrl);
        Sleep(20);
    }
    // Failure... restore original value.
    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, *SysCtrlRegVal);

    return FALSE;
}


/**
 * \brief  Release DDC bus
 */
STATIC UINT8 ReleaseDDC ( UINT8 SysCtrlRegVal)
{
    UINT8 DDCReqTimeout = T_DDC_ACCESS;
    UINT8 TPI_ControlImage;

    SysCtrlRegVal &= ~BITS_2_1;                    // Just to be sure bits [2:1] are 0 before it is written

    while (DDCReqTimeout--)                        // Loop till 0x1A[1] reads "0"
    {
        // Cannot use ReadClearWriteTPI() here. A read of TPI_SYSTEM_CONTROL is invalid while DDC is granted.
        // Doing so will return 0xFF, and cause an invalid value to be written back.
        //ReadClearWriteTPI(TPI_SYSTEM_CONTROL,BITS_2_1); // 0x1A[2:1] = "0" - release the DDC bus
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG, SysCtrlRegVal); 
        TPI_ControlImage = Sil9022a_Read( SIL9022A_SLAVE_ADDRESS, TPI_SYSTEM_CONTROL_DATA_REG);
        if (!(TPI_ControlImage & BITS_2_1))        // When 0x1A[2:1] read "0"
            return TRUE;
    }
    return FALSE;                                 // Failed to release DDC bus control
}


/**
 * \brief  Checks if EDID header is correct per VESA E-EDID standard
 * \param Accepts: Pointer to 1st EDID block
 */
STATIC UINT8 CheckEDID_Header (UINT8  *Block)
{
        UINT8  i = 0;

        if (Block[i])               // byte 0 must be 0
            return FALSE;

        for (i = 1; i < 1 + EDID_HDR_NO_OF_FF; i++)
        {
            if(Block[i] != 0xFF)    // bytes [1..6] must be 0xFF
                return FALSE;
        }

        if (Block[i])               // byte 7 must be 0
            return FALSE;

        return TRUE;
}

/**
 * \brief  Calculte checksum of the 128 byte block pointed to by the pointer passed as parameter
 * \param  Accepts: Pointer to a 128 byte block whose checksum needs to be calculated
 */
STATIC UINT8 DoEDID_Checksum (UINT8   *Block)
{
        UINT8   i;
        UINT8   CheckSum = 0;

        for (i = 0; i < EDID_BLOCK_SIZE; i++)
            CheckSum += Block[i];

        if (CheckSum)
            return FALSE;

        return TRUE;
}

STATIC VOID I2C_readEDIDBlock (
                               UINT8 blockAddr, 
                               UINT8 regAddr, 
                               UINT8 nBytes, 
                               UINT8 * data )
{
    UINT32 i;
    for(i = 0; i < nBytes; i++)
    {
        data[i] = Sil9022a_Read( blockAddr, regAddr+i);
        Sleep(10);
    }

}

/**
 * \brief  Parse CEA-861 extension short descriptors of the EDID block
 *  passed as a parameter and save them in global structure g_edid.
 * \param  Accepts: A pointer to the EDID 861 Extension block being parsed.
 * \note Fields that are not supported by the 9022/4 (such as deep color) were not parsed.
 */
STATIC UINT8 Parse861ShortDescriptors ( UINT8 *Data)
{
    UINT8 LongDescriptorOffset;
    UINT8 DataBlockLength;
    UINT8 DataIndex;
    UINT8 ExtendedTagCode;
    UINT8 VSDB_BaseOffset = 0;

    UINT8 V_DescriptorIndex = 0;  // STATIC to support more than one extension
    UINT8 A_DescriptorIndex = 0;  // STATIC to support more than one extension

    UINT8 TagCode;

    UINT32 i;
    UINT32 j;

    if (Data[EDID_TAG_ADDR] != EDID_EXTENSION_TAG)
    {
        kprintf("-E- EDID -> Extension Tag Error\n\r");
        return EDID_EXT_TAG_ERROR;
    }

    if (Data[EDID_REV_ADDR] != EDID_REV_THREE)
    {
        kprintf("-E- EDID -> Revision Error\n\r");
        return EDID_REV_ADDR_ERROR;
    }

    LongDescriptorOffset = Data[LONG_DESCR_PTR_IDX];    // block offset where long descriptors start

    g_edid.UnderScan = ((Data[MISC_SUPPORT_IDX]) >> 7) & LSBIT;  // byte #3 of CEA extension version 3
    g_edid.BasicAudio = ((Data[MISC_SUPPORT_IDX]) >> 6) & LSBIT;
    g_edid.YCbCr_4_4_4 = ((Data[MISC_SUPPORT_IDX]) >> 5) & LSBIT;
    g_edid.YCbCr_4_2_2 = ((Data[MISC_SUPPORT_IDX]) >> 4) & LSBIT;

    DataIndex = EDID_DATA_START;            // 4

    while (DataIndex < LongDescriptorOffset)
    {
        TagCode = (Data[DataIndex] >> 5) & THREE_LSBITS;
        DataBlockLength = Data[DataIndex++] & FIVE_LSBITS;
        if ((DataIndex + DataBlockLength) > LongDescriptorOffset)
        {
            kprintf("-E- EDID -> V Descriptor Overflow\n\r");
            return EDID_V_DESCR_OVERFLOW;
        }

        i = 0;                                  // num of short video descriptors in current data block

        switch (TagCode)
        {
            case VIDEO_D_BLOCK:
                while ((i < DataBlockLength) && (i < MAX_V_DESCRIPTORS))        // each SVD is 1 byte long
                {
                    g_edid.VideoDescriptor[V_DescriptorIndex++] = Data[DataIndex++];
                    i++;
                }
                DataIndex += DataBlockLength - i;   // if there are more STDs than MAX_V_DESCRIPTORS, skip the last ones. Update DataIndex

                break;

            case AUDIO_D_BLOCK:
                while (i < DataBlockLength/3)       // each SAD is 3 bytes long
                {
                    j = 0;
                    while (j < AUDIO_DESCR_SIZE)    // 3
                    {
                        g_edid.AudioDescriptor[A_DescriptorIndex][j++] = Data[DataIndex++];
                    }
                    A_DescriptorIndex++;
                    i++;
                }
                break;

            case  SPKR_ALLOC_D_BLOCK:
                g_edid.SpkrAlloc[i++] = Data[DataIndex++];       // although 3 bytes are assigned to Speaker Allocation, only
                DataIndex += 2;                                     // the first one carries information, so the next two are ignored by this code.
                break;

            case USE_EXTENDED_TAG:
                ExtendedTagCode = Data[DataIndex++];

                switch (ExtendedTagCode)
                {
                    case VIDEO_CAPABILITY_D_BLOCK:
                        // TO BE ADDED HERE: Save "video capability" parameters in g_edid data structure
                        // Need to modify that structure definition
                        // In the meantime: just increment DataIndex by 1
                        DataIndex += 1;    // replace with reading and saving the proper data per CEA-861 sec. 7.5.6 while incrementing DataIndex
                        break;

                    case COLORIMETRY_D_BLOCK:
                        g_edid.ColorimetrySupportFlags = Data[DataIndex++] & BITS_1_0;
                        g_edid.MetadataProfile = Data[DataIndex++] & BITS_2_1_0;
                        break;
                }
                break;

            case VENDOR_SPEC_D_BLOCK:
                VSDB_BaseOffset = DataIndex - 1;

                if ((Data[DataIndex++] == 0x03) &&    // check if sink is HDMI compatible
                    (Data[DataIndex++] == 0x0C) &&
                    (Data[DataIndex++] == 0x00))

                    g_edid.HDMI_Sink = TRUE;
                else
                    g_edid.HDMI_Sink = FALSE;

                g_edid.CEC_A_B = Data[DataIndex++];  // CEC Physical address
                g_edid.CEC_C_D = Data[DataIndex++];

#ifdef DEV_SUPPORT_CEC
                // Take the Address that was passed in the EDID and use this API
                // to set the physical address for CEC.
                {
                    word    phyAddr;
                    phyAddr = (word)g_edid.CEC_C_D;     // Low-order nibbles
                    phyAddr |= ((word)g_edid.CEC_A_B << 8); // Hi-order nibbles
                    // Is the new PA different from the current PA?
                    if (phyAddr != SI_CecGetDevicePA ())
                    {
                        // Yes!  So change the PA
                        SI_CecSetDevicePA (phyAddr);
                    }
                }
#endif

                if ((DataIndex + 7) > VSDB_BaseOffset + DataBlockLength)        // Offset of 3D_Present bit in VSDB
                        g_edid._3D_Supported = FALSE;
                else if (Data[DataIndex + 7] >> 7)
                        g_edid._3D_Supported = TRUE;
                else
                        g_edid._3D_Supported = FALSE;

                DataIndex += DataBlockLength - HDMI_SIGNATURE_LEN - CEC_PHYS_ADDR_LEN; // Point to start of next block
                //TEST_TRACE_INFO(("EDID -> Short Descriptor Vendor Block\n\r"));
                break;

            default:
                kprintf("-E- EDID -> Unknown Tag Code\n\r");
                return EDID_UNKNOWN_TAG_CODE;

        }                   // End, Switch statement
    }                       // End, while (DataIndex < LongDescriptorOffset) statement

    return EDID_SHORT_DESCRIPTORS_OK;
}

/**
 * \brief  Parse CEA-861 extension long descriptors of the EDID block
 *  passed as a parameter and printf() them to the screen.
 * \param  Accepts: A pointer to the EDID block being parsed
 */
STATIC UINT8 Parse861LongDescriptors ( UINT8 *Data)
{
    UINT8 LongDescriptorsOffset;
    UINT8 DescriptorNum = 1;

    LongDescriptorsOffset = Data[LONG_DESCR_PTR_IDX];   /* EDID block offset 2 holds the offset */

    if (!LongDescriptorsOffset)                         /* per CEA-861-D, table 27 */
    {
        return EDID_NO_DETAILED_DESCRIPTORS;
    }

    while (LongDescriptorsOffset + LONG_DESCR_LEN < EDID_BLOCK_SIZE)
    {
        LongDescriptorsOffset +=  LONG_DESCR_LEN;
        DescriptorNum++;
    }

    return EDID_LONG_DESCRIPTORS_OK;
}

/**
 * \brief  Parse CEA-861 extensions from EDID ROM (EDID blocks beyond
 *                  block #0). Save short descriptors in global structure
 *                  g_edid. printf() long descriptors to the screen.
 * \param  Accepts: The number of extensions in the EDID being parsed
 * \note   Fields that are not supported by the 9022/4 (such as deep color) were not parsed.
 */
STATIC UINT8 Parse861Extensions( UINT8  NumOfExtensions)
{
    UINT8  ErrCode;
    UINT8  Block = 0;
    g_edid.HDMI_Sink = FALSE;

    do
    {
        Block++;

        if (Block == 1)
        {
            I2C_readEDIDBlock( EDID_ROM_ADDR,EDID_BLOCK_1_OFFSET, EDID_BLOCK_SIZE, g_CommData);            
        }
        if ((NumOfExtensions > 1) && (Block == 1))
        {
            continue;
        }

        ErrCode = Parse861ShortDescriptors( g_CommData);
        if (ErrCode != EDID_SHORT_DESCRIPTORS_OK)
        {
            return ErrCode;
        }

        ErrCode = Parse861LongDescriptors( g_CommData);
        if (ErrCode != EDID_LONG_DESCRIPTORS_OK)
        {
            return ErrCode;
        }

    } while (Block < NumOfExtensions);

    return EDID_OK;
}

STATIC UINT8 ParseEDID (UINT8 *pEdid, UINT8 *numExt)
{
    if (!CheckEDID_Header(pEdid))                      //Checks if EDID header is correct per VESA E-EDID standard(/byte 0 must be 0, bytes [1..6] must be 0xFF,byte 7 must be 0)
    {
        // first 8 bytes of EDID must be {0, FF, FF, FF, FF, FF, FF, 0}
        // first 8 bytes of EDID must be {0, 0, FF, FF, FF, FF, FF, FF}
        kprintf("-E- EDID -> Incorrect Header\n\r");
        return EDID_INCORRECT_HEADER;
    }

    if (!DoEDID_Checksum(pEdid))
    {
        // non-zero EDID checksum
        kprintf("-E- EDID -> Checksum Error\n\r");
        return EDID_CHECKSUM_ERROR;
    }

    *numExt = pEdid[NUM_OF_EXTEN_ADDR];    // read # of extensions from offset 0x7E of block 0

    if (!(*numExt))
    {
        // No extensions to worry about
        return EDID_NO_861_EXTENSIONS;
    }

    //return Parse861Extensions(NumOfExtensions);            // Parse 861 Extensions (short and long descriptors);
    return (EDID_OK);
}

UINT8 Sil9022a_DoEdidRead(VOID)
{
    UINT8 SysCtrlReg;
    UINT8 Result;
    UINT8 NumOfExtensions;

    // If we already have valid EDID data, ship this whole thing
    if (g_edid.edidDataValid == FALSE)
    {
    // Request access to DDC bus from the receiver
        if (GetDDC_Access( &SysCtrlReg))
        {
            // read first 128 bytes of EDID ROM
            I2C_readEDIDBlock( EDID_ROM_ADDR,EDID_BLOCK_0_OFFSET, EDID_BLOCK_SIZE, g_CommData);       

            Result = ParseEDID(g_CommData, &NumOfExtensions);
            if (Result != EDID_OK)
            {
                if (Result == EDID_NO_861_EXTENSIONS)
                {
                    kprintf("-I- EDID -> No 861 Extensions\n\r");
                    g_edid.HDMI_Sink = FALSE;
                    g_edid.YCbCr_4_4_4 = FALSE;
                    g_edid.YCbCr_4_2_2 = FALSE;
                    g_edid.CEC_A_B = 0x00;
                    g_edid.CEC_C_D = 0x00;
                }
                else
                {
                    kprintf("-E- EDID -> Parse FAILED\n\r");
                    g_edid.HDMI_Sink = TRUE;
                    g_edid.YCbCr_4_4_4 = FALSE;
                    g_edid.YCbCr_4_2_2 = FALSE;
                    g_edid.CEC_A_B = 0x00;
                    g_edid.CEC_C_D = 0x00;
                }
            }
            else
            {
                kprintf("-I- EDID -> Parse OK\n\r");
                Result = Parse861Extensions( NumOfExtensions);        // Parse 861 Extensions (short and long descriptors);
                if (Result != EDID_OK)
                {
                    kprintf("-E- EDID -> Extension Parse FAILED\n\r");
                    g_edid.HDMI_Sink = TRUE;
                    g_edid.YCbCr_4_4_4 = FALSE;
                    g_edid.YCbCr_4_2_2 = FALSE;
                    g_edid.CEC_A_B = 0x00;
                    g_edid.CEC_C_D = 0x00;
                }
            }

            if (!ReleaseDDC( SysCtrlReg))                // Host must release DDC bus once it is done reading EDID
            {
                //TEST_TRACE_INFO (("EDID -> DDC bus release failed\n\r"));
                return EDID_DDC_BUS_RELEASE_FAILURE;
            }
        }
        else
        {
            //TEST_TRACE_INFO (("EDID -> DDC bus request failed\n\r"));
            g_edid.HDMI_Sink = TRUE;
            g_edid.YCbCr_4_4_4 = FALSE;
            g_edid.YCbCr_4_2_2 = FALSE;
            g_edid.CEC_A_B = 0x00;
            g_edid.CEC_C_D = 0x00; 
            return EDID_DDC_BUS_REQ_FAILURE;
        }

        /*TEST_TRACE_INFO(("EDID -> g_edid.HDMI_Sink = %d\n\r", g_edid.HDMI_Sink));
        TEST_TRACE_INFO(("EDID -> g_edid.YCbCr_4_4_4 = %d\n\r", (uint16_t)g_edid.YCbCr_4_4_4));
        TEST_TRACE_INFO(("EDID -> g_edid.YCbCr_4_2_2 = %d\n\r", (uint16_t)g_edid.YCbCr_4_2_2));
        TEST_TRACE_INFO(("EDID -> g_edid.CEC_A_B = 0x%x\n\r", (uint16_t)g_edid.CEC_A_B));
        TEST_TRACE_INFO(("EDID -> g_edid.CEC_C_D = 0x%x\n\r", (uint16_t)g_edid.CEC_C_D));*/

        g_edid.edidDataValid = TRUE;
    }

    g_sys.hdmiCableConnected = TRUE;
    return EDID_OK;
}

/**
 * \brief  Convert Video Identification Code to the corresponding
 * index of VModesTable[]. Conversion also depends on the 
 * value of the 3D_Structure parameter in the case of 3D video format.
 * \param  Accepts VIC to be converted; 3D_Structure value
 * \note  Conversion is for 861-D formats, HDMI_VIC or 3D
 */
STATIC UINT8 ConvertVIC_To_VM_Index (VOID)
{
    UINT8 index;

    //
    // The global VideoModeDescription contains all the information we need about
    // the Video mode for use to find its entry in the Videio mode table.
    //
    // The first issue.  The "VIC" may be a 891-D VIC code, or it might be an
    // HDMI_VIC code, or it might be a 3D code.  Each require different handling
    // to get the proper video mode table index.
    //
    if (siHdmiTx.HDMIVideoFormat == VMD_HDMIFORMAT_CEA_VIC)
    {
        //
        // This is a regular 861-D format VIC, so we use the VIC to Index
        // table to look up the index.
        //
        index = VIC2Index[siHdmiTx.VIC];
    }
    else if (siHdmiTx.HDMIVideoFormat == VMD_HDMIFORMAT_HDMI_VIC)
    {
        /*
         HDMI_VIC conversion is simple.  We need to subtract one because the codes start
         with one instead of zero.  These values are from HDMI 1.4 Spec Table 8-13. 
        */
        if ((siHdmiTx.VIC < 1) || (siHdmiTx.VIC > 4))
        {
            index = DEFAULT_VIDEO_MODE;
        }
        else
        {
            index = (HDMI_VIC_BASE - 1) + siHdmiTx.VIC;
        }
    }
    else if (siHdmiTx.HDMIVideoFormat == VMD_HDMIFORMAT_3D)
    {
        /*
         Currently there are only a few VIC modes that we can do in 3D.  If the VIC code is not
         one of these OR if the packing type is not supported for that VIC code, then it is an
         error and we go to the default video mode.  See HDMI Spec 1.4 Table H-6.
        */
        switch (siHdmiTx.VIC)
        {
            case VIC_FOR_480P_60Hz_4X3:
            case VIC_FOR_480P_60Hz_16X9:
                // We only support Side-by-Side (Half) for these modes
                if (siHdmiTx.ThreeDStructure == SIDE_BY_SIDE_HALF)
                    index = VIC_3D_BASE + 0;
                else
                    index = DEFAULT_VIDEO_MODE;
                break;

            case VIC_FOR_720P_60Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 1;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
                break;

            case VIC_FOR_1080i_60Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 2;
                        break;
                    case VMD_3D_FIELDALTERNATIVE:
                        index = VIC_3D_BASE + 3;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
                break;

            case VIC_FOR_1080p_60Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 4;
                        break;
                    case VMD_3D_LINEALTERNATIVE:
                        index = VIC_3D_BASE + 5;
                        break;
                    case SIDE_BY_SIDE_FULL:
                        index = VIC_3D_BASE + 6;
                        break;
                    case SIDE_BY_SIDE_HALF:
                        index = VIC_3D_BASE + 7;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
                break;

            case VIC_FOR_720P_50Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 8;
                        break;
                    case VMD_3D_LDEPTH:
                        index = VIC_3D_BASE + 9;
                        break;
                    case VMD_3D_LDEPTHGRAPHICS:
                        index = VIC_3D_BASE + 10;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
                break;

            case VIC_FOR_1080i_50Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 11;
                        break;
                    case VMD_3D_FIELDALTERNATIVE:
                        index = VIC_3D_BASE + 12;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
            break;

            case VIC_FOR_1080p_50Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 13;
                        break;
                    case VMD_3D_LINEALTERNATIVE:
                        index = VIC_3D_BASE + 14;
                        break;
                    case SIDE_BY_SIDE_FULL:
                        index = VIC_3D_BASE + 15;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
                break;

            case VIC_FOR_1080p_24Hz:
                switch(siHdmiTx.ThreeDStructure)
                {
                    case FRAME_PACKING:
                        index = VIC_3D_BASE + 16;
                        break;
                    default:
                        index = DEFAULT_VIDEO_MODE;
                        break;
                }
                break;

            default:
                index = DEFAULT_VIDEO_MODE;
                break;
        }
    }
    else if (siHdmiTx.HDMIVideoFormat == VMD_HDMIFORMAT_PC)
    {
        if (siHdmiTx.VIC < PC_SIZE)
        {
            index = siHdmiTx.VIC + PC_BASE;
        }
        else
        {
            index = DEFAULT_VIDEO_MODE;
        }
    }
    else
    {
        index = DEFAULT_VIDEO_MODE;
    }
    
        return index;
}

UINT8 TPI_REG0x63_SAVED = 0;

STATIC VOID EnableEmbeddedSync (VOID)
{
    /* set 0x60[7] = 0 for DE mode */
    Sil9022a_ModifyClear( SIL9022A_SLAVE_ADDRESS,TPI_SYNC_GEN_CTRL, MSBIT);
    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_DE_CTRL, 0x30);
    Sil9022a_ModifySet( SIL9022A_SLAVE_ADDRESS,TPI_SYNC_GEN_CTRL, MSBIT);// set 0x60[7] = 1 for Embedded Sync      
    Sil9022a_ModifySet( SIL9022A_SLAVE_ADDRESS,TPI_DE_CTRL, BIT_6);
}


/**
 * \brief  Set the 9022/4 format
 */
STATIC VOID SetFormat ( UINT8 *Data)
{
    /* Set HDMI mode to allow color space conversion */
    Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_SYSTEM_CONTROL_DATA_REG,OUTPUT_MODE_MASK, OUTPUT_MODE_HDMI);
    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_INPUT_FORMAT_REG, *Data);
    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_INPUT_FORMAT_REG+1, *(Data+1));
    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_END_RIGHT_BAR_MSB, 0x00);

    if (!g_edid.HDMI_Sink) 
    {
        Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_SYSTEM_CONTROL_DATA_REG, OUTPUT_MODE_MASK, OUTPUT_MODE_DVI);
    }

    if (siHdmiTx.SyncMode == EMBEDDED_SYNC)
        EnableEmbeddedSync();                    /* Last byte of TPI AVI InfoFrame resets Embedded Sync Extraction */
}


/**
 * \brief  Set the 9022/4 to the video mode determined by GetVideoMode()
 */
STATIC UINT8 InitVideo ( UINT8 TclkSel)

{
    UINT8 ModeTblIndex;

#ifdef DEEP_COLOR
    UINT8 temp;
#endif
    UINT8 B_Data[8];
    UINT8 Pattern;
    UINT32 i;
    ModeTblIndex = (UINT8)ConvertVIC_To_VM_Index();

    Pattern = (TclkSel << 6) & TWO_MSBITS;                            // Use TPI 0x08[7:6] for 9022A/24A video clock multiplier
    //TClkSel1:Ratio of output TMDS clock to input video clock,00-x0.5,01- x1 (default),10 -x2,11-x4
    Sil9022a_ModifySet( SIL9022A_SLAVE_ADDRESS, TPI_PIX_REPETITION, Pattern);
    
    // Take values from VModesTable[]:
    if( (siHdmiTx.VIC == 6) || (siHdmiTx.VIC == 7) ||    //480i
         (siHdmiTx.VIC == 21) || (siHdmiTx.VIC == 22) )    //576i
    {
        if( siHdmiTx.ColorSpace == YCBCR422_8BITS)    //27Mhz pixel clock
        {
            B_Data[0] = VModesTable[ModeTblIndex].PixClk & 0x00FF;
            B_Data[1] = (VModesTable[ModeTblIndex].PixClk >> 8) & 0xFF;
        }
        else                                            //13.5Mhz pixel clock
        {
            B_Data[0] = (VModesTable[ModeTblIndex].PixClk /2) & 0x00FF;
            B_Data[1] = ((VModesTable[ModeTblIndex].PixClk /2) >> 8) & 0xFF;
        }
    
    }
    else
    {
        B_Data[0] = VModesTable[ModeTblIndex].PixClk & 0x00FF;            // write Pixel clock to TPI registers 0x00, 0x01
        B_Data[1] = (VModesTable[ModeTblIndex].PixClk >> 8) & 0xFF;
    }
    
    B_Data[2] = VModesTable[ModeTblIndex].Tag.VFreq & 0x00FF;        // write Vertical Frequency to TPI registers 0x02, 0x03
    B_Data[3] = (VModesTable[ModeTblIndex].Tag.VFreq >> 8) & 0xFF;

    if( (siHdmiTx.VIC == 6) || (siHdmiTx.VIC == 7) ||    //480i
         (siHdmiTx.VIC == 21) || (siHdmiTx.VIC == 22) )    //576i
    {
        B_Data[4] = (VModesTable[ModeTblIndex].Tag.Total.Pixels /2) & 0x00FF;    // write total number of pixels to TPI registers 0x04, 0x05
        B_Data[5] = ((VModesTable[ModeTblIndex].Tag.Total.Pixels /2) >> 8) & 0xFF;
    }
    else
    {
        B_Data[4] = VModesTable[ModeTblIndex].Tag.Total.Pixels & 0x00FF;    // write total number of pixels to TPI registers 0x04, 0x05
        B_Data[5] = (VModesTable[ModeTblIndex].Tag.Total.Pixels >> 8) & 0xFF;
    }
    
    B_Data[6] = VModesTable[ModeTblIndex].Tag.Total.Lines & 0x00FF;    // write total number of lines to TPI registers 0x06, 0x07
    B_Data[7] = (VModesTable[ModeTblIndex].Tag.Total.Lines >> 8) & 0xFF;

    // Write TPI Mode data.//0x00-0x07 :Video Mode Defines the incoming resolution
    for(i=0;i<8;i++)
    {
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_PIX_CLK_LSB+i, B_Data[i]); 
    }                    

    // TPI Input Bus and Pixel Repetition Data
    // B_Data[0] = Reg0x08;
    B_Data[0] = 0;  // Set to default 0 for use again
    //B_Data[0] = (VModesTable[ModeTblIndex].PixRep) & LOW_BYTE;        // Set pixel replication field of 0x08
    B_Data[0] |= BIT_BUS_24;                                        // Set 24 bit bus:Input Bus Select. The input data bus can be either one pixel wide or 1/2  pixel wide. The bit defaults to 1 to select full pixel mode. In  1/2  pixel mode, the full pixel is brought in on two successive clock edges (one rising, one falling). 
                                                                //All parts support 24-bit full-pixel and 12-bit half-pixel input modes.
    B_Data[0] |= (TclkSel << 6) & TWO_MSBITS;

#ifdef CLOCK_EDGE_FALLING
    B_Data[0] &= ~BIT_EDGE_RISE;                                    // Set to falling edge
#endif
#ifdef CLOCK_EDGE_RISING
    B_Data[0] |= BIT_EDGE_RISE;                                    // Set to rising edge
#endif
    tpivmode[0] = B_Data[0]; // saved TPI Reg0x08 value.
    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_PIX_REPETITION, B_Data[0]);// 0x08                    

    // TPI AVI Input and Output Format Data
    // B_Data[0] = Reg0x09;
    // B_Data[1] = Reg0x0A;    
    B_Data[0] = 0;  // Set to default 0 for use again
    B_Data[1] = 0;  // Set to default 0 for use again
    
    if (siHdmiTx.SyncMode == EMBEDDED_SYNC)
    {
        //EMB_Status = SetEmbeddedSync();
        EnableEmbeddedSync();    //enablle EmbeddedSync
    }

    if (siHdmiTx.SyncMode == INTERNAL_DE)
    {
        Sil9022a_ModifyClear( SIL9022A_SLAVE_ADDRESS,TPI_SYNC_GEN_CTRL, MSBIT);    // set 0x60[7] = 0 for External Sync
        //DE_Status = SetDE();                                // Call SetDE() with Video Mode as a parameter
    }

    if (siHdmiTx.ColorSpace == RGB)
        B_Data[0] = (((BITS_IN_RGB | BITS_IN_AUTO_RANGE) & ~BIT_EN_DITHER_10_8) & ~BIT_EXTENDED_MODE); // reg0x09
    
    else if (siHdmiTx.ColorSpace == YCBCR444)
        B_Data[0] = (((BITS_IN_YCBCR444 | BITS_IN_AUTO_RANGE) & ~BIT_EN_DITHER_10_8) & ~BIT_EXTENDED_MODE); // 0x09
        
    else if ((siHdmiTx.ColorSpace == YCBCR422_16BITS) ||(siHdmiTx.ColorSpace == YCBCR422_8BITS))
        B_Data[0] = (((BITS_IN_YCBCR422 | BITS_IN_AUTO_RANGE) & ~BIT_EN_DITHER_10_8) & ~BIT_EXTENDED_MODE); // 0x09
    
#ifdef DEEP_COLOR
    switch (siHdmiTx.ColorDepth)
    {
        case 0:  temp = 0x00; Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_DEEP_COLOR_GCP, BIT_2, 0x00); break;
        case 1:  temp = 0x80; Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_DEEP_COLOR_GCP, BIT_2, BIT_2); break;
        case 2:  temp = 0xC0; Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_DEEP_COLOR_GCP, BIT_2, BIT_2); break;
        case 3:  temp = 0x40; Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_DEEP_COLOR_GCP, BIT_2, BIT_2); break;
        default: temp = 0x00; Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_DEEP_COLOR_GCP, BIT_2, 0x00); break;
        //General Control Packet C Deep color settings require the General Control Packet to be sent once per video field
        //with the correct PP and CD information. This must be enabled by software via TPI Deep Color Packet Enable
        //Register 0x40[2] = 1, enable transmission of the GCP packet.
    }
    B_Data[0] = ((B_Data[0] & 0x3F) | temp);// reg0x09
#endif

    B_Data[1] = (BITS_OUT_RGB | BITS_OUT_AUTO_RANGE);  //Reg0x0A

    if ((siHdmiTx.VIC == 6) || (siHdmiTx.VIC == 7) ||    //480i
         (siHdmiTx.VIC == 21) || (siHdmiTx.VIC == 22) ||//576i
         (siHdmiTx.VIC == 2) || (siHdmiTx.VIC == 3) ||    //480p
         (siHdmiTx.VIC == 17) ||(siHdmiTx.VIC == 18))    //576p
    {
        B_Data[1] &= ~BIT_BT_709;
    }
    else
    {
        B_Data[1] |= BIT_BT_709;
    }
    
#ifdef DEEP_COLOR
    B_Data[1] = ((B_Data[1] & 0x3F) | temp);
#endif

#ifdef DEV_SUPPORT_EDID
    if (g_edid.HDMI_Sink) 
    {
        B_Data[1] = ((B_Data[1] & 0xFC) | BITS_OUT_RGB);
    }
    else 
    {
        // Set YCbCr color space depending on EDID
        if (g_edid.YCbCr_4_4_4) 
        {
            B_Data[1] = ((B_Data[1] & 0xFC) | BITS_OUT_YCBCR444);
        }
        else 
        {
            if (g_edid.YCbCr_4_2_2)
            {
                B_Data[1] = ((B_Data[1] & 0xFC) | BITS_OUT_YCBCR422);
            }
            else
            {
                B_Data[1] = ((B_Data[1] & 0xFC) | BITS_OUT_RGB);
            }
        }
    }
#endif

    tpivmode[1] = B_Data[0];    // saved TPI Reg0x09 value.
    tpivmode[2] = B_Data[1];  // saved TPI Reg0x0A value.
    SetFormat( B_Data);

    Sil9022a_ModifyClear( SIL9022A_SLAVE_ADDRESS,TPI_SYNC_GEN_CTRL, BIT_2);    // Number HSync pulses from VSync active edge to Video Data Period should be 20 (VS_TO_VIDEO)

    return TRUE;
}

/**
 * \brief Set TX to D0 mode.
 */
STATIC VOID siHdmiTx_PowerStateD0 (VOID)
{
    Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_DEVICE_POWER_STATE_CTRL_REG, TX_POWER_STATE_MASK, TX_POWER_STATE_D0);
    //kprintf("TX Power State D0\n\r");
    g_sys.txPowerState = TX_POWER_STATE_D0;
}


/**
 * \brief Load AVI InfoFrame data into registers and send to sink.
 */
STATIC UINT8 SetAVI_InfoFrames (VOID)
{
    UINT8 B_Data[SIZE_AVI_INFOFRAME];
    UINT8 i;
    UINT8 TmpVal;
    UINT8 VModeTblIndex;
    for (i = 0; i < SIZE_AVI_INFOFRAME; i++)
           B_Data[i] = 0;

#ifdef DEV_SUPPORT_EDID
    if (g_edid.YCbCr_4_4_4)
        TmpVal = 2;
    else if (g_edid.YCbCr_4_2_2)
        TmpVal = 1;
    else
        TmpVal = 0;
#else
    TmpVal = 0;
#endif

    //fixed RGB
    TmpVal = 0;

    // AVI Byte1: Y1Y0 (output format)
    B_Data[1] = (TmpVal << 5) & BITS_OUT_FORMAT;    
    // A0 = 1; Active format identification data is present in the AVI InfoFrame. // S1:S0 = 01; Overscanned (television).
    B_Data[1] |= 0x11;  
    // Extended colorimetry - xvYCC
    if (siHdmiTx.ColorSpace == XVYCC444)                
    {
        // Extended colorimetry info (B_Data[3] valid (CEA-861D, Table 11)
        B_Data[2] = 0xC0;                                          
        if (siHdmiTx.Colorimetry == COLORIMETRY_601)           // xvYCC601
            B_Data[3] &= ~BITS_6_5_4;
        else if (siHdmiTx.Colorimetry == COLORIMETRY_709)    // xvYCC709
            B_Data[3] = (B_Data[3] & ~BITS_6_5_4) | BIT_4;
    }
    // BT.709
    else if (siHdmiTx.Colorimetry == COLORIMETRY_709)        
        B_Data[2] = 0x80;        // AVI Byte2: C1C0
    // BT.601
    else if (siHdmiTx.Colorimetry == COLORIMETRY_601)        
        B_Data[2] = 0x40;        // AVI Byte2: C1C0
    // Carries no data
    else                                                    
    {                                                    // AVI Byte2: C1C0
        B_Data[2] &= ~BITS_7_6;                            // colorimetry = 0
        B_Data[3] &= ~BITS_6_5_4;                        // Extended colorimetry = 0
    }

    VModeTblIndex = ConvertVIC_To_VM_Index();

    B_Data[4] = siHdmiTx.VIC;

    //  Set the Aspect Ration info into the Infoframe Byte 2
    if (siHdmiTx.AspectRatio == VMD_ASPECT_RATIO_16x9)
    {
        B_Data[2] |= _16_To_9;                          // AVI Byte2: M1M0
        // If the Video Mode table says this mode can be 4x3 OR 16x9, and we are pointing to the
        // table entry that is 4x3, then we bump to the next Video Table entry which will be for 16x9.
        if ((VModesTable[VModeTblIndex].AspectRatio == R_4or16) && (AspectRatioTable[siHdmiTx.VIC - 1] == R_4))
        {
            siHdmiTx.VIC++;
            B_Data[4]++;
        }
    }
    else
    {
        B_Data[2] |= _4_To_3;                       // AVI Byte4: VIC
    }

    B_Data[2] |= SAME_AS_AR;                        // AVI Byte2: R3..R1 - Set to "Same as Picture Aspect Ratio"
    B_Data[5] = VModesTable[VModeTblIndex].PixRep;      // AVI Byte5: Pixel Replication - PR3..PR0

    // Calculate AVI InfoFrame ChecKsum
    B_Data[0] = 0x82 + 0x02 +0x0D;
    for (i = 1; i < SIZE_AVI_INFOFRAME; i++)
    {
        B_Data[0] += B_Data[i];
    }
    B_Data[0] = 0x100 - B_Data[0];

    // Write the Inforframe data to the TPI Infoframe registers
    for(i=0;i<SIZE_AVI_INFOFRAME;i++)
    {
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, TPI_AVI_BYTE_0+i, B_Data[i]); 
    }    

    if (siHdmiTx.SyncMode == EMBEDDED_SYNC)
        EnableEmbeddedSync();

        return TRUE;
}


/**
 * \brief Mute audio
 */
VOID SetAudioMute ( UINT8 audioMute)
{
    Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_INTERFACE_REG, AUDIO_MUTE_MASK, audioMute);
}

/**
 * \brief Load Audio InfoFrame data into registers and send to sink
 */
STATIC UINT8 SetAudioInfoFrames(
                                  UINT8 ChannelCount,
                                  UINT8 CodingType, 
                                  UINT8 SS, 
                                  UINT8 Fs, 
                                  UINT8 SpeakerConfig)
{
    UINT8 B_Data[SIZE_AUDIO_INFOFRAME];  
    UINT8 i;

    for (i = 0; i < SIZE_AUDIO_INFOFRAME; i++)
    B_Data[i] = 0;

    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,MISC_INFO_FRAMES_CTRL, DISABLE_AUDIO);         //  Disbale MPEG/Vendor Specific InfoFrames

    B_Data[0] = TYPE_AUDIO_INFOFRAMES;      // 0x84
    B_Data[1] = AUDIO_INFOFRAMES_VERSION;   // 0x01
    B_Data[2] = AUDIO_INFOFRAMES_LENGTH;    // 0x0A
    B_Data[3] = TYPE_AUDIO_INFOFRAMES+      // Calculate checksum - 0x84 + 0x01 + 0x0A
    AUDIO_INFOFRAMES_VERSION+ 
    AUDIO_INFOFRAMES_LENGTH;

    B_Data[4] = ChannelCount;               // 0 for "Refer to Stream Header" or for 2 Channels. 0x07 for 8 Channels
    B_Data[4] |= (CodingType << 4);         // 0xC7[7:4] == 0b1001 for DSD Audio

    B_Data[5] = ((Fs & THREE_LSBITS) << 2) | (SS & TWO_LSBITS);

    B_Data[7] = SpeakerConfig;

    for (i = 4; i < SIZE_AUDIO_INFOFRAME; i++)
    B_Data[3] += B_Data[i];

    B_Data[3] = 0x100 - B_Data[3];

    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,MISC_INFO_FRAMES_CTRL, EN_AND_RPT_AUDIO);         // Re-enable Audio InfoFrame transmission and repeat

    for(i=0;i<SIZE_AUDIO_INFOFRAME;i++)
    {
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS, MISC_INFO_FRAMES_TYPE+i, B_Data[i]); 
    }
    
    if (siHdmiTx.SyncMode == EMBEDDED_SYNC)
    EnableEmbeddedSync();

    return TRUE;
}

/**
 * \brief Set the 9022/4 audio interface to basic audio.
 */
STATIC UINT8 siHdmiTx_AudioSet (VOID)
{
    SetAudioMute( AUDIO_MUTE_MUTED);    // mute output

    if (siHdmiTx.AudioMode == AMODE_I2S)      // I2S input
    {
            // 2. Select I2S input mode using register bits 0x26[7:6], with Mute enabled (bit [4] = 1).
            Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_INTERFACE_REG, AUDIO_SEL_MASK, AUD_IF_I2S);    // 0x26 = 0x80
            // Word Length = 24
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_HANDLING, 0x08 | AUD_DO_NOT_CHECK);      // 0x25
    }
    else    // SPDIF input
    {
            Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_INTERFACE_REG, AUDIO_SEL_MASK, AUD_IF_SPDIF);    // 0x26 = 0x40
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_HANDLING, AUD_PASS_BASIC);                   // 0x25 = 0x00
    }

#ifndef F_9022A_9334
    if (siHdmiTx.AudioChannels == ACHANNEL_2CH)
           SetChannelLayout(TWO_CHANNELS);             // Always 2 channesl in S/PDIF
       else
           SetChannelLayout(EIGHT_CHANNELS);
#else
    if (siHdmiTx.AudioChannels == ACHANNEL_2CH)
           Sil9022a_ModifyClear( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_INTERFACE_REG, BIT_5); // Use TPI 0x26[5] for 9022A/24A and 9334 channel layout
       else
           Sil9022a_ModifySet( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_INTERFACE_REG, BIT_5); // Use TPI 0x26[5] for 9022A/24A and 9334 channel layout
#endif

    if (siHdmiTx.AudioMode == AMODE_I2S)      // I2S input
    {
           // I2S - Map channels - replace with call to API MAPI2S
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_EN, 0x80); // 0x1F

        if (siHdmiTx.AudioChannels > ACHANNEL_2CH)
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_EN, 0x91);
        
        if (siHdmiTx.AudioChannels > ACHANNEL_4CH)
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_EN, 0xA2);
        
        if (siHdmiTx.AudioChannels > ACHANNEL_6CH)
            Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_EN, 0xB3);

           // I2S - Stream Header Settings - replace with call to API SetI2S_StreamHeader
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_CHST_0, 0x00); // 0x21
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_CHST_1, 0x00);
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_CHST_2, 0x00);
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_CHST_3, siHdmiTx.AudioFs);
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_CHST_4, (siHdmiTx.AudioFs << 4) |siHdmiTx.AudioWordLength);
        
        // Oscar 20100929 added for 16bit auido noise issue
        Sil9022a_WriteIndexedRegister(SIL9022A_SLAVE_ADDRESS,INDEXED_PAGE_1, AUDIO_INPUT_LENGTH, siHdmiTx.AudioWordLength);

           // I2S - Input Configuration
           Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_I2S_IN_CFG, siHdmiTx.AudioI2SFormat); //TPI_Reg0x20
    }

    Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_AUDIO_SAMPLE_CTRL, 0x58);//REFER_TO_STREAM_HDR
    // 7. BF-CD
    SetAudioInfoFrames( siHdmiTx.AudioChannels & THREE_LSBITS, REFER_TO_STREAM_HDR, REFER_TO_STREAM_HDR, REFER_TO_STREAM_HDR, 0x00); //REFER_TO_STREAM_HDR, REFER_TO_STREAM_HDR, REFER_TO_STREAM_HDR    
    // 9.
    SetAudioMute( AUDIO_MUTE_NORMAL);  // unmute output

    return AUDIO_MODE_SET_OK;
}


/**
 * \brief Set the 9022/4 video and video.
 */
STATIC VOID siHdmiTx_Init (VOID) 
{
    //kprintf(">>siHdmiTx_Init()\n\r");
    // workaround for Bug#18128
    if (siHdmiTx.ColorDepth == VMD_COLOR_DEPTH_8BIT)
    {
        // Yes it is, so force 16bpps first!
        siHdmiTx.ColorDepth = VMD_COLOR_DEPTH_16BIT;
        InitVideo( siHdmiTx.TclkSel);
        // Now put it back to 8bit and go do the expected InitVideo() call
        siHdmiTx.ColorDepth = VMD_COLOR_DEPTH_8BIT;
    }
    // end workaround

    InitVideo( siHdmiTx.TclkSel);            // Set PLL Multiplier to x1 upon power up

    siHdmiTx_PowerStateD0();

    if (g_edid.HDMI_Sink)                    // Set InfoFrames only if HDMI output mode
    {
        SetAVI_InfoFrames();
        siHdmiTx_AudioSet();            // set audio interface to basic audio (an external command is needed to set to any other mode
    }
    else
    {
        SetAudioMute( AUDIO_MUTE_MUTED);
    }

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!

// THIS PATCH IS NEEDED BECAUSE SETTING UP AVI InfoFrames CLEARS 0x63 and 0x60[5]
    if (siHdmiTx.ColorSpace == YCBCR422_8BITS)
        Sil9022a_ModifySet( SIL9022A_SLAVE_ADDRESS,TPI_SYNC_GEN_CTRL, BIT_5);       // Set 0x60[5] according to input color space.
                                               
// THIS PATCH IS NEEDED BECAUSE SETTING UP AVI InfoFrames CLEARS 0x63
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_DE_CTRL, TPI_REG0x63_SAVED);

// PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!PATCH!!!
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        //==========================================================
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_YC_Input_Mode, 0x00);

#if 0
    if ((g_hdcp.HDCP_TxSupports == TRUE) && (g_hdcp.HDCPAuthenticated == VMD_HDCP_AUTHENTICATED)&&(Sii9024A_HDCP_supported))
    {
        if (g_hdcp.HDCP_AksvValid == TRUE)
        {
            // AV MUTE
            TPI_DEBUG_PRINT (("TMDS -> Enabled (Video Muted)\n"));
            ReadModifyWriteTPI(TPI_SYSTEM_CONTROL_DATA_REG, LINK_INTEGRITY_MODE_MASK | TMDS_OUTPUT_CONTROL_MASK | AV_MUTE_MASK,
                LINK_INTEGRITY_DYNAMIC | TMDS_OUTPUT_CONTROL_ACTIVE | AV_MUTE_MUTED);
            
            WriteByteTPI(TPI_PIX_REPETITION, tpivmode[0]);              // Write register 0x08
            g_sys.tmdsPoweredUp = TRUE;
            EnableInterrupts(HOT_PLUG_EVENT | RX_SENSE_EVENT | AUDIO_ERROR_EVENT | SECURITY_CHANGE_EVENT | HDCP_CHANGE_EVENT);
        }
    }
    else
#endif
    {
        //kprintf("TMDS -> Enabled\n\r");
        Sil9022a_Modify( SIL9022A_SLAVE_ADDRESS,TPI_SYSTEM_CONTROL_DATA_REG, LINK_INTEGRITY_MODE_MASK | TMDS_OUTPUT_CONTROL_MASK | AV_MUTE_MASK, 
            LINK_INTEGRITY_DYNAMIC | TMDS_OUTPUT_CONTROL_ACTIVE | AV_MUTE_NORMAL);
            
        Sil9022a_Write( SIL9022A_SLAVE_ADDRESS,TPI_PIX_REPETITION, tpivmode[0]);              // Write register 0x08
        g_sys.tmdsPoweredUp = TRUE;
        Sil9022a_EnableInterrupt( SIL9022A_SLAVE_ADDRESS,HOT_PLUG_EVENT | RX_SENSE_EVENT | AUDIO_ERROR_EVENT);
    }

}

/**
 * \brief Implement Hot Plug Service Loop activities
 */
STATIC VOID HotPlugService (VOID)
{
    //kprintf(">>HotPlugService()\n\r");
    Sil9022a_DisableInterrupt( SIL9022A_SLAVE_ADDRESS,0xFF);
    
    //siHdmiTx.VIC = g_edid.VideoDescriptor[0];    // use 1st mode supported by sink
    
    siHdmiTx_Init();
}


/**
 * \brief DSRX power up handle.
 */
VOID Sil9022a_OnDownstreamRxPoweredUp (VOID)
{
    //kprintf("DSRX -> Powered Up\n\r");
    g_sys.dsRxPoweredUp = TRUE;

    HotPlugService();
}

UINT8 Sil9022a_HDMI_isSignatured(VOID)
{
    return g_edid.HDMI_Sink;
}

#endif
